package se.su.dsv.scipro.activityplan;

import com.mysema.query.annotations.QueryInit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.system.LazyDeletableDomainObject;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Comparator;
import java.util.Date;

@Data
@EqualsAndHashCode(callSuper = false, of = "id")
@Entity
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class Activity extends LazyDeletableDomainObject {
	@Id
	@GeneratedValue
	private Long id;

	@ManyToOne(optional=false)
	@JoinColumn (name="activityTemplate_id")
    @QueryInit("project")
	private ActivityPlan activityPlan;

	@Column(nullable=false)
	private Date date;

	@Column(nullable=false)
	private String title;

	private String description;

	@OneToOne(optional = true, cascade = CascadeType.ALL)
	private FileDescription fileUpload;

	@ManyToOne
	private Checklist checklist;

    private boolean editable = true;

    @Enumerated(EnumType.STRING)
    private Action action = Action.NONE;
	
	public Activity() {
		this.title = "";
		this.description = "";
	}

	@Override
	public String toString(){
		return "Event: "+ getTitle()+"@"+getDate();
	}

    public static class ByDateComparator implements Comparator<Activity>, Serializable {
        @Override
        public int compare(Activity o1, Activity o2) {
            return o1.getDate().compareTo(o2.getDate());
        }
    }

    public static class Builder implements IActivityPlan, IDate, IName, IDescription, IBuild {
        private ActivityPlan activityPlan;
        private Date date;
        private String name;
        private String description;
        private boolean editable = true;

        @Override
        public IDate activityPlan(ActivityPlan activityPlan) {
            this.activityPlan = activityPlan;
            return this;
        }

        @Override
        public IName date(Date date) {
            this.date = date;
            return this;
        }

        @Override
        public IBuild description(String description) {
            this.description = description;
            return this;
        }

        @Override
        public IDescription name(String name) {
            this.name = name;
            return this;
        }

        @Override
        public IBuild editable(boolean editable) {
            this.editable = editable;
            return this;
        }

        @Override
        public Activity build() {
            Activity pse = new Activity();
            pse.setActivityPlan(activityPlan);
            pse.setDate(date);
            pse.setTitle(name);
            pse.setDescription(description);
            pse.setEditable(editable);
            return pse;
        }
    }

    public static IActivityPlan builder(){
        return new Builder();
    }

    public interface IActivityPlan {
        IDate activityPlan(ActivityPlan activityPlan);
    }

    public interface IDate {
        IName date (Date date);
    }

    public interface IName {
        IDescription name(String name);
    }

    public interface IDescription {
        IBuild description(String description);
    }

    public interface IBuild {
        IBuild editable(boolean editable);
        Activity build();
    }
}
