package se.su.dsv.scipro.activityplan;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.Predicate;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;


public class ActivityPlanTemplateServiceImpl extends AbstractServiceImpl<ActivityPlanTemplate, Long> implements ActivityPlanTemplateService {
    @Inject
    public ActivityPlanTemplateServiceImpl(Provider<EntityManager> em) {
        super(em, ActivityPlanTemplate.class, QActivityPlanTemplate.activityPlanTemplate);
    }

    @Override
    public Iterable<ActivityPlanTemplate> findAll(Filter filter, Pageable pageable) {
        return findAll(toPredicate(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(toPredicate(filter));
    }

    protected static Predicate toPredicate(ActivityPlanTemplateService.Filter filter) {
        BooleanBuilder predicate = new BooleanBuilder();

        if (filter.getFilterString()!=null){
            predicate.and(filterString(filter.getFilterString()));
        }
        if (filter.getCreator()!=null){
            predicate.and(creator(filter.getCreator()));
        }
        return predicate;
    }

    private static Predicate creator(User creator) {
        return QActivityPlanTemplate.activityPlanTemplate.isSysAdminTemplate.isTrue().or(QActivityPlanTemplate.activityPlanTemplate.creator.eq(creator));
    }

    private static Predicate filterString(String filterString) {
        return QActivityPlanTemplate.activityPlanTemplate.title.containsIgnoreCase(filterString)
                .or(QActivityPlanTemplate.activityPlanTemplate.creator.firstName.containsIgnoreCase(filterString))
                .or(QActivityPlanTemplate.activityPlanTemplate.creator.lastName.containsIgnoreCase(filterString));
    }

    @Override
    public List<ActivityPlanTemplate> findAll(Filter filter) {
        return findAll(toPredicate(filter));
    }
}
