package se.su.dsv.scipro.checklist;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "checklist_answer")
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ChecklistAnswer extends DomainObject {
    @Id
    @GeneratedValue
    private Long id;

    @Enumerated(EnumType.STRING)
    @Column(nullable = false)
    private ChecklistAnswerEnum answer;

    @ManyToOne(optional = false)
    private User user;

    @Lob
    @Column
    private String comment;

    protected ChecklistAnswer() {
    }

    public ChecklistAnswer(User user) {
        this(user, null);
    }

    public ChecklistAnswer(User user, ChecklistAnswerEnum answer) {
        this.user = user;
        this.answer = answer != null ? answer : ChecklistAnswerEnum.NO_ANSWER;
    }

    public static IUser builder() {
        return new Builder();
    }

    private static class Builder implements IUser, IBuild {

        private ChecklistAnswer instance = new ChecklistAnswer();
        private User user;
        private ChecklistAnswerEnum answer;
        private String comment;

        @Override
        public IBuild user(User user) {
            this.user = user;
            return this;
        }

        @Override
        public IBuild answer(ChecklistAnswerEnum answer) {
            this.answer = answer;
            return this;
        }

        @Override
        public IBuild comment(String comment) {
            this.comment = comment;
            return this;
        }

        @Override
        public ChecklistAnswer build() {
            instance.setUser(user);
            instance.setAnswer(answer);
            instance.setComment(comment);
            return instance;
        }
    }

    public interface IUser {
        IBuild user(User user);
    }

    public interface IBuild {
        IBuild answer(ChecklistAnswerEnum answer);

        IBuild comment(String comment);

        ChecklistAnswer build();
    }
}