package se.su.dsv.scipro.forum;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.GroupThread;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

class GroupForumServiceImpl implements GroupForumService {
    private final GroupThreadRepository groupThreadRepository;
    private final FileDescriptionService fileDescriptionService;
    private final BasicForumService basicForumService;
    private final EventBus eventBus;

    @Inject
    GroupForumServiceImpl(
            final GroupThreadRepository groupThreadRepository,
            final FileDescriptionService fileDescriptionService,
            final BasicForumService basicForumService,
            final EventBus eventBus) {
        this.groupThreadRepository = groupThreadRepository;
        this.fileDescriptionService = fileDescriptionService;
        this.basicForumService = basicForumService;
        this.eventBus = eventBus;
    }


    @Override
    public GroupThread findOne(long threadId) {
        return groupThreadRepository.findOne(threadId);
    }

    @Transactional
    private ForumPost createThreadReply(GroupThread thread, User user, String content, Set<FileUpload> fileUploads) {
        return createReply(thread, user, content, storeFiles(fileUploads));
    }

    @Override
    @Transactional
    public GroupThread createThread(Group group, User user, String subject, String content, Set<FileUpload> fileUploads) {
        ForumThread forumThread = basicForumService.createThread(subject);
        GroupThread groupThread = new GroupThread();
        groupThread.setGroup(group);
        groupThread.setForumThread(forumThread);

        groupThread = groupThreadRepository.save(groupThread);

        createThreadReply(groupThread, user, content, fileUploads);

        return groupThread;
    }

    private Set<FileDescription> storeFiles(final Set<FileUpload> fileUploads) {
        Set<FileDescription> storedFiles = new HashSet<>();
        for (FileUpload fileUpload : fileUploads){
            storedFiles.add(fileDescriptionService.storeFile(fileUpload));
        }
        return storedFiles;
    }

    @Override
    public void markRead(final User user, final GroupThread groupThread) {
        basicForumService.setThreadRead(user, groupThread.getForumThread(), true);
    }

    @Override
    public List<GroupThread> getThreads(Group group) {
        return groupThreadRepository.findByGroup(group);
    }

    @Override
    public ForumPost createReply(final GroupThread groupThread, final User poster, final String content, final Set<FileDescription> attachments) {
        ForumPost reply = basicForumService.createReply(groupThread.getForumThread(), poster, content, attachments);
        eventBus.post(new NewGroupForumReplyEvent(groupThread, reply));
        return reply;
    }

    @Override
    public List<ForumPost> getPosts(final GroupThread groupThread) {
        return basicForumService.getPosts(groupThread.getForumThread());
    }
}
