package se.su.dsv.scipro.group;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.FilteredService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

public class GroupServiceImpl extends AbstractServiceImpl<Group, Long> implements GroupService, FilteredService<Group, Long, GroupService.Filter> {

    @Inject
    public GroupServiceImpl(Provider<EntityManager> em) {
        super(em, Group.class, QGroup.group);
    }


    private BooleanBuilder fromParams(GroupService.Filter params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getAuthor() != null){
            bb.and(hasAuthor(params.getAuthor()));
        }
        if (params.getCreator() != null){
            bb.and(isCreator(params.getCreator()));
        }
        if (params.getNonAuthorInvolvee() != null){
            bb.and(hasNonAuthorInvolvee(params.getNonAuthorInvolvee()));
        }
        if (params.getActive() != null){
            bb.and(isActive(params.getActive()));
        }
        if (params.getProject() != null) {
            bb.and(hasProject(params.getProject()));
        }
        return bb;
    }

    private BooleanExpression hasAuthor(final User user) {
        return QGroup.group.projects.any().projectParticipants.contains(user);
    }

    private BooleanExpression hasProject(Project project) {
        return QGroup.group.projects.contains(project);
    }

    private BooleanExpression isCreator(final User user) {
        return QGroup.group.user.eq(user);
    }

    private BooleanExpression isActive(final boolean active) {
        return QGroup.group.active.eq(active);
    }

    private BooleanExpression hasNonAuthorInvolvee(final User user) {
        return QGroup.group.user.eq(user)
                .or(QGroup.group.projects.any().headSupervisor.eq(user))
                .or(QGroup.group.projects.any().coSupervisors.contains(user))
                .or(QGroup.group.projects.any().reviewers.contains(user));
    }

    @Override
    public Iterable<Group> findAll(final Filter filter, final Pageable pageable) {
        return findAll(fromParams(filter), pageable);
    }

    @Override
    public long count(final Filter filter) {
        return count(fromParams(filter));
    }

    @Override
    public List<Group> findAll(final Filter filter) {
        return findAll(fromParams(filter));
    }
}