package se.su.dsv.scipro.integration.activityfinalseminar;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import scala.Option;
import se.su.dsv.scipro.activityplan.Action;
import se.su.dsv.scipro.activityplan.Activity;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.finalseminar.*;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.Date;

@Singleton
class FinalSeminarActivityHandler {

    private final ActivityPlanFacade activityPlanFacade;
    private final ActivityFinalSeminarRepository activityFinalSeminarRepository;

    @Inject
    FinalSeminarActivityHandler(final ActivityPlanFacade activityPlanFacade,
                                final ActivityFinalSeminarRepository activityFinalSeminarRepository,
                                final EventBus eventBus) {
        this.activityPlanFacade = activityPlanFacade;
        this.activityFinalSeminarRepository = activityFinalSeminarRepository;
        eventBus.register(this);
    }

    @Subscribe
    public void finalSeminarMoved(FinalSeminarMovedEvent finalSeminarMovedEvent) {
        synchronizeFinalSeminarActivity(finalSeminarMovedEvent.getFinalSeminar(), finalSeminarMovedEvent.getTo());
    }

    @Subscribe
    public void finalSeminarCreated(FinalSeminarCreatedEvent finalSeminarCreatedEvent) {
        final FinalSeminar finalSeminar = finalSeminarCreatedEvent.getFinalSeminar();
        synchronizeFinalSeminarActivity(finalSeminar, finalSeminar.getStartDate());
    }

    private void synchronizeFinalSeminarActivity(final FinalSeminar finalSeminar, final Date finalSeminarStartDate) {
        final Option<ActivityFinalSeminar> optFinalSeminarActivity = activityFinalSeminarRepository.findByFinalSeminar(finalSeminar);

        if (optFinalSeminarActivity.isDefined()) {
            updateActivity(optFinalSeminarActivity.get().getActivity(), finalSeminar.getStartDate());
        }
        else {
            createNewActivity(finalSeminar, finalSeminarStartDate);
        }
    }

    private void createNewActivity(final FinalSeminar finalSeminar, final Date finalSeminarStartDate) {
        final Activity activity = Activity.builder().
                activityPlan(activityPlanFacade.retrieveActivityPlan(finalSeminar.getProject())).
                date(finalSeminarStartDate).
                name("Final seminar").
                description("Actual final seminar").
                editable(false).
                build();
        activity.setAction(Action.FINAL_SEMINAR);
        final Activity savedActivity = activityPlanFacade.saveActivity(activity);
        final ActivityFinalSeminar finalSeminarActivity = new ActivityFinalSeminar(savedActivity, finalSeminar);
        activityFinalSeminarRepository.save(finalSeminarActivity);
    }

    private void updateActivity(final Activity activity, final Date startDate) {
        activity.setDate(startDate);
        activityPlanFacade.saveActivity(activity);
    }

    @Subscribe
    public void finalSeminarDeleted(FinalSeminarDeletedEvent finalSeminarDeletedEvent) {
        removeActivity(finalSeminarDeletedEvent.getFinalSeminar());
    }

    @Subscribe
    public void finalSeminarCancelled(FinalSeminarCancelledEvent finalSeminarCancelledEvent) {
        removeActivity(finalSeminarCancelledEvent.getFinalSeminar());
    }

    private void removeActivity(final FinalSeminar finalSeminar) {
        final Option<ActivityFinalSeminar> optFinalSeminarActivity = activityFinalSeminarRepository.findByFinalSeminar(finalSeminar);

        if (optFinalSeminarActivity.isDefined()) {
            ActivityFinalSeminar finalSeminarActivity = optFinalSeminarActivity.get();
            activityFinalSeminarRepository.delete(finalSeminarActivity);
            activityPlanFacade.deleteActivity(finalSeminarActivity.getActivity());
        }
    }
}