package se.su.dsv.scipro.integration.activityforum;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import scala.runtime.AbstractFunction0;
import se.su.dsv.scipro.activityplan.Activity;
import se.su.dsv.scipro.activityplan.ActivityFileUploadedEvent;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.Collections;
import java.util.Set;

@Singleton
class PostActivityUploadToForum {
    private final ProjectForumService projectForumService;
    private final ActivityThreadRepository activityThreadRepository;

    @Inject
    PostActivityUploadToForum(final ProjectForumService projectForumService,
                              final ActivityThreadRepository activityThreadRepository,
                              final EventBus eventBus) {
        this.projectForumService = projectForumService;
        this.activityThreadRepository = activityThreadRepository;
        eventBus.register(this);
    }

    @Subscribe
    public void postActivityUploadToForum(ActivityFileUploadedEvent activityFileUploadedEvent) {
        final Activity activity = activityFileUploadedEvent.getActivity();
        final FileDescription fileDescription = activityFileUploadedEvent.getFileDescription();
        final ProjectThread thread = activityThreadRepository.findByActivity(activity).getOrElse(new AbstractFunction0<ProjectThread>() {
            @Override
            public ProjectThread apply() {
                return createNewThread(activity); // WTB lazy arguments
            }
        });
        final User uploader = fileDescription.getUploader();
        final Set<FileDescription> attachments = Collections.singleton(fileDescription);
        projectForumService.createReply(thread, uploader, activity.getDescription(), attachments);
    }

    private ProjectThread createNewThread(final Activity activity) {
        ProjectThread thread = projectForumService.createThread(activity.getActivityPlan().getProject(), activity.getTitle());
        ActivityThread activityThread = new ActivityThread(thread, activity);
        activityThreadRepository.save(activityThread);
        return thread;
    }
}
