package se.su.dsv.scipro.mail;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

@Data
@Entity
@Table(name = "mail_event")
@Cacheable(false)
@EqualsAndHashCode(of = {"id"}, callSuper = false)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class MailEvent extends DomainObject {
    public static final int STRING_MAX_LENGTH = 255;
    @Id
    @GeneratedValue
    private Long id;

    @JoinTable(name = "mail_event_recipients")
    @ManyToMany(fetch = FetchType.EAGER)
    private Set<User> recipients = new HashSet<>();

    @ElementCollection
    private Set<String> nonUserRecipients = new HashSet<>();

    @Column(length = STRING_MAX_LENGTH)
    @Basic(optional = false)
    private String subject;

    @Column(length = STRING_MAX_LENGTH)
    @Basic(optional = true)
    private String fromName;

    @Column(length = STRING_MAX_LENGTH)
    @Basic(optional = true)
    private String fromEmail;

    @Lob
    @Basic(optional = false)
    private String messageBody;

    @Basic(optional = false)
    private boolean sent = false;

    @Basic(optional = true)
    private String messageID;

    @ManyToOne(optional = true)
    private FileDescription attachment;

    public MailEvent() {
    }

    public MailEvent(
            final String subject,
            final String messageBody,
            final User recipient,
            final String fromName,
            final String fromEmail) {
        this.subject = subject;
        this.messageBody = messageBody;
        this.recipients.add(recipient);
        this.fromName = fromName;
        this.fromEmail = fromEmail;
    }

    public MailEvent(
            final String subject,
            final String messageBody,
            final Collection<User> recipients,
            final String fromName,
            final String fromEmail
    ) {
        this.subject = subject;
        this.messageBody = messageBody;
        this.recipients.addAll(recipients);
        this.fromName = fromName;
        this.fromEmail = fromEmail;
    }

    public boolean isSent() {
        return sent;
    }

    public void markSent(final String messageID) {
        this.messageID = messageID;
        this.sent = true;
    }

    public Set<User> getRecipients() {
        return recipients;
    }

    public void setMessageBody(String messageBody) {
        this.messageBody = messageBody;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setFromName(String fromName) {
        this.fromName = fromName;
    }

    public void setFromEmail(String fromEmail) {
        this.fromEmail = fromEmail;
    }

    public void addRecipients(final Iterable<Recipient> recipients) {
        for (Recipient recipient : recipients) {
            recipient.accept(new RecipientVisitor<Boolean>() {
                @Override
                public Boolean visit(final EmailRecipient emailRecipient) {
                    return getNonUserRecipients().add(emailRecipient.getEmail());
                }

                @Override
                public Boolean visit(final UserRecipient userRecipient) {
                    return getRecipients().add(userRecipient.getUser());
                }
            });
        }
    }
}
