package se.su.dsv.scipro.match;

import com.google.inject.persist.Transactional;
import com.mysema.query.types.expr.BooleanExpression;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ResearchArea;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

class KeywordServiceImpl extends AbstractServiceImpl<Keyword,Long> implements KeywordService {

    private static final Logger LOGGER = LoggerFactory.getLogger(KeywordService.class);

    @Inject
	KeywordServiceImpl(Provider<EntityManager> em) {
        super(em, Keyword.class, QKeyword.keyword1);
	}

    private BooleanExpression byArea(ResearchArea area) {
    	return QKeyword.keyword1.researchAreas.contains(area);
    }
    
    private BooleanExpression nameContains(String input) {
    	return QKeyword.keyword1.keyword.containsIgnoreCase(input);
    }

    @Override
	@Transactional
	public void deleteConnection(Keyword keyword, ResearchArea area) {
		Keyword k = findOne(keyword.getId());
		k.removeResearchArea(area);
		LOGGER.info("Removing relationship between Keyword: " + k.getKeyword() + " and Research area: " + area.getTitle());
	}
	
	@Override
	@Transactional
	public void addConnection(Keyword keyword, ResearchArea area) {
		Keyword k = findOne(keyword.getId());
		k.addResearchArea(area);
		LOGGER.info("Relationship added, Keyword: " + k.getKeyword() + ", Research area: " + area.getTitle());
	}
	
	@Override
	public Iterable<Keyword> findByResearchArea(ResearchArea area, Pageable pageable) {
		return findAll(byArea(area), pageable);
	}

    @Override
	public List<Keyword> findByResearchArea(ResearchArea area) {
		return findAll(byArea(area));
	}

	@Override
	public List<Keyword> getAutoCompleteKeywords(String searchString, int limit) {
        return findAll(nameContains(searchString), new PageRequest(0, limit, new Sort(Sort.Direction.ASC, "keyword")));
	}
	
	@Override
	public Long countByResearchArea(ResearchArea area) {
		return count(byArea(area));
	}

}
