package se.su.dsv.scipro.milestones.service.impl;

import com.google.inject.persist.Transactional;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.dataobjects.QMilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;
import java.util.List;

import static com.mysema.query.support.Expressions.allOf;

public class MilestonePhaseTemplateServiceImpl extends AbstractServiceImpl<MilestonePhaseTemplate, Long> implements MilestonePhaseTemplateService {

    @Inject
    public MilestonePhaseTemplateServiceImpl(Provider<EntityManager> em) {
        super(em, MilestonePhaseTemplate.class, QMilestonePhaseTemplate.milestonePhaseTemplate);
    }

    @Override
    public List<MilestonePhaseTemplate> findAll(Sort sort) {
        return sorted(createQuery(), sort).list(QMilestonePhaseTemplate.milestonePhaseTemplate);
    }

    @Override
    public List<MilestonePhaseTemplate> findAllActive(Date projectStartDate) {
        QMilestonePhaseTemplate template = QMilestonePhaseTemplate.milestonePhaseTemplate;
        return from(template)
                .where(allOf(template.deleted.isFalse(), template.dateCreated.lt(projectStartDate)))
                .orderBy(template.sortOrder.asc())
                .list(template);
    }

    @Override
    @Transactional
    public <S extends MilestonePhaseTemplate> S save(S phase) {
        if (phase.getId() == null || phase.getId() == 0) {
            phase.setSortOrder(SciProUtilities.safeLongToInt(count()));
        }
        return super.save(phase);
    }

    @Override
    @Transactional
    public void moveUp(MilestonePhaseTemplate phase) {
        int current = phase.getSortOrder();
        MilestonePhaseTemplate downPhase = findOne(QMilestonePhaseTemplate.milestonePhaseTemplate.sortOrder.eq(current - 1));
        if (downPhase != null) {
            downPhase.setSortOrder(current);
            phase.setSortOrder(current - 1);
            save(phase);
            save(downPhase);
        }
    }

    @Override
    @Transactional
    public void moveDown(MilestonePhaseTemplate phase) {
        int current = phase.getSortOrder();
        MilestonePhaseTemplate upPhase = findOne(QMilestonePhaseTemplate.milestonePhaseTemplate.sortOrder.eq(current + 1));
        if (upPhase != null) {
            upPhase.setSortOrder(current);
            phase.setSortOrder(current + 1);
            save(phase);
            save(upPhase);
        }
    }
}
