package se.su.dsv.scipro.misc;

import org.joda.time.*;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;

import javax.inject.Inject;
import java.util.Date;

public class DaysServiceImpl implements DaysService {

    private static final Duration ONE_DAY = Duration.standardDays(1);

    private final NonWorkDayPeriodService nonWorkDays;

    @Inject
    public DaysServiceImpl(NonWorkDayPeriodService nonWorkDays) {
        this.nonWorkDays = nonWorkDays;
    }

    @Override
    public Date workDaysAfter(Date date, int days) {
        MutableDateTime m = new MutableDateTime(date);
        int daysLocal = days;
        while (daysLocal > 0) {
            m.addDays(1);
            if (isWorkDay(m)) {
                daysLocal--;
            }
        }
        return m.toDate();
    }

    @Override
    public Date workDaysAhead(Date date, int days) {
        MutableDateTime m = new MutableDateTime(date);
        int daysLocal = days;
        while (daysLocal > 0) {
            m.addDays(-1);
            if (isWorkDay(m)) {
                daysLocal--;
            }
        }
        return m.toDate();
    }

    @Override
    public int workDaysBetween(final Date startDate, final Date endDate) {
        return go(new DateTime(startDate), new DateTime(endDate), 0);
    }

    private int go(DateTime now, DateTime end, int acc) {
        final DateTime nextDay = now.plus(ONE_DAY);
        if (nextDay.isAfter(end)) { return acc; }
        else if (isWorkDay(nextDay)) { return go(nextDay, end, acc + 1); }
        else { return go(nextDay, end, acc); }
    }

    private boolean isWorkDay(ReadableDateTime date) {
        return !(isWeekend(date) || isNonWorkDay(date));
    }

    private boolean isWeekend(ReadableDateTime date) {
        return date.getDayOfWeek() == DateTimeConstants.SATURDAY ||
                date.getDayOfWeek() == DateTimeConstants.SUNDAY;
    }

    private boolean isNonWorkDay(ReadableDateTime date) {
        return nonWorkDays.isNonWorkDay(date.toDateTime().toDate());
    }
}
