package se.su.dsv.scipro.notes;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.FilteredService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class NoteServiceImpl extends AbstractServiceImpl<Note, Long> implements NoteService, FilteredService<Note, Long, NoteService.Filter> {

    @Inject
    public NoteServiceImpl(Provider<EntityManager> em) {
        super(em, Note.class, QNote.note);
    }

    private BooleanBuilder fromParams(Filter params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getUser() != null){
            bb.and(hasUser(params.getUser()));
        }
        return bb;
    }

    private BooleanExpression hasUser(final User user) {
        return QNote.note.user.eq(user);
    }

    @Override
    public Iterable<Note> findAll(Filter filter, Pageable pageable) {
        return findAll(fromParams(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(fromParams(filter));
    }

    @Override
    public Note getNote(User user) {
        Note note = findOne(hasUser(user));
        if (note == null){
            note = new Note();
            note.setUser(user);
            return save(note);
        }
        return note;
    }
}