package se.su.dsv.scipro.notifications.settings.service;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.settings.entities.QReceiverConfiguration;
import se.su.dsv.scipro.notifications.settings.entities.ReceiverConfiguration;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

import static com.mysema.query.support.Expressions.allOf;

public class ReceiverConfigurationServiceImpl extends AbstractServiceImpl<ReceiverConfiguration, Long> implements ReceiverConfigurationService {

    @Inject
    public ReceiverConfigurationServiceImpl(Provider<EntityManager> em) {
        super(em, ReceiverConfiguration.class, QReceiverConfiguration.receiverConfiguration);
    }

    @Override
    @Transactional
    public boolean setReceiving(Notification.Type type, Enum event, Member.Type member, boolean receives) {
        ReceiverConfiguration configuration = getConfigurationEntity(type, event, member);
        configuration.setEnabled(receives);
        configuration = save(configuration);
        return configuration.isEnabled();
    }

    @Override
    public boolean isReceiving(Notification.Type type, Enum event, Member.Type member) {
        ReceiverConfiguration configuration = getConfigurationEntity(type, event, member);
        return configuration.isEnabled();
    }

    private ReceiverConfiguration getConfigurationEntity(Notification.Type type, Enum event, Member.Type member) {
        QReceiverConfiguration qConfiguration = QReceiverConfiguration.receiverConfiguration;
        ReceiverConfiguration configuration = findOne(allOf(qConfiguration.type.eq(type), qConfiguration.event.eq(event.name()), qConfiguration.member.eq(member)));
        if (configuration == null) {
            configuration = new ReceiverConfiguration(type, event, member);
        }
        return configuration;
    }
}
