package se.su.dsv.scipro.peer;

import com.mysema.query.annotations.QueryInit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Data
@Entity
@Table(name = "peer_request")
@Cacheable(true)
@EqualsAndHashCode(of = {"id"}, callSuper = false)
@ToString(exclude = {"peerReviews"})
@SuppressWarnings("PMD.UnusedPrivateField")
public class PeerRequest extends DomainObject {
    @Id
    @GeneratedValue
    private Long id;

    @Lob
    @Basic(optional = true)
    private String comment;

    @ManyToOne(optional = false)
    @QueryInit({"headSupervisor", "projectType"})
    private Project project;

    @ManyToOne(optional = false)
    private User requester;

    @ManyToOne(optional = false, cascade = CascadeType.REMOVE)
    private FileDescription file;

    @ManyToOne(optional = true)
    private ChecklistTemplate checklistTemplate;

    @ManyToOne(optional = true)
    private Language language;

    @OneToMany(mappedBy = "peerRequest")
    private List<PeerReview> peerReviews = new ArrayList<>(1);

    @Enumerated(EnumType.STRING)
    @Column(nullable = false)
    private RequestStatus status = RequestStatus.WAITING;

    public List<Member> getMembers() {
        List<Member> members = project.getMembers();

        Iterator<Member> iterator = members.iterator();
        while (iterator.hasNext()) {
            Member member = iterator.next();
            if (member.getType() == Member.Type.AUTHOR &&
                    member.getUser().equals(requester)) {
                iterator.remove();
            }
        }

        members.add(new Member(requester, Member.Type.PEER_REQUESTER));
        return members;
    }

    public PeerReview getPeerReview() {
        for (PeerReview peerReview : peerReviews) {
            if (!peerReview.getStatus().equals(PeerReview.ReviewStatus.EXPIRED)) {
                return peerReview;
            }
        }
        return null;
    }
}
