package se.su.dsv.scipro.peer;

import se.su.dsv.scipro.system.Lifecycle;
import se.su.dsv.scipro.workerthreads.Scheduler;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.util.concurrent.TimeUnit;

@Singleton
class PeerWorkerSchedules implements Lifecycle {
    private final Scheduler scheduler;
    private final Provider<ExpiredRequestWorker> expiredRequestWorker;
    private final Provider<ExpiredReviewResetWorker> expiredReviewResetWorker;

    @Inject
    PeerWorkerSchedules(Scheduler scheduler, Provider<ExpiredRequestWorker> expiredRequestWorker, Provider<ExpiredReviewResetWorker> expiredReviewResetWorker) {
        this.scheduler = scheduler;
        this.expiredRequestWorker = expiredRequestWorker;
        this.expiredReviewResetWorker = expiredReviewResetWorker;
    }

    @Override
    public void start() {
        scheduler.schedule("Peer request-expiration worker").runBy(expiredRequestWorker).dailyAt(23, 0);
        scheduler.schedule("Peer review-expiration worker").runBy(expiredReviewResetWorker).every(10, TimeUnit.MINUTES);
    }

    @Override
    public void stop() {

    }
}
