package se.su.dsv.scipro.plagiarism;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

@Entity
@Table(name = "turnitincheck")
@Cacheable(true)

public class TurnItInCheck extends DomainObject {

    private static final Logger LOGGER = LoggerFactory.getLogger(TurnItInCheck.class);

    /**
     * DomainObject demands we use an orthodox id like this one
     */
    @Id
    @GeneratedValue
    private Long id;

    @Override
    public Long getId() {
        return id;
    }

    /*
     * Cascading delete, set document to nul will delete the filedescription but
     * not the actual file. Use FinarSeminarUploadController.deleteSeminarReport
     * to delete the document
     */
    @OneToOne(optional = true)
    private FileDescription fileDescription;

    /**
     * Hash of the fileDescriptions name and datecreated string value.
     * It will be used as the name for the upload in turnitin.
     */
    @Lob
    private byte[] hash;

    private String turnItInId;

    /**
     * no args constructor should never be used! or maybe we could use no args constructor and set fileDescription
     * attribute and then let the worker check if there is a hash or not and if none create one?
     */
    public TurnItInCheck() {
    }

    public TurnItInCheck(final FileDescription fileDescription) {
        this.fileDescription = fileDescription;
        String baseString = fileDescription.getName() + fileDescription.getDateCreated().toString();
        byte[] bytesOfMessage;
        try {
            bytesOfMessage = baseString.getBytes(StandardCharsets.UTF_8);
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            hash = md.digest(bytesOfMessage);
        } catch (NoSuchAlgorithmException e) {
            LOGGER.error("Algorithm error", e);
        }
    }

    public FileDescription getFileDescription() {
        return fileDescription;
    }

    public void setFileDescription(FileDescription document) {
        this.fileDescription = document;
    }

    public byte[] getHash() {
        return hash;
    }

    public void setHash(byte[] hash) {
        this.hash = hash.clone();
    }

    public String getTurnItInId() {
        return turnItInId;
    }

    public void setTurnItInId(String turnItInId) {
        this.turnItInId = turnItInId;
    }
}