package se.su.dsv.scipro.projectpartner;

import org.joda.time.DateTime;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;
import java.util.List;

public class ProjectPartnerServiceImpl extends AbstractServiceImpl<ProjectPartner,Long> implements ProjectPartnerService {

    @Inject
    public ProjectPartnerServiceImpl(Provider<EntityManager> em) {
        super(em, ProjectPartner.class, QProjectPartner.projectPartner);
    }

    @Override
    public List<ProjectPartner> getProjectPartnerInSpan(int days, final ProjectType projectType, Pageable pageable) {
        if (days == 0) {
            return findAll(QProjectPartner.projectPartner.projectType.eq(projectType), pageable);
        }
        Date date = new DateTime().minusDays(days).toDate();
        return findAll(QProjectPartner.projectPartner.dateCreated.after(date).and(QProjectPartner.projectPartner.projectType.eq(projectType)), pageable);
    }

    @Override
    public long countProjectPartnerInSpan(int days, final ProjectType projectType) {
        if (days == 0) {
            return count(QProjectPartner.projectPartner.projectType.eq(projectType));
        }
        Date date = new DateTime().minusDays(days).toDate();
        return count(QProjectPartner.projectPartner.dateCreated.after(date).and(QProjectPartner.projectPartner.projectType.eq(projectType)));
    }
}
