package se.su.dsv.scipro.report;

public abstract class AbstractMaster15GradeCalculator implements GradeCalculator {

    public static final int POINTS_REQUIRED_FOR_A = 25;
    public static final int POINTS_REQUIRED_FOR_B = 23;
    public static final int POINTS_REQUIRED_FOR_C = 21;
    public static final int POINTS_REQUIRED_FOR_D = 19;
    public static final int NUMBER_OF_PROJECT_CRITERIA_REQUIRED_FOR_FX = 10;
    public static final int NUMBER_OF_INDIVIDUAL_CRITERIA_REQUIRED_FOR_FX = 2;

    @Override
    public long getPoints(GradingReport gradingReport) {
        return new Master15Calculation(getGradingCriteria(gradingReport)).calculateTotalPoints();
    }

    @Override
    public GradingReport.Grade getGrade(GradingReport gradingReport) {
        return new Master15Calculation(getGradingCriteria(gradingReport)).getGrade();
    }

    public abstract Iterable<? extends GradingCriterion> getGradingCriteria(GradingReport gradingReport);

    private static final class Master15Calculation extends Calculation {

        private Iterable<? extends GradingCriterion> gradingCriteria;

        private Master15Calculation(Iterable<? extends GradingCriterion> gradingCriteria) {
            this.gradingCriteria = gradingCriteria;
        }

        @Override
        protected GradingReport.Grade convertPointsToGrade() {
            int total = calculateTotalPoints();
            if (meetsRequirementsForA(total)) {
                return GradingReport.Grade.A;
            } else if (total > POINTS_REQUIRED_FOR_B) {
                return GradingReport.Grade.B;
            } else if (total > POINTS_REQUIRED_FOR_C) {
                return GradingReport.Grade.C;
            } else if (total > POINTS_REQUIRED_FOR_D) {
                return GradingReport.Grade.D;
            } else {
                return GradingReport.Grade.E;
            }
        }

        private boolean meetsRequirementsForA(int total) {
            return total > POINTS_REQUIRED_FOR_A && hasPointsForDeadlineCriterion();
        }

        private boolean hasPointsForDeadlineCriterion() {
            for (GradingCriterion gradingCriterion : gradingCriteria) {
                if (gradingCriterion.getTitle().equals("Ö4 Deadlines") && gradingCriterion.getPoints() == 1) {
                    return true;
                }
            }
            return false;
        }

        @Override
        protected int getNumberOfProjectCriteriaRequiredToGetFx() {
            return NUMBER_OF_PROJECT_CRITERIA_REQUIRED_FOR_FX;
        }

        @Override
        protected int getNumberOfIndividualCriteriaRequiredToGetFx() {
            return NUMBER_OF_INDIVIDUAL_CRITERIA_REQUIRED_FOR_FX;
        }

        @Override
        protected Iterable<? extends GradingCriterion> getGradingCriteria() {
            return gradingCriteria;
        }
    }
}
