package se.su.dsv.scipro.report;


import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.file.FileDescriptionRepo;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.finalseminar.FinalSeminarOppositionRepo;

import javax.inject.Inject;
import javax.inject.Named;

@Named
public class OppositionReportServiceImpl implements OppositionReportService {

    private OppositionReportRepo oppositionReportRepo;
    private GradingReportTemplateRepo gradingReportTemplateRepo;
    private FileDescriptionRepo fileDescriptionRepo;
    private FinalSeminarOppositionRepo finalSeminarOppositionRepo;

    @Inject
    public OppositionReportServiceImpl(OppositionReportRepo oppositionReportRepo, GradingReportTemplateRepo gradingReportTemplateRepo, FileDescriptionRepo fileDescriptionRepo, FinalSeminarOppositionRepo finalSeminarOppositionRepo) {
        this.oppositionReportRepo = oppositionReportRepo;
        this.gradingReportTemplateRepo = gradingReportTemplateRepo;
        this.fileDescriptionRepo = fileDescriptionRepo;
        this.finalSeminarOppositionRepo = finalSeminarOppositionRepo;
    }

    @Override
    public OppositionReport findOrCreateReport(FinalSeminarOpposition finalSeminarOpposition) {
        OppositionReport oppositionReport = oppositionReportRepo.findByFinalSeminarOpposition(finalSeminarOpposition);
        if (oppositionReport != null) {
            return oppositionReport;
        } else {
            OppositionReport newReport = gradingReportTemplateRepo.findByProjectTypeAndCredits(finalSeminarOpposition.getProjectType(), finalSeminarOpposition.getProject().getCredits())
                    .createOppositionReport(finalSeminarOpposition);
            return oppositionReportRepo.save(newReport);
        }
    }


    @Override
    public void save(OppositionReport oppositionReport) {
        oppositionReportRepo.save(oppositionReport);
    }

    @Override
    @Transactional
    public void deleteOppositionReport(FinalSeminarOpposition finalSeminarOpposition) {
        if (finalSeminarOpposition.getOppositionReport() != null) {
            oppositionReportRepo.delete(finalSeminarOpposition.getOppositionReport());
            finalSeminarOpposition.setOppositionReport(null);
            finalSeminarOppositionRepo.save(finalSeminarOpposition);
        }
    }

    @Override
    @Transactional
    public void deleteOpponentReport(FinalSeminarOpposition finalSeminarOpposition) {
        if (finalSeminarOpposition.getOpponentReport() != null) {
            fileDescriptionRepo.delete(finalSeminarOpposition.getOpponentReport());
            finalSeminarOpposition.setOpponentReport(null);
            finalSeminarOppositionRepo.save(finalSeminarOpposition);
        }
    }
}
