package se.su.dsv.scipro.reviewing;

import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.query.ListSubQuery;
import se.su.dsv.scipro.system.AbstractRepository;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.mysema.query.types.expr.BooleanExpression.allOf;
import static com.mysema.query.types.expr.BooleanExpression.anyOf;

class ReviewerDeadlineFollowupServiceImpl extends AbstractRepository implements ReviewerDeadlineFollowupService {

    @Inject
    ReviewerDeadlineFollowupServiceImpl(final Provider<EntityManager> em) {
        super(em);
    }

    @Override
    public List<Pair<? extends ReviewerApproval, Decision>> getLateReviewerApprovals(ReviewerApproval.Step step, Date startDate, Date endDate, final Date now) {
        List<Decision> decisions = from(QDecision.decision)
                .where(allOf(QDecision.decision.requested.after(startDate),
                        QDecision.decision.requested.before(endDate),
                        QDecision.decision.reviewerApproval.id.in(getSubQueryList(step)),
                        anyOf(
                                allOf(QDecision.decision.deadline.before(now), QDecision.decision.decisionDate.isNull()),
                                QDecision.decision.decisionDate.after(QDecision.decision.deadline))
                        )
                )
                        .list(QDecision.decision);
        return getLateDecisions(decisions);
    }

    private ListSubQuery<Long> getSubQueryList(ReviewerApproval.Step step) {
        if (step.equals(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL))
            return new JPASubQuery().from(QRoughDraftApproval.roughDraftApproval)
                    .list(QRoughDraftApproval.roughDraftApproval.id);
        return new JPASubQuery().from(QFinalSeminarApproval.finalSeminarApproval)
                .list(QFinalSeminarApproval.finalSeminarApproval.id);
    }

    private List<Pair<? extends ReviewerApproval, Decision>> getLateDecisions(final List<Decision> decisions) {
        final List<Pair<? extends ReviewerApproval, Decision>> late = new ArrayList<>();
        for (final Decision decision : decisions) {
            boolean isLate = decision.isLate();
            if (isLate) {
                late.add(new Pair<>(decision.getReviewerApproval(), decision));
            }
        }
        return late;
    }
}
