package se.su.dsv.scipro.system;

import lombok.ToString;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

@ToString
@MappedSuperclass
@SuppressWarnings("PMD.UnusedPrivateField")
public abstract class DomainObject implements Serializable {

    @Basic(optional=false)
    private Date dateCreated = new Date();

    @Basic(optional = false)
    private Date lastModified = new Date();

    @Version
    @Column(nullable = false)
	private int version=0;

    public abstract Long getId();

    @PreUpdate
    @PrePersist
	public void updateTimeStamps() {
	    lastModified = new Date();
	}

    @Override
    public boolean equals(Object obj) {
        if(this == obj) {
            return true;
        }
        if(obj == null) {
            return false;
        }
        if(this.getClass() != obj.getClass()) {
            return false;
        }
        final DomainObject other = (DomainObject) obj;
        final Long thisID = getId();
        final Long thatID = other.getId();
        return (thisID==null?thatID==null:thisID.equals(thatID));
    }

    @Override
    public int hashCode() {
        int result = 17;
        result = 31 * result + (getId()==null?0:getId().hashCode());
        return result;
    }

    public Date getDateCreated() {
        return (Date) dateCreated.clone();
    }

    public Date getLastModified() {
        return (Date) lastModified.clone();
    }

    public void setDateCreated(Date dateCreated) {
        this.dateCreated = (Date) dateCreated.clone();
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = (Date) lastModified.clone();
    }
}
