package se.su.dsv.scipro.system;


import lombok.SneakyThrows;

import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import java.security.GeneralSecurityException;
import java.security.spec.KeySpec;
import java.util.Random;

/**
 * Class for generating password hashes, and for checking password matches.
 *
 * @author Arian Jafari
 */
public final class PasswordHandler {
    private static final int ITERATION_COUNT = 65536;
    private static final int KEY_LENGTH = 128;
    private static final int SALT_LENGTH = 32;
    private static final String ALGORITHM = "PBKDF2WithHmacSHA1";

    private PasswordHandler() {
    }

    public static byte[] generateSalt() {
        Random random = new Random();
        byte[] salt = new byte[SALT_LENGTH];
        random.nextBytes(salt);

        return salt;
    }

    @SneakyThrows(GeneralSecurityException.class)
    public static byte[] generateHash(char[] password, byte[] salt) {
        KeySpec spec = new PBEKeySpec(password, salt, ITERATION_COUNT, KEY_LENGTH);
        SecretKeyFactory skf = SecretKeyFactory.getInstance(ALGORITHM);

        return skf.generateSecret(spec).getEncoded();
    }

    private static boolean isMatch(char[] password, byte[] storedSalt, byte[] storedHash) {
        byte[] generatedHash = PasswordHandler.generateHash(password, storedSalt);
        for (int i = 0; i < generatedHash.length; i++) {
            if (generatedHash[i] != storedHash[i]) {
                return false;
            }
        }
        return true;
    }

    public static boolean isMatch(char[] passwordFromUser, Password password) {
        return isMatch(passwordFromUser, password.getSalt(), password.getHash());
    }
}
