package se.su.dsv.scipro.system;

import lombok.*;

import javax.persistence.*;
import java.util.EnumSet;
import java.util.Set;

@Data
@EqualsAndHashCode(callSuper = true, of = {"id", "name"})
@ToString(exclude = {"projectTypeSettings", "projectModules"})
@Entity
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ProjectType extends LazyDeletableDomainObject {
	public static final DegreeLevel MASTER = DegreeLevel.SECOND_CYCLE;
	public static final DegreeLevel BACHELOR = DegreeLevel.FIRST_CYCLE;
	public static final DegreeLevel UNKNOWN = DegreeLevel.NONE;
    public static final DegreeLevel PHD = DegreeLevel.THIRD_CYCLE;
    public static final int HP_30 = 30;
    public static final int HP_15 = 15;

    public static final String BACHELOR_STRING = "Bachelor";
    public static final String MASTER_STRING = "Master";
    public static final String PHD_STRING = "PhD";

	@Id
	@GeneratedValue
	private Long id;
	
	@Basic(optional=false)
	private String name;

	@OneToOne(mappedBy= "projectType", orphanRemoval = true, cascade = CascadeType.ALL)
	private ProjectTypeSettings projectTypeSettings = new ProjectTypeSettings(this);

    @Enumerated(EnumType.STRING)
    @Column(nullable = false)
    private DegreeLevel degreeLevel = DegreeLevel.NONE;
	
	@Lob
	private String description;

    @ElementCollection
    @Enumerated(EnumType.STRING)
    @JoinTable(name = "project_type_project_modules")
    @Setter(AccessLevel.NONE)
    private Set<ProjectModule> projectModules = EnumSet.allOf(ProjectModule.class);

    @Basic(optional = false)
    private boolean reviewed = true;

	public ProjectType() {

	}

	public ProjectType(DegreeLevel degreeLevel, String name, String description) {
        this.degreeLevel = degreeLevel;
        this.name = name;
        this.description = description;
    }

	public int getMaxOpponents() {
		return projectTypeSettings.getMaxOpponentsOnFinalSeminar();
	}
	
	public int getMaxActiveParticipants() {
		return projectTypeSettings.getMaxFinalSeminarActiveParticipation();
	}

    public int getMinAuthors() {
        return projectTypeSettings.getMinAuthors();
    }

    public int getMaxAuthors() {
        return projectTypeSettings.getMaxAuthors();
    }

    public void addModule(ProjectModule projectModule) {
        this.projectModules.add(projectModule);
    }

    public void removeModule(ProjectModule projectModule) {
        this.projectModules.remove(projectModule);
    }

    public boolean hasModule(ProjectModule projectModule) {
        return this.projectModules.contains(projectModule);
    }
}
