package se.su.dsv.scipro.system;

import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.persistence.*;

@Entity
@Table(name="project_type_settings")
@Cacheable(true)
@EqualsAndHashCode(of = {"id"}, callSuper = false)
@Data
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ProjectTypeSettings extends DomainObject {
	public ProjectTypeSettings(){}
	
	public ProjectTypeSettings(ProjectType projectType){
		this.projectType = projectType;
	}

	@Id
	@GeneratedValue
	private Long id;

	@OneToOne(optional = false)
	private ProjectType projectType;

	@Basic(optional=false)
	private int minAuthors = 1;
	@Basic(optional=false)
	private int maxAuthors = 2;

	@Basic(optional=false)
	private int maxFinalSeminarActiveParticipation;
	@Basic(optional=false)
	private int maxOpponentsOnFinalSeminar;
	@Basic(optional=false)
	private int minFinalSeminarActiveParticipation;
	@Basic(optional=false)
	private int minOpponentsOnFinalSeminar;

	public static final int DEFAULT_NUM_DAYS_BETWEEN_REVIEWS_ON_SAME_PROJECT = 7;
	public static final int DEFAULT_NUM_DAYS_TO_SUBMIT_PEER_REVIEW = 3;
	public static final int DEFAULT_NUM_DAYS_BEFORE_CANCELLED_PEERS = 90;
	
	/*
	 * Defines the time span between reviews on the same project
	 */
	@Basic(optional = false)
	private int numDaysBetweenPeerReviewsOnSameProject = DEFAULT_NUM_DAYS_BETWEEN_REVIEWS_ON_SAME_PROJECT;
	
	/*
	 * Defines the number of days between accepting a review and the deadline for review submission
	 */
	@Basic(optional = false)
	private int numDaysToSubmitPeerReview = DEFAULT_NUM_DAYS_TO_SUBMIT_PEER_REVIEW;

	/*
	 * Defines the number of days between accepting a review and the deadline for review submission
	 */
	@Basic(optional = false)
	private int numDaysBeforePeerGetsCancelled = DEFAULT_NUM_DAYS_BEFORE_CANCELLED_PEERS;
}
