package se.su.dsv.scipro.system;

import com.mysema.query.jpa.JPASubQuery;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.security.auth.roles.Roles;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

import static com.mysema.query.types.ExpressionUtils.allOf;
import static com.mysema.query.types.ExpressionUtils.anyOf;

public class UserRepoImpl extends GenericRepo<User, Long> implements UserRepo {
    private final Set<UserSearchService> userSearchServices;

    @Inject
    public UserRepoImpl(Provider<EntityManager> em, Set<UserSearchService> userSearchServices) {
        super(em, User.class, QUser.user);
        this.userSearchServices = userSearchServices;
    }

    @Override
    public Collection<User> findByNameIncludes(String term, Roles role, Pageable pageable) {
        QUser user = QUser.user;
        final Set<User> results = new HashSet<>();
        results.addAll(findAll(allOf(
                user.deleted.isFalse(),
                user.roles.contains(role),
                anyOf(
                        user.firstName.concat(" ").concat(user.lastName).contains(term),
                        user.emailAddress.contains(term),
                        user.in(new JPASubQuery().from(QUsername.username1)
                                .where(QUsername.username1.username.contains(term))
                                .list(QUsername.username1.user))
                )
        ), pageable));

        long remaining = pageable.getLimit() - results.size();
        for (UserSearchService userSearchService : userSearchServices) {
            final Collection<? extends User> users = remaining > 0
                    ? userSearchService.search(term, role, remaining)
                    : Collections.<User>emptySet();

            results.addAll(users);
            remaining -= users.size();
        }

        return results;
    }

}
