package se.su.dsv.scipro.workerthreads;

import javax.inject.Provider;
import java.util.concurrent.TimeUnit;

class TaskScheduling implements ScheduleBuilder, WorkerBuilder {

    private final SchedulerImpl scheduler;
    private String description;
    private Provider<? extends Worker> runnable;

    public TaskScheduling(final SchedulerImpl scheduler) {
        this.scheduler = scheduler;
    }

    TaskScheduling withDescription(final String description) {
        this.description = description;
        return this;
    }

    @Override
    public ScheduleBuilder runBy(final Provider<? extends Worker> provider) {
        this.runnable = provider;
        return this;
    }

    @Override
    public Job every(long delay, TimeUnit timeUnit) {
        return schedule(Schedule.every(delay, timeUnit));
    }

    @Override
    public Job dailyAt(int hour, int minute) {
        return schedule(Schedule.dailyAt(hour, minute));
    }

    private Job schedule(final Schedule schedule) {
        return scheduler.addTask(description, runnable, schedule);
    }
}
