package se.su.dsv.scipro.workerthreads;

import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.finalseminar.FinalSeminarSettingsService;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.SeminarEvent;
import se.su.dsv.scipro.reusable.SciProUtilities;

import javax.inject.Inject;
import java.util.Date;

public class ThesisUploadReminderWorker extends AbstractWorker {
    @Inject
    FinalSeminarSettingsService service;
    @Inject
    FinalSeminarService finalSeminarService;
    @Inject
    NotificationController notificationController;
    @Inject
    DaysService daysService;

    @Override
    protected void doWork() {
        this.beginTransaction();

        try {

            for (final FinalSeminar fs : finalSeminarService.findByStartDateAfter(new Date())) {

                /**
                 * Take the deadline
                 */
                final int daysAhead = service.getInstance().getDaysAheadToUploadThesis();
                Date uploadDeadline = daysService.workDaysAhead(fs.getStartDate(), daysAhead);

                /**
                 * Take the remind date
                 */
                Date remindDate = daysService.workDaysAhead(uploadDeadline, 1);

                /**
                 * Make a date for 00:00 today
                 */
                Date todayStart = new Date();
                todayStart = SciProUtilities.setTimeOfDayTo0000(todayStart);

                /**
                 * Make a date for 23:59 today
                 */
                Date todayEnd = new Date();
                todayEnd = SciProUtilities.setTimeOfDayTo2359(todayEnd);

                /**
                 * If the remind date is today, then remind
                 */
                if ((fs.getDocument()==null ||
                        fs.getDocument().getId()==null) &&
                        remindDate.after(todayStart) &&
                        remindDate.before(todayEnd)) {
                    notificationController.notifySeminar(fs, SeminarEvent.Event.THESIS_UPLOAD_REMIND, new NotificationSource());
                }
            }
            commitTransaction();
        } catch (Exception e) {
            this.rollbackTransaction();
            this.setSuccessfulWorker(false);
            throw new RuntimeException(e);
        }
    }
}