package se.su.dsv.scipro.workerthreads;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.plagiarism.*;

import javax.inject.Inject;
import javax.inject.Named;

@Named
public class TurnItInCheckWorker extends AbstractWorker {

    private static final Logger LOGGER = LoggerFactory.getLogger(TurnItInCheckWorker.class);

    @Inject
    private TurnItInCheckService turnItInCheckService;
    @Inject
    private PlagiarismControl plagiarismControl;
    @Inject
    private FinalSeminarService finalSeminarService;
    @Inject
    private TurnitinSettingsService turnitinSettingsService;

    public TurnItInCheckWorker() {
    }

    @Override
    protected void doWork() {
        TurnitinSettings settings = turnitinSettingsService.getSettings();
        if (settings.isEnabled()) {
            try {
                for (TurnItInCheck turnItInCheck : turnItInCheckService.findAll()) {
                    if (turnItInCheck.getFileDescription() == null || turnItInCheck.getFileDescription().getName() == null || finalSeminarService.findByFileDescription(turnItInCheck.getFileDescription()) == null) {
                        turnItInCheckService.delete(turnItInCheck.getId());
                    }
                }
                for (TurnItInCheck turnItInCheck : turnItInCheckService.findAll()) {

                    double filesizedouble = turnItInCheck.getFileDescription().getSize();
                    double filesizeinmegabytes = (filesizedouble / 1024) / 1000;
                    double maxfilesizedouble = settings.getMaxFileSize();

                    if (turnItInCheck.getTurnItInId() == null
                            && turnItInCheck.getFileDescription().getName() != null
                            && finalSeminarService.findByFileDescription(turnItInCheck.getFileDescription()) != null
                            && filesizeinmegabytes <= maxfilesizedouble) {
                        submitPaper(turnItInCheck);
                    }
                }
            } catch (Exception e) {
                LOGGER.error("Worker error", e);
                setSuccessfulWorker(false);
            }
        }
    }

    private void submitPaper(TurnItInCheck turnItInCheck) {
        try {
            String returnValue = plagiarismControl.submitPaper(turnItInCheck);
            if (returnValue != null) {
                turnItInCheck.setTurnItInId(returnValue);
                turnItInCheckService.save(turnItInCheck);
            }
        } catch (Exception e) {
            LOGGER.error("Could not submit paper", e);
        }
    }
}
