#Create a table for keeping template of 'grading criterion points', this table should be initialized via DataInitializer when we run the application
CREATE TABLE IF NOT EXISTS `grading_criterion_point_template` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `dateCreated` datetime NOT NULL,
  `lastModified` datetime NOT NULL,
  `version` int(11) NOT NULL,
  `gradingCriterionTemplate_id` bigint(20) NOT NULL,
  `point` INT(11) NOT NULL,
  `description` VARCHAR(600) DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `FK_gradingCriterionTemplate_id` (`gradingCriterionTemplate_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

ALTER TABLE `grading_criterion_point_template`
ADD CONSTRAINT `FK_gradingCriterionTemplate_id` FOREIGN KEY (`gradingCriterionTemplate_id`) REFERENCES `grading_criterion_template` (`id`);

#Since the template can change during the time, we need a table for keeping the template that we use at a time
CREATE TABLE IF NOT EXISTS `GradingCriterionPoint` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `dateCreated` datetime NOT NULL,
  `lastModified` datetime NOT NULL,
  `version` int(11) NOT NULL,
  `GradingCriterion_id` bigint(20) NOT NULL,
  `point` INT(11) NOT NULL,
  `description` longtext DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `FK_GradingCriterion_id` (`GradingCriterion_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

ALTER TABLE `GradingCriterionPoint`
ADD CONSTRAINT `FK_GradingCriterion_id` FOREIGN KEY (`GradingCriterion_id`) REFERENCES `GradingCriterion` (`id`);

#Now we need to fill GradingCroterionPoint via extracting the old data from GradingCriterion
#First of all we add a row with point equal zero and no description for each GradingCriterion, because each criterion has a point equal 0 with no description
INSERT INTO `GradingCriterionPoint` (dateCreated, lastModified, version, GradingCriterion_id, point)
  (SELECT dateCreated, lastModified, version, id, 0 FROM scipro.GradingCriterion);

#Now we need to extract descriptions for every individual point via the general description that we had before
#We start with point 1 and we try to extract the part of description which is belonged to point 1
#and we do the same for the other points (2, 3 & 4) as well
SET @param="1";
INSERT INTO `GradingCriterionPoint` (`dateCreated`, `lastModified`, `version`, `GradingCriterion_id`, `point`, `description`)
  (SELECT
     `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(`description`, (INSTR(`description`,@param) + 8), (INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') ))
   FROM `GradingCriterion`
   Where INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') <> 0
  ) UNION (
    SELECT
      `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(description, (INSTR(description, @param) + 8), 2000)
    FROM `GradingCriterion`
    WHERE INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`, @param) + 8 ,2000), '\n') = 0);


SET @param="2";
INSERT INTO `GradingCriterionPoint` (`dateCreated`, `lastModified`, `version`, `GradingCriterion_id`, `point`, `description`)
  (SELECT
     `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(`description`, (INSTR(`description`,@param) + 8), (INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') ))
   FROM `GradingCriterion`
   Where INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') <> 0
  ) UNION (
    SELECT
      `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(description, (INSTR(description, @param) + 8), 2000)
    FROM `GradingCriterion`
    WHERE INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`, @param) + 8 ,2000), '\n') = 0);


SET @param="3";
INSERT INTO `GradingCriterionPoint` (`dateCreated`, `lastModified`, `version`, `GradingCriterion_id`, `point`, `description`)
  (SELECT
     `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(`description`, (INSTR(`description`,@param) + 8), (INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') ))
   FROM `GradingCriterion`
   Where INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') <> 0
  ) UNION (
    SELECT
      `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(description, (INSTR(description, @param) + 8), 2000)
    FROM `GradingCriterion`
    WHERE INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`, @param) + 8 ,2000), '\n') = 0);


SET @param="4";
INSERT INTO `GradingCriterionPoint` (`dateCreated`, `lastModified`, `version`, `GradingCriterion_id`, `point`, `description`)
  (SELECT
     `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(`description`, (INSTR(`description`,@param) + 8), (INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') ))
   FROM `GradingCriterion`
   Where INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`,@param) + 8 ,2000), '\n') <> 0
  ) UNION (
    SELECT
      `dateCreated`, `lastModified`, `version`, `id`, @param, SUBSTRING(description, (INSTR(description, @param) + 8), 2000)
    FROM `GradingCriterion`
    WHERE INSTR(`description`,@param) <> 0 AND INSTR(SUBSTRING(`description` ,INSTR(`description`, @param) + 8 ,2000), '\n') = 0);

#We do not need the old descriptions because now we have one description for each point separately in the new tables
UPDATE `GradingCriterion` SET `description`=NULL;
UPDATE `grading_criterion_template` SET `description`=NULL;

#we don't need maxPoints and description because we should find the max points, and the descriptions  via GradingCriterionPointTemplate and GradingCriterionPoint
ALTER TABLE `grading_criterion_template`
DROP COLUMN `maxPoints`, DROP COLUMN `description`;

ALTER TABLE `GradingCriterion`
DROP COLUMN `maxPoints`, DROP COLUMN `description`;
