package se.su.dsv.scipro.milestones

import se.su.dsv.scipro.finalseminar._
import se.su.dsv.scipro.finalthesis.FinalThesisApprovedEvent
import se.su.dsv.scipro.grading.GradingCompletedEvent
import se.su.dsv.scipro.peer.{FirstPeerReviewCompletedEvent, SecondPeerReviewCompletedEvent}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.report.{ReviewerGradingReportSubmittedEvent, SupervisorGradingReportSubmittedEvent}
import se.su.dsv.scipro.reviewing.{RoughDraftApprovalApprovedEvent, RoughDraftApprovalRequestedEvent, FinalSeminarApprovalApprovedEvent}
import se.su.dsv.scipro.system.User

import scala.Predef
import scala.collection.immutable.Set

package object service {
  implicit object oppositionApprovedTC extends Authorship[OppositionApprovedEvent] {
    override def name(a: OppositionApprovedEvent): Set[String] = Set("OppositionGradingEvent")

    override def author(a: OppositionApprovedEvent): User = a.getStudent

    override def project(a: OppositionApprovedEvent): Project = a.getProject
  }

  implicit object participationApprovedTC extends Authorship[ParticipationApprovedEvent] {
    override def name(a: ParticipationApprovedEvent): Set[String] = Set("ParticipationGradingEvent")

    override def author(a: ParticipationApprovedEvent): User = a.getStudent

    override def project(a: ParticipationApprovedEvent): Project = a.getProject
  }

  implicit object respondentApprovedTC extends Authorship[RespondentApprovedEvent] {
    override def name(a: RespondentApprovedEvent): Set[String] = Set("RespondentGradingEvent")

    override def author(a: RespondentApprovedEvent): User = a.getStudent

    override def project(a: RespondentApprovedEvent): Project = a.getProject
  }

  implicit object oppositionFailedTC extends Authorship[OppositionFailedEvent] {
    override def name(a: OppositionFailedEvent): Set[String] = Set("OppositionGradingEvent")

    override def author(a: OppositionFailedEvent): User = a.getStudent

    override def project(a: OppositionFailedEvent): Project = a.getProject
  }

  implicit object participationFailedTC extends Authorship[ParticipationFailedEvent] {
    override def name(a: ParticipationFailedEvent): Set[String] = Set("ParticipationGradingEvent")

    override def author(a: ParticipationFailedEvent): User = a.getStudent

    override def project(a: ParticipationFailedEvent): Project = a.getProject
  }

  implicit object respondentFailedTC extends Authorship[RespondentFailedEvent] {
    override def name(a: RespondentFailedEvent): Set[String] = Set("RespondentGradingEvent")

    override def author(a: RespondentFailedEvent): User = a.getStudent

    override def project(a: RespondentFailedEvent): Project = a.getProject
  }

  implicit object firstPeerReviewCompletedTC extends Authorship[FirstPeerReviewCompletedEvent] {
    override def name(a: FirstPeerReviewCompletedEvent): Set[String] = Set("FirstPeerReviewCompleted")

    override def author(a: FirstPeerReviewCompletedEvent): User = a.getStudent

    override def project(a: FirstPeerReviewCompletedEvent): Project = a.getProject
  }

  implicit object secondPeerReviewCompletedTC extends Authorship[SecondPeerReviewCompletedEvent] {
    override def name(a: SecondPeerReviewCompletedEvent): Set[String] = Set("SecondPeerReviewCompleted")

    override def author(a: SecondPeerReviewCompletedEvent): User = a.getStudent

    override def project(a: SecondPeerReviewCompletedEvent): Project = a.getProject
  }

  implicit object finalSeminarThesisUploadedTC extends ProjectAware[FinalSeminarThesisUploadedEvent] {
    override def name(a: FinalSeminarThesisUploadedEvent): Set[String] = Set("FinalSeminarThesisUploaded")

    override def project(a: FinalSeminarThesisUploadedEvent): Project = a.getProject
  }

  implicit object finalSeminarThesisDeletedTC extends ProjectAware[FinalSeminarThesisDeletedEvent] {
    override def project(a: FinalSeminarThesisDeletedEvent): Project = a.getFinalSeminar.getProject

    override def name(a: FinalSeminarThesisDeletedEvent): Set[String] = Set("FinalSeminarThesisUploaded")
  }

  implicit object finalSeminarCreatedTC extends ProjectAware[FinalSeminarCreatedEvent] {
    override def name(a: FinalSeminarCreatedEvent): Set[String] = Set("FinalSeminarCreated")

    override def project(a: FinalSeminarCreatedEvent): Project = a.getProject
  }

  implicit object finalSeminarDeletedTC extends ProjectAware[FinalSeminarDeletedEvent] {
    override def name(a: FinalSeminarDeletedEvent): Set[String] = Set("FinalSeminarCreated", "FinalSeminarThesisUploaded")

    override def project(a: FinalSeminarDeletedEvent): Project = a.getProject
  }

  implicit object reviewerGradingReportSubmittedTC extends ProjectAware[ReviewerGradingReportSubmittedEvent] {
    override def name(a: ReviewerGradingReportSubmittedEvent): Set[String] = Set("ReviewerGradingReportSubmitted")

    override def project(a: ReviewerGradingReportSubmittedEvent): Project = a.getProject
  }

  implicit object supervisorGradingReportSubmittedTC extends Authorship[SupervisorGradingReportSubmittedEvent] {
    override def name(a: SupervisorGradingReportSubmittedEvent): Set[String] = Set("SupervisorGradingReportSubmitted")

    override def project(a: SupervisorGradingReportSubmittedEvent): Project = a.getProject

    override def author(a: SupervisorGradingReportSubmittedEvent): User = a.getStudent
  }

  implicit object thesisApprovalApprovedTC extends ProjectAware[FinalSeminarApprovalApprovedEvent] {
    override def name(a: FinalSeminarApprovalApprovedEvent): Set[String] = Set(a.getName)

    override def project(a: FinalSeminarApprovalApprovedEvent): Project = a.getProject
  }

  implicit object roughDraftApprovalApprovedTC extends ProjectAware[RoughDraftApprovalApprovedEvent] {
    override def name(a: RoughDraftApprovalApprovedEvent): Set[String] = Set(a.getName)

    override def project(a: RoughDraftApprovalApprovedEvent): Project = a.getProject
  }

  implicit object finalThesisApprovedTC extends ProjectAware[FinalThesisApprovedEvent] {
    override def name(a: FinalThesisApprovedEvent): Set[String] = Set("FinalThesisApproved")

    override def project(a: FinalThesisApprovedEvent): Project = a.getProject
  }

  implicit object roughDraftApprovalRequestedTC extends ProjectAware[RoughDraftApprovalRequestedEvent] {
    override def name(a: RoughDraftApprovalRequestedEvent): Set[String] = Set("RoughDraftApprovalRequested")

    override def project(a: RoughDraftApprovalRequestedEvent): Project = a.getProject
  }

  implicit object gradingCompletedTC extends Authorship[GradingCompletedEvent] {
    override def author(a: GradingCompletedEvent): User = a.getStudent

    override def project(a: GradingCompletedEvent): Project = a.getProject

    override def name(a: GradingCompletedEvent): Predef.Set[String] = Set("SupervisorGradingReportSubmitted")
  }
}
