package se.su.dsv.scipro.finalseminar;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Date;

import static org.junit.Assert.assertEquals;

public class FinalSeminarOppositionServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private FinalSeminarOppositionServiceImpl finalSeminarOppositionService;
    private FinalSeminar finalSeminar;
    private ProjectType projectType;

    @Before
    public void setUp() throws Exception {
        projectType = createProjectType();
        finalSeminar = createFinalSeminar(createProject(projectType));
    }

    @Test
    public void delete_opposition() throws Exception {
        FinalSeminarOpposition opposition = createOpposition(finalSeminar, createUser());
        finalSeminarOppositionService.delete(opposition.getId());
        assertEquals(0, finalSeminarOppositionService.count());
    }

    @Test
    public void delete_opposition_with_report() throws Exception {
        FinalSeminarOpposition opposition = createOpposition(finalSeminar, createUser());
        createOppositionReport(opposition);
        finalSeminarOppositionService.delete(opposition.getId());
        assertEquals(0, finalSeminarOppositionService.count());
    }

    private void createOppositionReport(FinalSeminarOpposition opposition) {
        OppositionReport report = new OppositionReport(createGradingReportTemplate(), opposition);
        opposition.setOppositionReport(report);
        save(opposition);
    }

    private GradingReportTemplate createGradingReportTemplate() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(projectType, 30);
        return save(gradingReportTemplate);
    }

    private User createUser() {
        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        return save(user);
    }

    private ProjectType createProjectType() {
        return save(new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING));
    }

    private Project createProject(ProjectType projectType) {
        Project project = Project.builder()
                .title("title")
                .projectType(projectType)
                .headSupervisor(createUser())
                .build();
        return save(project);
    }

    private FinalSeminar createFinalSeminar(Project project) {
        Language english = new Language();
        english.setName("English");
        save(english);
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setRoom("101");
        finalSeminar.setStartDate(new Date());
        finalSeminar.setProject(project);
        finalSeminar.setPresentationLanguage(english);
        finalSeminar.setReportLanguage(english);
        return save(finalSeminar);
    }

    private FinalSeminarOpposition createOpposition(FinalSeminar finalSeminar, User student) {
        FinalSeminarOpposition opposition = new FinalSeminarOpposition();
        opposition.setFinalSeminar(finalSeminar);
        opposition.setUser(student);
        opposition.setProject(createProject(createProjectType()));
        return save(opposition);
    }
}
