package se.su.dsv.scipro.forum;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.GroupThread;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Collections;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;

public class GroupForumServiceTest extends ForumModuleTest {
    @Inject
    private GroupForumService groupForumService;

    private Group group;
    private User groupOwner;

    @Before
    public void setUp() throws Exception {
        groupOwner = save(User.builder().firstName("Bob").lastName("Example").emailAddress("bob@example.com").build());
        save(groupOwner);
        final Group group = new Group();
        group.setUser(groupOwner);
        group.setTitle("My group");
        this.group = save(group);
    }

    @Test
    public void create_thread() throws Exception {
        final String subject = "Subject";
        final String content = "Content";
        final GroupThread groupThread = groupForumService.createThread(group, groupOwner, subject, content, Collections.<FileUpload>emptySet());

        assertThat(groupThread.getGroup(), is(group));
        ForumThread forumThread = groupThread.getForumThread();
        assertThat(forumThread.getCreatedBy(), is(groupOwner));
        assertThat(forumThread.getSubject(), is(subject));
    }

    @Test
    public void reply() throws Exception {
        final GroupThread groupThread = groupForumService.createThread(group, groupOwner, "Subject", "Content", Collections.<FileUpload>emptySet());
        final String content = "Content";

        final ForumPost reply = groupForumService.createReply(groupThread, groupOwner, content, Collections.<FileDescription>emptySet());

        assertThat(reply.getForumThread(), is(groupThread.getForumThread()));
        assertThat(reply.getContent(), is(content));
        assertThat(reply.getPostedBy(), is(groupOwner));
    }

    @Test
    public void find_one() throws Exception {
        final GroupThread groupThread = groupForumService.createThread(group, groupOwner, "Subject", "Content", Collections.<FileUpload>emptySet());

        assertThat(groupForumService.findOne(groupThread.getId()), is(groupThread));
    }
}