package se.su.dsv.scipro.forummail;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.*;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import javax.inject.Inject;
import javax.mail.Part;
import java.util.Collections;

import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class ReadForumMailTest extends ForumMailModuleTest {

    @Inject
    private ReadForumMail readForumMail;

    private ForumMailMessage message;
    private GroupThread thread;

    @Before
    public void setUp() throws Exception {
        message = new ForumMailMessage("replyToID", "This is the body", Collections.singleton(new MailAttachment(mock(Part.class))));
        thread = createGroupThread();
    }

    @Test
    public void creates_group_thread_reply_if_found_message_matches_mail_event_message_id() throws Exception {
        ForumMailGroup forumMailGroup = createForumMailGroup();
        readForumMail.accept(message);
        verify(groupForumService).createReply(eq(forumMailGroup.getGroupThread()), eq(forumMailGroup.getUser()), eq(message.getMailBody()), anySetOf(FileDescription.class));
    }

    @Test
    public void creates_projct_thread_reply_if_found_message_matches_mail_event_message_id() throws Exception {
        ForumMailProject forumMailGroup = createForumMailProject();
        readForumMail.accept(message);
        verify(projectForumService).createReply(eq(forumMailGroup.getProjectThread()), eq(forumMailGroup.getUser()), eq(message.getMailBody()), anySetOf(FileDescription.class));
    }

    @Test
    public void no_mails_exists_should_not_create_thread_reply() throws Exception {
        readForumMail.accept(message);
        verify(groupForumService, never()).createReply(any(GroupThread.class), any(User.class), anyString(), anySetOf(FileDescription.class));
    }

    private ForumThread createThread() {
        ForumThread forumThread = new ForumThread();
        forumThread.setSubject("Subject");
        return save(forumThread);
    }

    private GroupThread createGroupThread() {
        Group group = createGroup();
        ForumThread forumThread1 = createThread();
        GroupThread groupThread = new GroupThread();
        groupThread.setForumThread(forumThread1);
        groupThread.setGroup(group);
        return save(groupThread);
    }

    private Group createGroup() {
        User user = save(User.builder().firstName("Bill").lastName("Tester").emailAddress("bill@example.com").build());
        Group group = new Group();
        group.setTitle("Group title");
        group.setUser(user);
        return save(group);
    }

    private MailEvent createMailEvent() {
        MailEvent mail = new MailEvent();
        mail.setSubject("subject");
        mail.setMessageBody("body");
        mail.setMessageID(message.getInReplyTo());
        return save(mail);
    }

    private ForumMailGroup createForumMailGroup() {
        MailEvent mailEvent = createMailEvent();
        return save(new ForumMailGroup(mailEvent, ObjectMother.SOME_USER, thread));
    }

    private ForumMailProject createForumMailProject() {
        MailEvent mailEvent = createMailEvent();
        return save(new ForumMailProject(mailEvent, createUser(), createProjectThread()));
    }

    private ProjectThread createProjectThread() {
        ProjectType bachelor = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        User supervisor = createUser();
        Project project = save(Project.builder().title("Project").projectType(bachelor).headSupervisor(supervisor).build());
        ProjectThread projectThread = new ProjectThread();
        projectThread.setProject(project);
        projectThread.setForumThread(createThread());
        return save(projectThread);
    }

    private User createUser() {
        return save(User.builder().firstName("Bob").lastName("Example").emailAddress("bob@example.com").build());
    }
}