package se.su.dsv.scipro.integration.activityforum;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.activityplan.Activity;
import se.su.dsv.scipro.activityplan.ActivityFileUploadedEvent;
import se.su.dsv.scipro.activityplan.ActivityPlan;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import java.util.HashSet;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class PostActivityUploadToForumTest {
    @Mock
    private ProjectForumService projectForumService;
    @Mock
    private ActivityThreadRepository activityThreadRepository;
    @Mock
    private EventBus eventBus;
    @InjectMocks
    private PostActivityUploadToForum integration;
    private Activity activity;
    private FileDescription fileDescription;
    private ProjectThread thread;
    private User uploader;

    @Before
    public void setUp() throws Exception {
        Project project = new Project();
        ActivityPlan activityPlan = new ActivityPlan();
        activityPlan.setProject(project);
        activity = new Activity();
        activity.setActivityPlan(activityPlan);
        fileDescription = new FileDescription();
        uploader = new User();
        fileDescription.setUploader(uploader);
        thread = new ProjectThread();

        when(projectForumService.createThread(activity.getActivityPlan().getProject(), activity.getTitle()))
                .thenReturn(thread);
    }

    @Test
    public void when_uploading_the_first_file_to_an_activity_should_create_thread() throws Exception {
        isFirstUpload();
        fireEvent();
        verify(projectForumService).createThread(activity.getActivityPlan().getProject(), activity.getTitle());
    }

    @Test
    public void when_uploading_additional_files_should_not_create_new_thread() throws Exception {
        isAdditionalUpload();
        fireEvent();
        verify(projectForumService, never()).createThread(any(Project.class), anyString());
    }

    @Test
    public void creates_connection_if_first_upload() throws Exception {
        isFirstUpload();
        fireEvent();
        ArgumentCaptor<ActivityThread> captor = ArgumentCaptor.forClass(ActivityThread.class);
        verify(activityThreadRepository).save(captor.capture());
        ActivityThread activityThread = captor.getValue();
        assertThat(activityThread.getActivity(), is(activity));
        assertThat(activityThread.getThread(), is(thread));
    }

    @Test
    public void should_post_the_file_as_reply_to_the_thread() throws Exception {
        isFirstUpload();
        fireEvent();
        HashSet<FileDescription> attachments = new HashSet<>();
        attachments.add(fileDescription);
        verify(projectForumService).createReply(thread, uploader, activity.getDescription(), attachments);//Optional.of(fileDescription));
    }

    private void isAdditionalUpload() {
        when(activityThreadRepository.findByActivity(activity)).thenReturn(Option.apply(thread));
    }

    private void isFirstUpload() {
        when(activityThreadRepository.findByActivity(activity)).thenReturn(Option.<ProjectThread>empty());
    }

    private void fireEvent() {
        integration.postActivityUploadToForum(new ActivityFileUploadedEvent(activity, fileDescription));
    }
}