package se.su.dsv.scipro.match;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Tuple2;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.activityplan.ActivityPlanTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Date;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class AddActivityPlanOnProjectStartTest {
    @Mock
    private ActivityPlanFacade activityPlanFacade;

    private ActivityPlanTemplate template;
    private ProjectType withActivityPlan;
    private ProjectType withoutActivityPlan;

    private AddActivityPlanOnProjectStart handler;

    @Before
    public void setUp() {
        template = new ActivityPlanTemplate();
        template.setId(2309482L);
        withActivityPlan = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        withActivityPlan.addModule(ProjectModule.ACTIVITY_PLAN);
        withoutActivityPlan = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
        withoutActivityPlan.removeModule(ProjectModule.ACTIVITY_PLAN);

        handler = new AddActivityPlanOnProjectStart(activityPlanFacade);
    }

    @Test
    public void adds_activity_plan() {
        Tuple2<Project, ProjectStartedEvent> tuple = prepareEventWithTemplate(withActivityPlan);

        handler.projectStarted(tuple._2());

        verify(activityPlanFacade).addActivitiesFromTemplate(eq(tuple._1()), eq(template), any(Date.class));
    }

    @Test
    public void does_not_add_activity_plan_if_activity_plan_module_is_turned_off() throws Exception {
        Tuple2<Project, ProjectStartedEvent> tuple = prepareEventWithTemplate(withoutActivityPlan);

        handler.projectStarted(tuple._2());

        verify(activityPlanFacade, never()).addActivitiesFromTemplate(eq(tuple._1()), eq(template), any(Date.class));
    }

    @Test
    public void does_not_add_activity_plan_if_no_default_is_specified() throws Exception {
        Tuple2<Project, ProjectStartedEvent> tuple = prepareEventWithoutTemplate(withActivityPlan);

        handler.projectStarted(tuple._2());

        verify(activityPlanFacade, never()).addActivitiesFromTemplate(eq(tuple._1()), any(ActivityPlanTemplate.class), any(Date.class));
    }

    private Tuple2<Project, ProjectStartedEvent> prepareEventWithoutTemplate(final ProjectType projectType) {
        return prepareEvent(projectType, null);
    }

    private Tuple2<Project, ProjectStartedEvent> prepareEventWithTemplate(ProjectType projectType) {
        return prepareEvent(projectType, template);
    }

    private Tuple2<Project, ProjectStartedEvent> prepareEvent(final ProjectType projectType, final ActivityPlanTemplate activityPlanTemplate) {
        Project project = Project.builder().title("Title").projectType(projectType).build();
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        ApplicationPeriodProjectType applicationPeriodProjectType = new ApplicationPeriodProjectType(applicationPeriod, projectType);
        applicationPeriodProjectType.setActivityPlanTemplate(activityPlanTemplate);
        applicationPeriod.getApplicationPeriodProjectTypes().add(applicationPeriodProjectType);
        Idea idea = new Idea();
        idea.setProject(project);
        idea.setApplicationPeriod(applicationPeriod);
        ProjectStartedEvent event = new ProjectStartedEvent(idea);
        return new Tuple2<>(project, event);
    }
}