package se.su.dsv.scipro.match;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class IdeaFacadeTest {

    @Mock
    private TargetService targetService;

    @Mock
    private IdeaService ideaService;
    @InjectMocks
    private IdeaFacade ideaFacade;
    private ApplicationPeriod applicationPeriod;
    private IdeaService.Filter params;
    private Idea idea;
    private List<Idea> ideas;

    @Before
    public void setUp() throws Exception {
        ideaFacade.ideaService = ideaService;
        ideaFacade.targetService = targetService;
        applicationPeriod = new ApplicationPeriod();
        params = new IdeaService.Filter();
        idea = new Idea();
        idea.setProjectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        Match match = new Match();
        match.setSupervisor(new UserBuilder().create());
        idea.setMatch(match);
        ideas = Collections.singletonList(idea);
    }

    @Test
    public void findIdeasPossibleToMatch() throws Exception {
        Pageable pageRequest = new PageRequest(0, 1);

        when(ideaService.findAll(params)).thenReturn(ideas);
        when(targetService.hasTargetsLeft(applicationPeriod, idea.getMatch().getSupervisor(), idea.getProjectType())).thenReturn(true);
        when(ideaService.findAllInList(ideas, pageRequest)).thenReturn(ideas);

        assertEquals(ideas, ideaFacade.findIdeasPossibleToMatch(applicationPeriod, params, pageRequest));
    }

    @Test
    public void countIdeasPossibleToMatch() throws Exception {
        int expectedInt = 1;

        when(ideaService.findAll(params)).thenReturn(ideas);
        when(targetService.hasTargetsLeft(applicationPeriod, idea.getMatch().getSupervisor(), idea.getProjectType())).thenReturn(true);

        assertEquals(expectedInt, ideaFacade.countIdeasPossibleToMatch(applicationPeriod, params));
    }
}
