package se.su.dsv.scipro.match;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.IdeaEvent;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;

import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class ProjectStartNotifierTest {

    @Mock
    private EventBus eventBus;
    @Mock
    private NotificationController notificationController;
    @InjectMocks
    private ProjectStartNotifier projectStartNotifier;
    private Idea idea;

    @Before
    public void setUp() throws Exception {
        idea = new Idea();
        idea.setExport(new IdeaExport());
    }

    @Test
    public void sends_success_notifications() throws Exception {
        projectStartNotifier.projectStarted(new ProjectStartedEvent(idea));
        verify(notificationController).notifyProject(eq(idea.getProject()), eq(ProjectEvent.Event.EXPORTED_SUCCESS), isA(NotificationSource.class));
    }

    @Test
    public void sends_failure_notifications() throws Exception {
        projectStartNotifier.projectStartDenied(new ProjectStartDeniedEvent(idea));
        verify(notificationController).notifyIdea(eq(idea), eq(IdeaEvent.Event.EXPORTED_FAIL), isA(NotificationSource.class));
    }
}
