package se.su.dsv.scipro.match;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Date;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.*;

public class TargetServiceImplTest extends MatchModuleTest {

    @Inject
    private TargetServiceImpl targetService;
    private ApplicationPeriod applicationPeriod;
    private User supervisor;
    private ProjectType projectType;

    @Before
    public void setUp() throws Exception {
        applicationPeriod = newApplicationPeriod();
        supervisor = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        projectType = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
    }

    @Test
    public void find_one() {
        assertNotNull(targetService.findOne(applicationPeriod, supervisor, projectType));
    }

    @Test
    public void hasTargetsLeft() throws Exception {
        createTarget(2);
        createIdea();
        assertTrue(targetService.hasTargetsLeft(applicationPeriod, supervisor));
    }

    @Test
    public void hasNoTargetsLeft() throws Exception {
        createTarget(1);
        createIdea();
        assertFalse(targetService.hasTargetsLeft(applicationPeriod, supervisor));
    }

    @Test
    public void ideasLeftToCreateToMeetTarget() throws Exception {
        createTarget(3);
        assertEquals(3, targetService.ideasLeftToCreateToMeetTarget(applicationPeriod, supervisor, projectType));
    }

    @Test
    public void ideasLeftToCreateToMeetTargetNoIdeasLeft() throws Exception {
        createTarget(2);
        createIdea();
        assertEquals(1, targetService.ideasLeftToCreateToMeetTarget(applicationPeriod, supervisor, projectType));
    }

    @Test
    public void has_targets_left_in_specific_project_type() {
        createTarget(1);
        assertTrue(targetService.hasTargetsLeft(applicationPeriod, supervisor, projectType));
    }

    @Test
    public void project_types_with_targets_left() {
        createTarget(0);
        assertTrue(targetService.projectTypesWithTargetsLeft(applicationPeriod, supervisor).isEmpty());
    }

    @Test
    public void project_types_with_targets_left_2() {
        createTarget(1);
        assertThat(targetService.projectTypesWithTargetsLeft(applicationPeriod, supervisor), hasItem(projectType));
    }

    private void createIdea() {
        Idea idea = new Idea();
        Match match = new Match();
        match.setSupervisor(supervisor);
        match.setStatus(Idea.Status.MATCHED);
        match.setIdea(idea);
        idea.setMatch(match);
        idea.setProjectType(projectType);
        idea.setTitle("My idea");
        idea.setCreator(supervisor);
        idea.setApplicationPeriod(applicationPeriod);
        idea.addIdeaParticipation(new IdeaParticipation(supervisor, idea));
        save(idea);
    }

    private Target createTarget(int targetLimit) {
        Target target = new Target(supervisor, applicationPeriod, projectType);
        target.setTarget(targetLimit);
        return save(target);
    }

    private ApplicationPeriod newApplicationPeriod() {
        ApplicationPeriod period = new ApplicationPeriod();
        period.setStartDate(new Date());
        period.setEndDate(new Date());
        period.setCourseStartDate(new Date());
        period.setName("My period");
        return save(period);
    }
}
