package se.su.dsv.scipro.milestones.service.impl;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Collection;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertThat;

public class MilestoneStatisticsServiceImplTest extends IntegrationTest {

    @Inject
    private MilestoneStatisticsServiceImpl milestoneStatisticsService;

    private ProjectType bachelor;
    private Project project;
    private MilestoneActivityTemplate firstMilestone;
    private MilestoneActivityTemplate secondMilestone;

    @Before
    public void setUp() throws Exception {
        bachelor = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        project = createProject();
        MilestonePhaseTemplate phase = save(new MilestonePhaseTemplate("Phase 1", "Description"));
        this.firstMilestone = createMilestone(phase);
        this.secondMilestone = createMilestone(phase);
    }

    @Test
    public void count_progressed_projects() throws Exception {
        Project completed = createProject();

        confirmMilestone(project, secondMilestone);
        confirmMilestone(completed, secondMilestone);
        confirmMilestone(completed, firstMilestone);

        assertThat(milestoneStatisticsService.countProgressedProjects(secondMilestone, firstMilestone), is(1L));
    }

    @Test
    public void progressed_projects() throws Exception {
        Project notCompleted = createProject();
        Project completed = createProject();

        confirmMilestone(project, secondMilestone);
        confirmMilestone(completed, secondMilestone);
        confirmMilestone(completed, firstMilestone);

        Collection<Project> progressedProjects = milestoneStatisticsService.getProgressedProjects(secondMilestone, firstMilestone, new PageRequest(0, 10));
        assertThat(progressedProjects, hasItem(project));
        assertThat(progressedProjects, not(hasItem(notCompleted)));
        assertThat(progressedProjects, not(hasItem(completed)));
    }

    private Project createProject() {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        return save(Project.builder().title("Title").projectType(bachelor).headSupervisor(headSupervisor).build());
    }

    private MilestoneActivityTemplate createMilestone(final MilestonePhaseTemplate phase) {
        MilestoneActivityTemplate individualMilestone = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT, "Project milestone", "Description");
        individualMilestone.setMilestonePhaseTemplate(phase);
        individualMilestone.addProjectType(bachelor);
        return save(individualMilestone);
    }

    private void confirmMilestone(final Project project, final MilestoneActivityTemplate projectMilestone) {
        Milestone milestone = new Milestone(project, projectMilestone);
        milestone.setConfirmed(true);
        save(milestone);
    }
}