package se.su.dsv.scipro.nonworkperiod;

import org.joda.time.DateTime;
import org.junit.Test;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Date;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class NonWorkDayPeriodServiceImplTest extends IntegrationTest {

    @Inject
    private NonWorkDayPeriodServiceImpl service;

    @Test
    public void add_non_work_day_period() throws Exception {
        newNonWorkDayPeriod(-10, 10);
        assertTrue(service.isNonWorkDay(new Date()));
    }

    @Test
    public void day_is_work_day_without_periods() throws Exception {
        assertFalse(service.isNonWorkDay(new Date()));
    }

    @Test(expected = IllegalStateException.class)
    public void periods_may_not_overlap() throws Exception {
        newNonWorkDayPeriod(-10, 10);
        newNonWorkDayPeriod(9, 11);
    }

    @Test
    public void period_does_not_overlap_with_itself() throws Exception {
        NonWorkDayPeriod nonWorkDayPeriod = newNonWorkDayPeriod(-10, 10);
        service.save(nonWorkDayPeriod);
    }

    private NonWorkDayPeriod newNonWorkDayPeriod(int startDayOffset, int endDayOffset) {
        NonWorkDayPeriod nonWorkDayPeriod = new NonWorkDayPeriod();
        nonWorkDayPeriod.setStartDate(new DateTime().plusDays(startDayOffset).toDate());
        nonWorkDayPeriod.setEndDate(new DateTime().plusDays(endDayOffset).toDate());
        nonWorkDayPeriod.setComment("Comment");
        return service.save(nonWorkDayPeriod);
    }
}