package se.su.dsv.scipro.notifications.interfaces.impl;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.NotificationMail;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Arrays;
import java.util.Collection;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;

public class NotificationMailFormatterImplTest {

    NotificationMailFormatterImpl formatter;

    @Before
    public void setUp() {
        formatter = new NotificationMailFormatterImpl();
        formatter.systemSettingsService = mock(GeneralSystemSettingsService.class);
        GeneralSystemSettings settings = new GeneralSystemSettings();
        settings.setSciproURL("http://test/");
        doReturn(settings).when(formatter.systemSettingsService).getGeneralSystemSettingsInstance();
    }

    @Test
    public void testFormat() throws Exception {
        Project project = ObjectMother._Project.create(1L);
        project.setHeadSupervisor(ObjectMother._User.create(17L, "Kalle", "Testare"));
        ProjectEvent event = new ProjectEvent();
        event.setProject(project);
        event.setEvent(ProjectEvent.Event.CREATED);
        Notification notification = new Notification();
        notification.setNotificationEvent(event);

        NotificationMail mail = formatter.format(notification);

        assertTrue(mail.getSubject().length() > 0);
        assertTrue(mail.getContent().length() > 0);
    }

    @Test
    public void testCompile() throws Exception {
        Project project = ObjectMother._Project.create(1L);
        ProjectEvent event = new ProjectEvent();
        event.setProject(project);
        event.setEvent(ProjectEvent.Event.CREATED);
        Notification notification = new Notification();
        notification.setNotificationEvent(event);
        Collection<Notification> notificationCollection = Arrays.asList(notification, notification);

        NotificationMail mail = formatter.compile(notificationCollection);

        assertTrue(mail.getSubject().length() > 0);
        assertTrue(mail.getContent().length() > 0);
    }
}
