package se.su.dsv.scipro.peer;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.*;

public class CommentThreadServiceImplTest extends IntegrationTest {


    @Inject
    private CommentThreadServiceImpl commentThreadService;

    private PeerReview peerReview;
    private ProjectType projectType;
    private CommentThread commentThread;

    @Before
    public void setUp() throws Exception {
        createDefaultProjectType();
        PeerRequest peerRequest = createPeerRequest(createProject());
        peerReview = createPeerReview(peerRequest);
        commentThread = createCommentThread(peerReview);
    }

    @Test
    public void save() throws Exception {
        assertThat(commentThreadService.commentThreadRepo.findAll(), hasItem(commentThread));
    }

    @Test
    public void getCommentThread() throws Exception {
        assertEquals(commentThread, commentThreadService.getCommentThread(peerReview));
    }

    @Test
    public void returns_null_when_noting_found() throws Exception {
        assertNull(commentThreadService.getCommentThread(createPeerReview(createPeerRequest(createProject()))));
    }

    private PeerReview createPeerReview(PeerRequest peerRequest) {
        PeerReview peerReview = new PeerReview();
        peerReview.setPeerRequest(peerRequest);
        peerReview.setProject(createProject());
        peerReview.setReviewer(createUser());
        return save(peerReview);
    }

    private CommentThread createCommentThread(PeerReview peerReview) {
        return save(new CommentThread(peerReview));
    }

    private PeerRequest createPeerRequest(Project project) {
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(project);
        peerRequest.setRequester(createUser());
        peerRequest.setFile(save(new FileDescription()));
        peerRequest = save(peerRequest);
        return peerRequest;
    }

    private User createUser() {
        return save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    private Project createProject() {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        return save(Project.builder().title("project").projectType(projectType).headSupervisor(headSupervisor).build());
    }

    private void createDefaultProjectType() {
        ProjectType projectType = save(new ProjectType(DegreeLevel.NONE, "name", "description"));
        this.projectType = save(projectType);
    }

}
