package se.su.dsv.scipro.project;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import static org.hamcrest.core.IsEqual.equalTo;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

public class ProjectTest {

    private Project project;

    @Before
    public void setUp() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        project = Project.builder().title("Foo").projectType(bachelor).build();
    }

    @Test
    public void clears_non_printable_unicode_characters_before_persisting() throws Exception {
        project.setTitle("å\u0094ä\u0096ö");

        project.cleanTitle();

        assertThat(project.getTitle(), equalTo("åäö"));
    }

    @Test
    public void when_status_is_changed_to_complete_state_of_mind_is_set_to_fine() throws Exception {
        project.setStateOfMind(StateOfMind.NEEDHELP);

        project.setProjectStatus(ProjectStatus.COMPLETED);

        assertEquals(StateOfMind.FINE, project.getStateOfMind());
    }

    @Test
    public void changing_status_to_inactive_does_not_change_state_of_mind() throws Exception {
        StateOfMind stateOfMind = StateOfMind.NEEDHELP;
        project.setStateOfMind(stateOfMind);

        project.setProjectStatus(ProjectStatus.ACTIVE);

        assertEquals(stateOfMind, project.getStateOfMind());
    }
}
