package se.su.dsv.scipro.report;

import com.google.common.eventbus.EventBus;
import com.google.inject.util.Providers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.EntityManager;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class GradingReportServiceImplTest {
    @Mock
    private EntityManager entityManager;
    @Mock
    private EventBus eventBus;

    private GradingReportServiceImpl gradingReportService;
    private ReviewerGradingReport reviewerReport;
    private SupervisorGradingReport supervisorReport;

    @Before
    public void setUp() throws Exception {
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        final GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(bachelor, 15);
        final Project project = Project.builder().title("My project").projectType(bachelor).build();
        final User user = User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build();
        reviewerReport = gradingReportTemplate.createReviewerReport(project);
        supervisorReport = gradingReportTemplate.createSupervisorReport(project, user);
        gradingReportService = new GradingReportServiceImpl(Providers.of(entityManager), eventBus);
    }

    @Test
    public void sends_out_event_when_reviewer_completes_initial_assessment() throws Exception {
        gradingReportService.initialAssessmentDone(reviewerReport);
        verify(eventBus).post(isA(ReviewerGradingReportInitialAssessmentDoneEvent.class));
    }

    @Test
    public void sounds_out_event_when_the_supervisor_completes_initial_assessment() throws Exception {
        gradingReportService.initialAssessmentDone(supervisorReport);
        verify(eventBus).post(isA(SupervisorGradingReportInitialAssessmentDoneEvent.class));
    }
}