package se.su.dsv.scipro.report;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;

public class SupervisorMaster30GradeCalculatorTest extends GradeCalculatorTest {
    private SupervisorGradingReport gradingReport;
    private SupervisorMaster30GradeCalculator gradeCalculator;

    @Before
    public void setUp() throws Exception {
        GradingReportTemplate gradingReportTemplate = prepareTemplate();

        gradingReport = gradingReportTemplate.createSupervisorReport(ObjectMother.SOME_MASTER_PROJECT, mock(User.class));
        gradeCalculator = new SupervisorMaster30GradeCalculator();
    }

    @Test
    public void report_with_30_points_gets_A() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(31);
        assertEquals(GradingReport.Grade.A, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void report_with_0_on_Ö4_gets_B() throws Exception {
        assessAllCriteriaWithMinimumAndTotalSkipDeadlineCriterion(30);
        assertEquals(GradingReport.Grade.B, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void report_with_28_points_gets_B() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(28);
        assertEquals(GradingReport.Grade.B, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void report_with_26_points_gets_C() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(26);
        assertEquals(GradingReport.Grade.C, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void report_with_24_points_gets_C() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(24);
        assertEquals(GradingReport.Grade.D, gradingReport.getGrade(gradeCalculator));
    }

    @Test
    public void report_with_22_points_gets_E() throws Exception {
        assessAllCriteriaWithMinimumAndTotal(22);
        assertEquals(GradingReport.Grade.E, gradingReport.getGrade(gradeCalculator));
    }

    @Override
    protected GradingReport getGradingReport() {
        return gradingReport;
    }

    @Override
    protected GradeCalculator getGradeCalculator() {
        return gradeCalculator;
    }

    @Override
    protected GradingReportTemplate prepareTemplate() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(ObjectMother.MASTER, 30);

        gradingReportTemplate.addProjectCriterion("U1 Sammanfattning", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U2 Introduktion", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U3 Problem", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U4 Frågeställning", 1, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U5 Vetenskaplig förankring", 2, getPointTemplates(3));
        gradingReportTemplate.addProjectCriterion("U6 Metodval", 2, getPointTemplates(3));
        gradingReportTemplate.addProjectCriterion("U7 Metodtillämpning", 2, getPointTemplates(3));
        gradingReportTemplate.addProjectCriterion("U8 Resultat", 2, getPointTemplates(3));
        gradingReportTemplate.addProjectCriterion("U9 Slutsatser och diskussion", 2, getPointTemplates(2));
        gradingReportTemplate.addProjectCriterion("U10 Form, struktur och språk", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U11 Argumentation", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U12 Källhänvisningar och dokumentation", 1, getPointTemplates(1));
        gradingReportTemplate.addProjectCriterion("U13 Originalitet och signifikans", 2, getPointTemplates(4));

        gradingReportTemplate.addIndividualCriterion("Ö1 Oppositionsrapport", 1, getPointTemplates(2));
        gradingReportTemplate.addIndividualCriterion("Ö2 Presentationer", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö3 Aktivitet vid seminarier och möten", 1, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö4 Deadlines", 0, getPointTemplates(1));
        gradingReportTemplate.addIndividualCriterion("Ö5 Revision efter slutseminarium", 0, getPointTemplates(1));

        return gradingReportTemplate;
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }

    private void assessAllCriteriaWithMinimumAndTotalSkipDeadlineCriterion(int remaining) {
        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            int toAssign = Math.min(remaining, gradingCriterion.getPointsRequiredToPass());
            if (!gradingCriterion.getTitle().equals("Ö4 Deadlines")) {
                int leftAfterAssignment = assignAtMost(toAssign, gradingCriterion);
                remaining -= (toAssign - leftAfterAssignment);
            }
        }

        for (GradingCriterion gradingCriterion : getGradingReport().getGradingCriteria()) {
            remaining = assignAtMost(remaining, gradingCriterion);
        }
    }
}
