package se.su.dsv.scipro.reviewing;

import com.google.common.base.Function;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class FinalSeminarApprovalServiceImplTest extends ReviewingModuleTest {

    @Inject
    private FinalSeminarApprovalServiceImpl finalSeminarApprovalService;
    private Project project;

    @Before
    public void setUp() throws Exception {
        User reviewer = save(User.builder().firstName("Ronny").lastName("Reviewer").emailAddress("reviewer@dsv.su.se").build());
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        ProjectType bachelor = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        project = save(Project.builder().title("My project").projectType(bachelor).headSupervisor(headSupervisor).build());
        project.addReviewer(reviewer);
    }

    @Test
    public void request_approval() throws Exception {
        finalSeminarApprovalService.requestApproval(project, createFileUpload(), "test");
        assertTrue(finalSeminarApprovalService.findBy(project).isDefined());
    }

    @Test
    public void findOne() throws Exception {
        finalSeminarApprovalService.requestApproval(project, createFileUpload(), "test");
        Option<FinalSeminarApproval> finalSeminarApproval = finalSeminarApprovalService.findBy(project);
        assertTrue(finalSeminarApproval.isDefined());
        assertEquals(finalSeminarApproval.get(), finalSeminarApprovalService.findOne(finalSeminarApproval.get().getId()));
    }

    @Test
    public void requires_upload() throws Exception {
        boolean requiresUpload = finalSeminarApprovalService.requiresUpload(project);
        assertTrue(requiresUpload);
    }

    @Test
    public void undecided_approval_does_not_require_approval() throws Exception {
        finalSeminarApprovalService.requestApproval(project, createFileUpload(), "test");
        boolean requiresUpload = finalSeminarApprovalService.requiresUpload(project);
        assertFalse(requiresUpload);
    }

    @SuppressWarnings("unchecked")
    private FileUpload createFileUpload() {
        FileUpload fileUpload = mock(FileUpload.class);
        when(fileUpload.handleData(any(Function.class))).thenReturn(null);
        return fileUpload;
    }
}
