package se.su.dsv.scipro.system;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.util.Pair;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class UserFacadeImplTest {

    @Mock
    private UserService userService;
    @Mock
    private UserNameService userNameService;
    @Mock
    private HashService hashService;

    private UserFacade userFacade;

    @Before
    public void setUp() throws Exception {
       userFacade = new UserFacadeImpl(userService, userNameService, hashService);
    }

    @Test
    public void valuesMissing() throws Exception {
        Pair<User, String> userStringPair = userFacade.attemptCreateUser("", "", "");
        Assert.assertNull(userStringPair.getHead());
        Assert.assertEquals(userStringPair.getTail(), "values missing");
    }

    @Test
    public void alreadyExists() throws Exception {
        when(userService.findByUsername(isA(String.class))).thenReturn(new User());
        when(userService.findByEmailAddress(isA(String.class))).thenReturn(new User());
        Pair<User, String> userStringPair = userFacade.attemptCreateUser("John", "Rambo", "rambo@us.gov");
        Assert.assertNull(userStringPair.getHead());
        Assert.assertEquals(userStringPair.getTail(), "user with same username/email already exists");
    }

    @Test
    public void invalidEmail() throws Exception {
        Pair<User, String> userStringPair = userFacade.attemptCreateUser("John", "Rambo", "asd");
        Assert.assertNull(userStringPair.getHead());
        Assert.assertEquals(userStringPair.getTail(), "invalid email");
    }

    @Test
    public void success() throws Exception {
        when(userService.findByUsername(isA(String.class))).thenReturn(null);
        when(userService.findByEmailAddress(isA(String.class))).thenReturn(null);
        Hash hash = new Hash();
        hash.setHash("anicehash");
        when(hashService.createForUser(isA(User.class))).thenReturn(hash);
        Pair<User, String> userStringPair = userFacade.attemptCreateUser("John", "Rambo", "rambo@us.gov");
        Assert.assertNotNull(userStringPair.getHead());
        Assert.assertEquals(userStringPair.getTail(), "anicehash");
    }

    @Test
    public void calls() throws Exception {
        when(userService.save(isA(User.class))).thenReturn(new User());
        when(userNameService.save(isA(Username.class))).thenReturn(new Username());
        userFacade.createUser(new User());
        verify(userService).save(isA(User.class));
        verify(userNameService).save(isA(Username.class));
    }
}