package se.su.dsv.scipro.finalseminar

import java.util.Date

import org.hamcrest.CoreMatchers._
import org.junit.Assert._
import org.junit.{Before, Test}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.system.{DegreeLevel, ProjectType}

class ExtendedFinalSeminarServiceImplTest {

  var exempted: FinalSeminar = _
  var notExempted: FinalSeminar = _
  val future = new Date
  val past = new Date(0)

  @Before
  def setUp(): Unit = {
    val master: ProjectType = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master")
    val exemptedProject = Project.builder().title("Title").projectType(master).build()
    exemptedProject.setFinalSeminarRuleExempted(true)
    val notExemptedProject = Project.builder().title("Title").projectType(master).build()

    exempted = new FinalSeminar(exemptedProject)
    notExempted = new FinalSeminar(notExemptedProject)
  }
  
  @Test
  def exempted_final_seminar_will_always_move(): Unit = {
    val move = ExtendedFinalSeminarServiceImpl.tryMove(exempted, to = future, earliest = past, allDaysAreHoliday)

    assertTrue(move.isRight)
  }

  @Test
  def trying_to_move_to_a_non_work_day_is_not_allowed(): Unit = {
    val move = ExtendedFinalSeminarServiceImpl.tryMove(notExempted, to = future, earliest = past, allDaysAreHoliday)

    assertThat(move, is[Either[SchedulingError, FinalSeminar]](Left(NonWorkDay(future))))
  }

  @Test
  def trying_to_move_before_earliest_date_is_not_allowed(): Unit = {
    val move = ExtendedFinalSeminarServiceImpl.tryMove(notExempted, to = past, earliest = future, allDaysAreWorkDays)

    assertThat(move, is[Either[SchedulingError, FinalSeminar]](Left(NotEnoughWorkDays(scheduling = past, earliest = future))))
  }

  @Test
  def move_to_non_work_day_after_earliest_date_is_allowed(): Unit = {
    val move = ExtendedFinalSeminarServiceImpl.tryMove(notExempted, to = future, earliest = past, allDaysAreWorkDays)

    assertTrue(move.isRight)
  }

  def allDaysAreHoliday: (Date) ⇒ Boolean = _ ⇒ true

  def allDaysAreWorkDays: (Date) ⇒ Boolean = _ ⇒ false
}