package se.su.dsv.scipro.grading

import java.util.Date
import javax.inject.Inject

import org.hamcrest.CoreMatchers.is
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.{Description, Matcher, TypeSafeMatcher}
import org.junit.{Before, Test}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.system.{DegreeLevel, ProjectType, User}

class GradeServiceTest extends GradingModuleTest {
  @Inject private var gradeService: GradeService = _
  private var project: Project = _
  private var user: User = _
  
  @Before def setUp(): Unit = {
    val bachelor: ProjectType = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
    user = save(User.builder.firstName("John").lastName("Doe").emailAddress("john@example.com").build)
    project = save(Project.builder.title("Project").projectType(bachelor).headSupervisor(user).build)
  }

  @Test def new_project_has_no_grade(): Unit = {
    assertThat(gradeService.getGrade(project), is(Option.empty[Grade]))
  }


  @Test def report_new_grade(): Unit = {
    val reported = gradeService.reportGrade(project, user, new Date, "A")
    assertThat(gradeService.getGrade(project), isSome[Grade](_.getValue == reported.getValue))
  }

  @Test def update_grade(): Unit = {
    gradeService.reportGrade(project, user, new Date, "B")
    val updated = gradeService.reportGrade(project, user, new Date, "C")
    assertThat(gradeService.getGrade(project), isSome[Grade](_.getValue == updated.getValue))
  }

  @Test def remove_grade(): Unit = {
    gradeService.reportGrade(project, user, new Date, "F")
    gradeService.removeGrade(project)
    assertThat(gradeService.getGrade(project), is(Option.empty[Grade]))
  }

  def isSome[A](predicate: A => Boolean): Matcher[Option[A]] = {
    new TypeSafeMatcher[Option[A]]() {
      override def matchesSafely(item: Option[A]): Boolean = item match {
        case Some(a) if predicate(a) => true
        case _ => false
      }

      override def describeTo(description: Description): Unit = {
        description.appendText("Some(_) that satisfies a predicate")
        ()
      }
    }
  }
}