package se.su.dsv.scipro.integration.daisy;

import com.google.inject.multibindings.Multibinder;
import com.google.inject.name.Names;
import com.google.inject.servlet.ServletModule;
import lombok.SneakyThrows;
import se.su.dsv.scipro.integration.daisy.workers.ProjectExporter;
import se.su.dsv.scipro.integration.daisy.workers.UserImportWorker;
import se.su.dsv.scipro.io.ExternalExporter;
import se.su.dsv.scipro.io.ExternalImporter;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.io.http.DaisyAPIImpl;
import se.su.dsv.scipro.io.impl.ExternalExporterDaisyImpl;
import se.su.dsv.scipro.io.impl.ExternalImporterDaisyImpl;
import se.su.dsv.scipro.io.impl.ImporterTransactions;
import se.su.dsv.scipro.io.impl.ImporterTransactionsImpl;
import se.su.dsv.scipro.match.IdeaCreationJudge;
import se.su.dsv.scipro.system.UserSearchService;

import java.io.IOException;
import java.util.Properties;

public class DaisyModule extends ServletModule {
    @Override
    protected void configureServlets() {
        filter("/*").through(SSOFilter.class);
        Multibinder<IdeaCreationJudge> judges = Multibinder.newSetBinder(binder(), IdeaCreationJudge.class);
        judges.addBinding().to(Daisy.class);

        bind(ExternalExporter.class).to(ExternalExporterDaisyImpl.class);

        bind(UserImportWorker.class);
        bind(ProjectExporter.class);
        bind(DaisyWorkerInitialization.class).asEagerSingleton();

        Properties properties = loadProperties();
        bindProperty(properties, "url");
        bindProperty(properties, "user");
        bindProperty(properties, "password");
        bind(ExternalImporter.class).to(ExternalImporterDaisyImpl.class);
        bind(ImporterTransactions.class).to(ImporterTransactionsImpl.class);
        bind(DaisyAPI.class).to(DaisyAPIImpl.class);

        Multibinder.newSetBinder(binder(), UserSearchService.class)
                .addBinding().to(DaisyUserSearchService.class);
    }

    @SneakyThrows(IOException.class)
    private Properties loadProperties() {
        Properties properties = new Properties();
        properties.load(getClass().getResourceAsStream(getClass().getSimpleName() + ".properties"));
        return properties;
    }

    protected void bindProperty(Properties properties, String property) {
        bindConstant().annotatedWith(Names.named(property)).to(properties.getProperty(property));
    }
}
