package se.su.dsv.scipro.integration.daisy.workers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import scala.Option;
import se.su.dsv.scipro.io.dto.STATUS;
import se.su.dsv.scipro.io.dto.Thesis;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.*;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.inject.Named;

@Named
public class ProjectFinalizer extends AbstractWorker {

    private static final Logger LOG = LoggerFactory.getLogger(ProjectFinalizer.class);

    private final ProjectService projectService;
    private final DaisyAPI daisyAPI;

    @Inject
    public ProjectFinalizer(ProjectService projectService, DaisyAPI daisyAPI) {
        this.projectService = projectService;
        this.daisyAPI = daisyAPI;
    }

    @Override
    protected void doWork() {
        beginTransaction();
        Iterable<Project> projects = projectService.findAll(QProject.project.projectStatus.eq(ProjectStatus.ACTIVE));
        for (Project project : projects) {
            try {
                if (project.getIdentifier() != null) {
                    Option<Thesis> thesisId = daisyAPI.getThesis(project.getIdentifier());
                    if (thesisId.isDefined()) {
                        Thesis thesis = thesisId.get();
                        if (STATUS.FINISHED.equals(thesis.getStatus())) {
                            projectService.complete(project);
                            LOG.info("Finalized project ({})", project);
                        }
                    }else {
                        LOG.info("Failed to finalize, did not find project: ({}) ", project);
                    }
                }
            } catch (Exception e) {
                LOG.info("Failed to finalize project ({})", project, e);
            }
        }
        commitTransaction();
    }
}
