package se.su.dsv.scipro.integration.daisy;

import com.google.inject.util.Providers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.io.ExternalImporter;
import se.su.dsv.scipro.system.CurrentUser;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class SSOFilterTest {

    @Mock
    private UserService userService;
    @Mock
    private CurrentUser currentUser;
    @Mock
    private ExternalImporter externalImporter;

    private SSOFilter ssoFilter;
    private User user;

    @Before
    public void setUp() throws Exception {
        ssoFilter = new SSOFilter(userService, externalImporter, Providers.of(currentUser));
        user = User.builder().firstName("Kalle").lastName("Tester").emailAddress("kalle@dsv.su.se").build();
    }

    @Test
    public void if_remote_user_is_provided_and_is_for_an_existing_user_then_log_in() throws Exception {
        when(currentUser.exists()).thenReturn(false);

        filterWithRemoteUser();

        verify(currentUser).set(user);
    }

    @Test
    public void if_already_logged_in_should_not_overwrite() throws Exception {
        when(currentUser.exists()).thenReturn(true);

        filterWithRemoteUser();

        verify(currentUser, never()).set(any(User.class));
    }

    @Test
    public void does_nothing_if_no_remote_user() throws Exception {
        when(currentUser.exists()).thenReturn(false);

        filterWithoutRemoteUser();

        verify(currentUser, never()).set(any(User.class));
    }

    @Test
    public void if_remote_user_is_provided_but_no_local_user_import_from_daisy() throws Exception {
        String username = "username@dsv.su.se";
        when(externalImporter.importUser(username)).thenReturn(Option.apply(user));

        filterWithRemoteUser(username);

        verify(currentUser).set(user);
    }

    private void filterWithoutRemoteUser() throws IOException, ServletException {
        ssoFilter.doFilter(mock(HttpServletRequest.class), mock(ServletResponse.class), mock(FilterChain.class));
    }

    private void filterWithRemoteUser() throws IOException, ServletException {
        String username = "kalle@dsv.su.se";
        when(userService.findByUsername(username)).thenReturn(user);

        filterWithRemoteUser(username);
    }

    private void filterWithRemoteUser(final String username) throws IOException, ServletException {
        HttpServletRequest request = mock(HttpServletRequest.class);
        when(request.getRemoteUser()).thenReturn(username);

        ssoFilter.doFilter(request, mock(ServletResponse.class), mock(FilterChain.class));
    }
}
