package se.su.dsv.scipro.integration.daisy.workers;

import com.google.inject.persist.UnitOfWork;
import com.google.inject.util.Providers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.io.dto.STATUS;
import se.su.dsv.scipro.io.dto.Thesis;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.QProject;
import se.su.dsv.scipro.workerthreads.WorkerData;
import se.su.dsv.scipro.workerthreads.WorkerDataService;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ProjectFinalizerTest {

    private ProjectFinalizer projectFinalizer;

    Project project = new Project();

    private @Mock DaisyAPI daisyAPI;
    private @Mock ProjectService projectService;
    private @Mock UnitOfWork unitOfWork;
    private @Mock EntityManager entityManager;
    private @Mock EntityTransaction entityTransaction;
    private @Mock WorkerDataService workerDataService;


    @Before
    public void setup() {
        projectFinalizer = new ProjectFinalizer(projectService, daisyAPI);
        projectFinalizer.setUnitOfWork(unitOfWork);
        projectFinalizer.setTxManager(Providers.of(entityManager));
        projectFinalizer.setWorkerDataService(workerDataService);
        when(workerDataService.save(any(WorkerData.class))).thenReturn(new WorkerData());
        when(entityManager.getTransaction()).thenReturn(entityTransaction);

        project.setId(3493L);
        project.setIdentifier(234L);
        project.setProjectStatus(ProjectStatus.ACTIVE);
        List<Project> projects = Collections.singletonList(project);
        when(projectService.findAll(QProject.project.projectStatus.eq(ProjectStatus.ACTIVE))).thenReturn(projects);

        Thesis thesis = new Thesis();
        thesis.setStatus(STATUS.FINISHED);
        when(daisyAPI.getThesis(anyLong())).thenReturn(Option.apply(thesis));
    }

    @Test
    public void testDoWork() throws Exception {
        projectFinalizer.run();

        verify(projectService).complete(project);
    }
}