package se.su.dsv.scipro.io.impl;

import org.apache.commons.lang3.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.io.dto.Thesis;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.Unit;

import javax.ws.rs.core.Response;
import java.net.URI;
import java.util.Date;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ExternalExporterDaisyImplTest {
    private static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");

    @Mock
    private DaisyAPI daisyAPI;

    private ExternalExporterDaisyImpl daisy;

    @Before
    public void setUp() throws Exception {
        daisy = new ExternalExporterDaisyImpl(daisyAPI);

        when(daisyAPI.createProject(any(Thesis.class))).thenReturn(Response.created(URI.create("/1")).build());
    }

    @Test
    public void truncates_title_length_if_too_long() throws Exception {
        String title = StringUtils.repeat('a', ExternalExporterDaisyImpl.MAX_TITLE_LENGTH);
        String longerTitle = title + "more title";
        Project project = Project.builder().title(longerTitle).projectType(BACHELOR).dateCreated(new Date()).build();

        daisy.createProject(project, new Unit());

        ArgumentCaptor<Thesis> captor = ArgumentCaptor.forClass(Thesis.class);
        verify(daisyAPI).createProject(captor.capture());

        assertThat(captor.getValue().getTitle(), is(title));
    }

    @Test
    public void application_period_start_date_on_project_to_be_exported() throws Exception {
        Date daisyStartDate = new Date();
        Project project = Project.builder().title("hej").projectType(BACHELOR).dateCreated(daisyStartDate).build();

        daisy.createProject(project, new Unit());

        ArgumentCaptor<Thesis> captor = ArgumentCaptor.forClass(Thesis.class);
        verify(daisyAPI).createProject(captor.capture());

        assertEquals(captor.getValue().getStartDate().getTime(), daisyStartDate);
    }
}
