package se.su.dsv.scipro.activityplan;

import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.project.Project;

import java.util.Date;
import java.util.Iterator;

public class ActivityDataProvider extends SortableDataProvider<Activity, String> {
    public enum SearchMode {
        PROJECT_ALL {
            @Override
            public String toString() {
                return "all";
            }
        }, PROJECT_PAST {
            @Override
            public String toString() {
                return "past";
            }
        }, PROJECT_FUTURE {
            @Override
            public String toString() {
                return "upcoming";
            }
        }
    }

    private ActivityPlanFacade activityPlanFacade;
    private ActivityService activityService;

    private SearchMode searchMode = SearchMode.PROJECT_ALL;
    private final Project project;

    public ActivityDataProvider(final ActivityPlanFacade activityPlanFacade, final ActivityService activityService, final Project p) {
        this.activityPlanFacade = activityPlanFacade;
        this.activityService = activityService;
        this.project = p;
    }

    /**
     * No backed support available for sorting yet, default facilities (with full sorting support) are used when search mode is ALL.
     */
    @Override
    public Iterator<Activity> iterator(long first, long count) {
        PageAdapter pageable = new PageAdapter(first, count, getSort());
        switch (searchMode) {
            case PROJECT_ALL:
                return activityPlanFacade.findEventsByProject(project, pageable).iterator();
            case PROJECT_PAST:
                return activityPlanFacade.findEventsByProject(project, null, new Date(), pageable).iterator();
            case PROJECT_FUTURE:
                return activityPlanFacade.findEventsByProject(project, new Date(), null, pageable).iterator();
            default:
                throw new IllegalStateException("Unknown search mode");
        }
    }

    @Override
    public long size() {
        switch (searchMode) {
            case PROJECT_ALL:
                return activityPlanFacade.countEventsByProject(project, null, null);
            case PROJECT_PAST:
                return activityPlanFacade.countEventsByProject(project, null, new Date());
            case PROJECT_FUTURE:
                return activityPlanFacade.countEventsByProject(project, new Date(), null);
            default:
                throw new IllegalStateException("Unknown search mode");
        }
    }

    @Override
    public IModel<Activity> model(Activity object) {
        return new DetachableServiceModel<>(activityService, object);
    }

}
