package se.su.dsv.scipro.activityplan;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AjaxIconLinkPanel;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public abstract class ActivityPlanTemplateDataPanel extends Panel {

    @Inject
    private ActivityPlanTemplateService activityPlanTemplateService;

    private ExportableDataPanel<ActivityPlanTemplate, String> dataPanel;

    private ActivityPlanTemplateService.Filter filter;

    public abstract void onNewClick();

    public abstract void onTitleClick(IModel<ActivityPlanTemplate> clicked, AjaxRequestTarget target);

    public abstract void onEditClick(IModel<ActivityPlanTemplate> clicked);

    public ActivityPlanTemplateDataPanel(String id) {
        super(id);

        filter = new ActivityPlanTemplateService.Filter();
        filter.setCreator(SciProSession.get().getUser());

        AjaxLink<Void> newLink = new AjaxLink<Void>("newLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                onNewClick();
            }
        };
        add(newLink);

        addTextFilter();
        addCheckBoxFilter();
        addDataTable();
    }

    private void addCheckBoxFilter() {
        AjaxCheckBox filterBox = new AjaxCheckBox("filterbox", new Model<>(true)) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                if (getModelObject()){
                    filter.setCreator(SciProSession.get().getUser());
                } else {
                    filter.setCreator(null);
                }
                target.add(dataPanel);
            }
        };
        add(new Label("filterboxLabel", "Show my templates and system templates"));
        add(filterBox);
    }

    private void addTextFilter() {
        TextField<String> titleField = new TextField<>("titleFilter", new PropertyModel<String>(filter, "filterString"));
        titleField.add(new OnChangeAjaxBehavior() {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(dataPanel);
            }
        });
        add(titleField);
    }

    private void addDataTable() {
        SortableDataProvider<ActivityPlanTemplate, String> provider = new SortableDataProvider<ActivityPlanTemplate, String>() {
            @Override
            public Iterator<ActivityPlanTemplate> iterator(long first, long count) {
                return activityPlanTemplateService.findAll(filter, new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return activityPlanTemplateService.count(filter);
            }

            @Override
            public IModel<ActivityPlanTemplate> model(ActivityPlanTemplate object) {
                return new DetachableServiceModel<>(activityPlanTemplateService, object);
            }
        };
        provider.setSort("title", SortOrder.DESCENDING);
        dataPanel = new ExportableDataPanel<>("dataTable", createColumns(), provider);
        add(dataPanel);
    }

    private List<IColumn<ActivityPlanTemplate, String>> createColumns() {
        List<IColumn<ActivityPlanTemplate, String>> columns = new ArrayList<>();

        columns.add(new PropertyColumn<ActivityPlanTemplate, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<ActivityPlanTemplate>> components, String s, final IModel<ActivityPlanTemplate> model) {
                components.add(new AjaxLinkPanel<ActivityPlanTemplate>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ActivityPlanTemplate> model) {
                        onTitleClick(model, target);
                    }
                });
            }
        });

        columns.add(new AbstractColumn<ActivityPlanTemplate, String>(Model.of("Edit")) {
            @Override
            public void populateItem(Item<ICellPopulator<ActivityPlanTemplate>> cellItem, String componentId, final IModel<ActivityPlanTemplate> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<ActivityPlanTemplate>(componentId, rowModel, Model.of(AjaxIconLinkPanel.EDIT)) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ActivityPlanTemplate> model) {
                        onEditClick(model);
                        target.add(dataPanel);
                    }

                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(SciProSession.get().getUser().hasRole(Roles.SYSADMIN) || rowModel.getObject().getCreator().equals(SciProSession.get().getUser()));
                    }
                });
            }
        });

        columns.add(new AbstractColumn<ActivityPlanTemplate, String>(Model.of("Delete")) {
            @Override
            public void populateItem(Item<ICellPopulator<ActivityPlanTemplate>> cellItem, String componentId, final IModel<ActivityPlanTemplate> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<ActivityPlanTemplate>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), getString("delete.confirmation")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ActivityPlanTemplate> model) {
                        activityPlanTemplateService.delete(model.getObject().getId());
                        target.add(dataPanel);
                    }

                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(SciProSession.get().getUser().hasRole(Roles.SYSADMIN) || rowModel.getObject().getCreator().equals(SciProSession.get().getUser()));
                    }
                });
            }
        });
        return columns;
    }
}
