package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class EditActivityTemplatePanel extends GenericPanel<ActivityPlanTemplate> {
    public static final String FORM = "form";
    public static final String TITLE_FIELD = "titleField";
    public static final String DESCRIPTION_AREA = "descriptionArea";
    public static final String ACTIVITY_LIST = "activityList";
    public static final String ACTIVITY_LINK = "activityLink";
    public static final String DIALOG = "dialog";

    public static final String ADD_LINK = "addLink";
    public static final String UP = "up";
    public static final String DOWN = "down";

    public static final String CONTAINER = "container";
    public static final String REMOVE = "remove";
    private final PreviewActivityPlanTemplatePanel preview;

    @Inject
    private ActivityPlanTemplateService activityPlanTemplateService;

    private ModalWindowPlus modalWindowPlus;

    public boolean isSysAdminTemplate(){
        return false;
    }

    public EditActivityTemplatePanel(String id, IModel<ActivityPlanTemplate> model) {
        super(id, model);

        modalWindowPlus = new ModalWindowPlus(DIALOG);
        add(modalWindowPlus);

        TemplateForm form = new TemplateForm(FORM, model);
        add(form);
        preview = new PreviewActivityPlanTemplatePanel("preview", Model.ofList(form.activityTemplates));
        preview.setOutputMarkupId(true);
        add(preview);
    }

    protected class TemplateForm extends Form<ActivityPlanTemplate> {
        List<ActivityTemplate> activityTemplates = new ArrayList<>();

        private void swapActivities(int index1, int index2){
            ActivityTemplate pet1 = activityTemplates.get(index1);
            ActivityTemplate pet2 = activityTemplates.get(index2);

            pet1.setNumberInOrder(index2);
            pet2.setNumberInOrder(index1);

            activityTemplates.set(index2, activityTemplates.get(index1));
            activityTemplates.set(index1, pet2);
        }

        public void addActivity(ActivityTemplate activity){
            activity.setActivityPlanTemplate(getModelObject());
            activity.setNumberInOrder(activityTemplates.size());
            activityTemplates.add(activity);
        }

        private void removeActivity(final ActivityTemplate event){
            activityTemplates.remove(event);
            for (ActivityTemplate pet : activityTemplates){
                pet.setNumberInOrder(activityTemplates.indexOf(pet));
            }
        }

        @Override
        protected void onValidate() {
            if (activityTemplates.isEmpty()) {
                error(getString("empty"));
            }
        }

        @Override
        public void onSubmit() {
            if (getModelObject().getCreator() == null){
                getModelObject().setCreator(SciProSession.get().getUser());
            }
            getModelObject().setIsSysAdminTemplate(isSysAdminTemplate());

            getModelObject().clearActivities();
            getModelObject().addActivities(activityTemplates);

            activityPlanTemplateService.save(getModelObject());
            activityTemplates.clear();
            activityTemplates.addAll(getModelObject().getActivityTemplates());
            info(getString("saved"));
        }

        public TemplateForm(String id, final IModel<ActivityPlanTemplate> model) {
            super(id, model);

            activityTemplates.addAll(model.getObject().getActivityTemplates());

            final WebMarkupContainer container = new WebMarkupContainer(CONTAINER);
            container.setOutputMarkupId(true);
            add(container);

            TextField<String> titleField = new TextField<>(TITLE_FIELD, new PropertyModel<String>(model, "title"));
            titleField.setRequired(true);
            add(titleField);

            add(new TextArea<>(DESCRIPTION_AREA, new PropertyModel<>(model, "description")));

            add(new AjaxLink<Void>(ADD_LINK) {
                @Override
                public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                    modalWindowPlus.setContent(new AdminEditActivityPanel(modalWindowPlus.getContentId(), new Model<>(new ActivityTemplate())) {
                        @Override
                        protected void onSubmit(AjaxRequestTarget target, IModel<ActivityTemplate> activity) {
                            modalWindowPlus.close(target);
                            addActivity(activity.getObject());
                            target.add(container);
                            target.add(preview);
                        }
                    });
                    modalWindowPlus.show(ajaxRequestTarget);
                }
            });

            container.add(new ListView<ActivityTemplate>(ACTIVITY_LIST, new PropertyModel<List<ActivityTemplate>>(this, "activityTemplates")) {
                @Override
                protected void populateItem(final ListItem<ActivityTemplate> item) {
                    AjaxLink<Void> activityLink = new AjaxLink<Void>(ACTIVITY_LINK) {
                        @Override
                        public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                            modalWindowPlus.setContent(new AdminEditActivityPanel(modalWindowPlus.getContentId(), item.getModel()) {
                                @Override
                                protected void onSubmit(AjaxRequestTarget target, IModel<ActivityTemplate> activity) {
                                    modalWindowPlus.close(target);
                                    target.add(container);
                                    target.add(preview);
                                }
                            });
                            modalWindowPlus.show(ajaxRequestTarget);
                        }
                    };
                    activityLink.setBody(new PropertyModel<ActivityTemplate>(item.getModel(), "title"));
                    item.add(activityLink);

                    item.add(new AjaxLink<Void>(UP) {
                        @Override
                        public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                            ajaxRequestTarget.add(container);
                            ajaxRequestTarget.add(preview);
                            swapActivities(item.getIndex(), item.getIndex() - 1);
                        }

                        @Override
                        protected void onConfigure() {
                            setVisibilityAllowed(item.getIndex() != 0);
                        }
                    });

                    item.add(new AjaxLink<Void>(DOWN) {
                        @Override
                        public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                            ajaxRequestTarget.add(container);
                            ajaxRequestTarget.add(preview);
                            swapActivities(item.getIndex(), item.getIndex() + 1);
                        }

                        @Override
                        protected void onConfigure() {
                            setVisibilityAllowed(item.getIndex() + 1 != activityTemplates.size());
                        }
                    });

                    item.add(new AjaxLink<Void>(REMOVE) {
                        @Override
                        public void onClick(AjaxRequestTarget ajaxRequestTarget) {
                            ajaxRequestTarget.add(container);
                            ajaxRequestTarget.add(preview);
                            removeActivity(item.getModelObject());
                        }
                    });
                }
            });
        }
    }
}