package se.su.dsv.scipro.activityplan;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.BootstrapDatePicker;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.misc.DaysService;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

import static se.su.dsv.scipro.activityplan.PreviewActivityPlanTemplatePanelHelper.getOffset;

public class PreviewActivityPlanTemplatePanel extends Panel {

    @Inject
    private DaysService daysService;

    private final IModel<Date> startDate = Model.of(new Date());

    public PreviewActivityPlanTemplatePanel(final String id, final IModel<List<? extends ActivityTemplate>> model) {
        super(id, model);

        final WebMarkupContainer container = new WebMarkupContainer("container");
        container.setOutputMarkupId(true);
        add(container);

        final BootstrapDateField startDateField = new BootstrapDateField("startDate", startDate);
        startDateField.add(new BootstrapDatePicker());
        startDateField.add(new OnChangeAjaxBehavior() {
            @Override
            protected void onUpdate(final AjaxRequestTarget target) {
                target.add(container);
            }
        });
        add(startDateField);

        container.add(new PropertyListView<ActivityTemplate>("activities", model) {
            @Override
            protected void populateItem(final ListItem<ActivityTemplate> item) {
                item.add(new Label("title"));
                item.add(new DateLabel("date", new LoadableDetachableModel<Date>() {
                    @Override
                    protected Date load() {
                        return calculateDate(startDate, model, item.getModel());
                    }
                }));
            }
        });
    }

    private Date calculateDate(
            final IModel<Date> startDateModel,
            final IModel<List<? extends ActivityTemplate>> activityPlanTemplateModel,
            final IModel<ActivityTemplate> activityTemplateModel) {
        return daysService.workDaysAfter(startDateModel.getObject(), getOffset(activityPlanTemplateModel.getObject(), activityTemplateModel.getObject()));
    }
}
