package se.su.dsv.scipro.admin.pages;

import com.google.common.base.Joiner;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.admin.panels.AdminViewMailPanel;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class AdminMailLogPage extends AbstractAdminMailPage {

    public static final String SUBJECT = "subject";
    @Inject
    private MailEventService mailEventService;

    public static final String MAIL_EVENTS = "mailEvents";
    public static final String MAIL_MODAL = "mailModal";

    private ModalWindowPlus mailModal;

    public AdminMailLogPage() {
        IModel<MailEventService.Filter> filter = newFilter();
        SortableDataProvider<MailEvent, String> provider = new FilteredDataProvider<>(mailEventService, filter);
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>(MAIL_EVENTS, columns(), provider));
        add(new FilterForm<>("filter", filter));
        mailModal = new ModalWindowPlus(MAIL_MODAL);
        add(mailModal);
    }

    private IModel<MailEventService.Filter> newFilter() {
        return Model.of(new MailEventService.Filter());
    }

    private List<? extends IColumn<MailEvent, String>> columns() {
        return Arrays.asList(getMailEventDateColumn(), getTitleColumn(), getUserRecipientsColumn(), getNonUserRecipientsColumn());
    }

    private DateColumn<MailEvent> getMailEventDateColumn() {
        return new DateColumn<>(new ResourceModel("sent").wrapOnAssignment(this), "dateCreated", "dateCreated", DateStyle.DATETIME);
    }

    private MultipleUsersColumn<MailEvent, String> getUserRecipientsColumn() {
        return new MultipleUsersColumn<MailEvent, String>(new ResourceModel("recipients").wrapOnAssignment(this)) {
            @Override
            public IModel<List<User>> getUsers(IModel<MailEvent> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<Set<User>>(rowModel, "recipients"));
            }
        };
    }
    private AbstractExportableColumn<MailEvent, String, String> getNonUserRecipientsColumn() {
        return new AbstractExportableColumn<MailEvent, String, String>(new ResourceModel("nonUserRecipients").wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<MailEvent>> cellItem, String componentId, IModel<MailEvent> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<String>(componentId,
                        new ListAdapterModel<>(new PropertyModel<Set<? extends String>>(rowModel, "nonUserRecipients"))) {

                    @Override
                    public Component getComponent(String componentId, IModel<String> t) {
                        return new Label(componentId, t);
                    }
                });
            }

            @Override
            public IModel<String> getDataModel(IModel<MailEvent> maileventIModel) {
                return Model.of(Joiner.on(", ").join(maileventIModel.getObject().getNonUserRecipients()));
            }
        };
    }

    private AbstractColumn<MailEvent, String> getTitleColumn() {
        return new AbstractColumn<MailEvent, String>(new ResourceModel("subject").wrapOnAssignment(this), SUBJECT) {
            @Override
            public void populateItem(Item<ICellPopulator<MailEvent>> item, String s, IModel<MailEvent> iModel) {
                item.add(new AjaxLinkPanel<MailEvent>(s, iModel, new PropertyModel<>(iModel, SUBJECT)) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<MailEvent> model) {
                        openMailModal(model, target);
                    }
                });
            }
        };
    }

    private void openMailModal(IModel<MailEvent> model, AjaxRequestTarget target) {
        mailModal.setTitle(model.getObject().getSubject());
        mailModal.setContent(new AdminViewMailPanel(mailModal.getContentId(), model));
        mailModal.show(target);
    }

    private static class FilterForm<T> extends Form<T> {
        public FilterForm(String id, IModel<T> model) {
            super(id, new CompoundPropertyModel<>(model));

            add(new TextField<String>(SUBJECT));
            add(new UserAutoComplete("user"));
            add(new DeactivatableDatePickerPanel("sentBetween",
                    new PropertyModel<Date>(model, "sentAfter"),
                    new PropertyModel<Date>(model, "sentBefore")));
        }
    }
}
