package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.LanguageService;

import javax.inject.Inject;
import java.util.List;

public class AdminLanguageSettingsPage extends AbstractAdminSystemPage {
    @Inject
    private LanguageService languageService;

    private FeedbackPanel feedbackPanel;
    private WebMarkupContainer wmc;
    private final ListView<Language> languages;

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminLanguageSettingsPage.class);

    public AdminLanguageSettingsPage() {

        feedbackPanel = new FeedbackPanel("feedback");
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        wmc.add(new LanguageForm("form", Model.of(new Language())));

        languages = new ListView<Language>("languages", getLanguages()) {
            @Override
            protected void populateItem(final ListItem<Language> item) {
                item.add(new Label("name", new PropertyModel<>(item.getModel(), "name")));
                item.add(new AjaxLink<Language>("delete", item.getModel()) {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        try {
                            languageService.delete(getModelObject());
                            success("Deleted language");
                            languages.setModel(getLanguages());
                        } catch (Exception e) {
                            String message = "Language is being used and can't be deleted";
                            error(message);
                            LOGGER.info(message, e);
                        }
                        target.add(feedbackPanel);
                        target.add(wmc);
                    }
                });
            }
        };
        wmc.add(languages);
    }

    private IModel<List<Language>> getLanguages(){
        return new LoadableDetachableModel<List<Language>>() {
            @Override
            protected List<Language> load() {
                return languageService.findAll();
            }
        };
    }

    private class LanguageForm extends Form<Language> {
        public LanguageForm(String id, IModel<Language> model) {
            super(id, model);
            add(new RequiredTextField<>("name", new PropertyModel<String>(model, "name")));
            add(new AjaxButton("add") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    languageService.save(LanguageForm.this.getModelObject());
                    LanguageForm.this.setModelObject(new Language());
                    success("Language added");
                    target.add(feedbackPanel);
                    target.add(wmc);
                }
            });
        }
    }
}
