package se.su.dsv.scipro.admin.panels;

import com.google.inject.persist.Transactional;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.system.FooterAddress;
import se.su.dsv.scipro.system.FooterAddressRepo;

import javax.inject.Inject;
import java.util.List;

public class AdminEditFooterAddressPanel extends Panel {

    @Inject
    FooterAddressRepo footerAddressRepo;

    public AdminEditFooterAddressPanel(String id) {
        super(id);
        List<FooterAddress> addresses = footerAddressRepo.findAll();
        FooterAddress footerAddress = addresses.isEmpty() ? new FooterAddress() : addresses.get(0);
        add(new AddressForm("form", Model.of(footerAddress)));
    }

    private class AddressForm extends Form<FooterAddress> {

        private final RequiredTextField<String> title;
        private final RequiredTextField<String> address;

        public AddressForm(String form, IModel<FooterAddress> footerAddressModel) {
            super(form, footerAddressModel);

            title = new RequiredTextField<>("title", new PropertyModel<String>(footerAddressModel, "title"));
            add(title);
            address = new RequiredTextField<>("address", new PropertyModel<String>(footerAddressModel, "address"));
            add(address);
        }

        @Override
        @Transactional
        protected void onSubmit() {
            FooterAddress footerAddress = getModelObject();
            if (footerAddress.getId() != null) {
                footerAddress = footerAddressRepo.findOne(footerAddress.getId());
            }
            footerAddress.setTitle(title.getModelObject());
            footerAddress.setAddress(address.getModelObject());
            footerAddressRepo.save(footerAddress);
            info(getString("saved"));
        }
    }
}
