package se.su.dsv.scipro.admin.panels;

import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Arrays;

public class AdminProjectTypePanel extends Panel {

    @Inject
    private ProjectTypeService projectTypeService;

    public AdminProjectTypePanel(final String id) {
        super(id);

        add(new ProjectTypeSettingsForm("projectTypeForm", new Model<>(new ProjectType()), false));
    }

    public AdminProjectTypePanel(String id, IModel<ProjectType> model) {
        super(id);

        add(new ProjectTypeSettingsForm("projectTypeForm", model, true));
    }

    private class ProjectTypeSettingsForm extends Form<ProjectType> {

        IModel<ProjectType> projectTypeModel;

        public ProjectTypeSettingsForm(String id, final IModel<ProjectType> model, final boolean editMode) {
            super(id, new CompoundPropertyModel<>(model));

            projectTypeModel = model;

	        add(new FeedbackPanel("feedback"));

	        TextField<ProjectType> name = new TextField<>("name");
	        add(name.setRequired(true));
            add(new TextArea<ProjectType>("description"));
	        DropDownChoice<DegreeLevel> degreeLevel = new DropDownChoice<>("degreeLevel", Arrays.asList(DegreeLevel.values()));
	        degreeLevel.setRequired(true);
	        add(degreeLevel);
            add(new CheckBox("reviewed"));

            Button createButton = new Button("createButton") {
	            @Override
                public void onSubmit() {

                    if (editMode) {
                        ProjectType pc = projectTypeService.save(projectTypeModel.getObject());
                        projectTypeModel.setObject(pc);
                        info("Type edited");
                    } else {
                        ProjectType pc = projectTypeService.save(projectTypeModel.getObject());
                        projectTypeModel.setObject(pc);
                        info("New type created");
                    }
                }
            };
            add(createButton);

            Button deleteButton = new Button("deleteButton") {

                @Override
                public void onSubmit() {
                    projectTypeModel.getObject().setDeleted(true);
                    projectTypeService.save(projectTypeModel.getObject());
                    info("Type inactivated");
                }
            };
            add(deleteButton);
            deleteButton.setEnabled(editMode);
        }
    }

}
