package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.BootstrapDateTimeComponent;
import se.su.dsv.scipro.components.DatesValidator;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminApplicationPeriod;
import se.su.dsv.scipro.match.AbstractAdminMatchPage;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodService;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class AdminEditApplicationPeriodPage extends AbstractAdminMatchPage implements MenuHighlightAdminApplicationPeriod {

    public static final String PROJECT_TYPES = "projectTypes";
    public static final String FORM = "form";
    public static final String START_DATE = "startDate";
    public static final String END_DATE = "endDate";
    public static final String COURSE_START_DATE = "courseStartDate";
    public static final String TITLE = "title";
    public static final String FEEDBACK = "Feedback";
    public static final String TITLE_FEEDBACK = "titleFeedback";
    public static final String CHECKBOX = "checkbox";

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;

    public AdminEditApplicationPeriodPage(PageParameters pp) {
        add(new EditForm(FORM, getLoaded(pp)));
    }

    private class EditForm extends Form<ApplicationPeriod> {

        public EditForm(String id, IModel<ApplicationPeriod> model) {
            super(id, model);
            add(new ComponentFeedbackPanel("formFeedback", this));
            RequiredTextField<Object> title = new RequiredTextField<>(TITLE, new PropertyModel<>(getModel(), "name"));
            add(new ComponentFeedbackPanel(TITLE_FEEDBACK, title));
            add(title);
            add(new PropertyListView<ProjectType>(PROJECT_TYPES, availableProjectTypes()) {
                @Override
                protected void populateItem(ListItem<ProjectType> item) {
                    item.add(new CheckBox(CHECKBOX, new ProjectTypeSelectionModel(item.getModel())));
                    item.add(new Label("name"));
                }
            });
            FormComponent<Date> startDate = addDateField(START_DATE);
            FormComponent<Date> endDate = addDateField(END_DATE);
            FormComponent<Date> courseStartDate = addDateField(COURSE_START_DATE);
            add(new DatesValidator(startDate, endDate));
            add(new DatesValidator(endDate, courseStartDate));
        }

        private IModel<List<ProjectType>> availableProjectTypes() {
            return new LoadableDetachableModel<List<ProjectType>>() {
                @Override
                protected List<ProjectType> load() {
                    return projectTypeService.findWithModule(ProjectModule.MATCH);
                }
            };
        }

        private FormComponent<Date> addDateField(String id) {
            FormComponent<Date> date = new BootstrapDateTimeComponent(id, new PropertyModel<Date>(getModel(), id));
            date.setRequired(true);
            add(new FencedFeedbackPanel(id + FEEDBACK, date));
            add(date);
            return date;
        }

        @Override
        protected void onSubmit() {
            applicationPeriodService.save(getModelObject());
            success(getString("success"));
        }

        @Override
        protected void onValidateModelObjects() {
            if (applicationPeriodService.doesPeriodOverlap(getModelObject())) {
                getRootForm().error(getString("overlapping"));
            }
        }

        private class ProjectTypeSelectionModel extends AbstractCheckBoxModel {
            private final IModel<ProjectType> model;

            public ProjectTypeSelectionModel(IModel<ProjectType> model) {
                this.model = model;
            }

            @Override
            public boolean isSelected() {
                return getModelObject().getProjectTypes().contains(model.getObject());
            }

            @Override
            public void select() {
                getModelObject().addProjectType(model.getObject());
            }

            @Override
            public void unselect() {
                getModelObject().removeProjectType(model.getObject());
            }
        }
    }

    private LoadableDetachableModel<ApplicationPeriod> getLoaded(final PageParameters pp){
        return new LoadableDetachableModel<ApplicationPeriod>() {
            @Override
            protected ApplicationPeriod load() {
                if (pp.get(PageParameterKeys.MAP.get(ApplicationPeriod.class)).isNull()) {
                    return new ApplicationPeriod();
                } else {
                     return applicationPeriodService.findOne(pp.get(PageParameterKeys.MAP.get(ApplicationPeriod.class)).toLong());
                }
            }
        };
    }
}