package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class ApplicationPeriodInfoPanel extends Panel {
    @Inject
    private ApplicationPeriodService periodService;
    @Inject
    private IdeaService ideaService;
    @Inject
    private TargetService targetService;

    private final boolean supervisorView;

    static final String PERIOD_LIST = "currentPeriods";
    static final String PROJECT_TYPE_LIST = "list";
    static final String PROJECT_TYPE = "projectType";
    static final String START_DATE = "startDate";
    static final String END_DATE = "endDate";
    static final String COURSE_START_DATE = "courseStartDate";
    static final String EMPTY = "emptyLabel";
    static final String STUDENT_CONTAINER = "studentContainer";
    static final String SUPERVISOR_CONTAINER = "supervisorContainer";
    static final String SELECT_STUDENT = "selectStudentIdeas";
    static final String SELECT_SUPERVISOR = "selectSupervisorIdeas";
    static final String CREATE = "createIdea";
    static final String TITLE = "title";

    public ApplicationPeriodInfoPanel(String id, boolean supervisorView) {
        super(id);
        this.supervisorView = supervisorView;

        List<ApplicationPeriod> currentPeriods;
        if (supervisorView) {
            currentPeriods = periodService.getCurrentPeriods();
        } else {
            currentPeriods = periodService.getCurrentPeriods(SciProSession.get().getUser());
        }

        addListOfPeriods(currentPeriods);
    }

    private void addListOfPeriods(final List<ApplicationPeriod> currentPeriods) {
        add(new AutoHidingListView<ApplicationPeriod>(PERIOD_LIST, currentPeriods) {

            @Override
            protected void populateItem(ListItem<ApplicationPeriod> item) {
                addPeriodInfo(item);
            }

            private void addStudentStuff(final ListItem<ApplicationPeriod> item, final PageParameters pp) {
                WebMarkupContainer studentContainer = new WebMarkupContainer(STUDENT_CONTAINER) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(!supervisorView && !ideaService.authorParticipatingOnActiveIdea(SciProSession.get().getUser(), item.getModelObject()));
                    }
                };
                item.add(studentContainer);
                studentContainer.add(new BookmarkablePageLink<Void>(SELECT_SUPERVISOR, ProjectNewAllSupervisorIdeasPage.class, pp));
                studentContainer.add(new BookmarkablePageLink<Void>(CREATE, ProjectIdeaSubmissionPage.class, pp));
            }

            private void addPeriodInfo(final ListItem<ApplicationPeriod> item) {
                final ApplicationPeriod ap = item.getModelObject();
                item.add(new Label(TITLE, ap.getName()));

                item.add(new ListView<ProjectType>(PROJECT_TYPE_LIST, new ArrayList<>(ap.getProjectTypes())) {
                    @Override
                    protected void populateItem(ListItem<ProjectType> item) {
                        item.add(new Label(PROJECT_TYPE, item.getModelObject().getName()));
                    }
                });

                item.add(new DateLabel(START_DATE, new PropertyModel<Date>(ap, "startDate")));
                item.add(new DateLabel(END_DATE, new PropertyModel<Date>(ap, "endDate")));
                item.add(new DateLabel(COURSE_START_DATE, new PropertyModel<Date>(ap, "courseStartDate")));

                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), item.getModelObject().getId());

                addStudentStuff(item, pp);

                WebMarkupContainer supervisorContainer = new WebMarkupContainer(SUPERVISOR_CONTAINER) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(supervisorView);
                    }
                };
                item.add(supervisorContainer);

                supervisorContainer.add(new BookmarkablePageLink<Void>(SELECT_STUDENT, SupervisorSelectableStudentIdeasPage.class, pp) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(targetService.hasTargetsLeft(ap, SciProSession.get().getUser()));
                    }
                });

                if (supervisorView) {
                    supervisorContainer.add(new ApplicationPeriodSupervisorPanel("sup", item.getModel()));
                } else {
                    supervisorContainer.add(new EmptyPanel("sup"));
                }
            }
        });

        add(new Label(EMPTY, "No application period open") {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(currentPeriods.isEmpty());
            }
        });
    }
}
