package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.StringResourceModel;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.match.TargetService;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class ApplicationPeriodSupervisorPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private TargetService targetService;

    private User employee;
    private final IModel<ApplicationPeriod> model;

    protected static final String LIST = "list";
    protected static final String PROJECT_TYPE = "name";
    protected static final String TARGET = "target";
    protected static final String CURRENT = "current";
    protected static final String INFO = "info";

    public ApplicationPeriodSupervisorPanel(String id, final IModel<ApplicationPeriod> model) {
        super(id, model);

        this.model = model;
        employee = SciProSession.get().getUser();

        List<ProjectType> projectTypes = new ArrayList<>(model.getObject().getProjectTypes());

        add(new PropertyListView<ProjectType>(LIST, projectTypes) {
            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                item.add(new Label(PROJECT_TYPE));
                item.add(new Label(TARGET, Integer.toString(getTarget(item.getModelObject()))));
                item.add(new Label(CURRENT, Long.toString(getCurrent(item.getModelObject()))));

                final int ideasLeftToCreateToMeetTarget = targetService.ideasLeftToCreateToMeetTarget(model.getObject(), employee, item.getModelObject());


                item.add(new Label(INFO, new StringResourceModel("info", this, item.getModel(), ideasLeftToCreateToMeetTarget)) {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(ideasLeftToCreateToMeetTarget > 0);
                    }
                });
            }
        });
    }

    private int getTarget(ProjectType pc) {
        return targetService.findOne(model.getObject(), employee, pc).getTarget();
    }

    private Long getCurrent(ProjectType projectType) {
        return ideaService.countMatched(model.getObject(), employee, projectType);
    }
}
