package se.su.dsv.scipro.basepanels;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.List;

public class FooterPanel extends Panel {
    @Inject
    private FooterLinkService footerLinkService;
    @Inject
    private FooterAddressRepo footerAddressRepo;

    public FooterPanel(String id) {
        super(id);

        add(new FooterLinks("first", FooterColumn.FIRST));
        add(new FooterLinks("second", FooterColumn.SECOND));
        add(new FooterLinks("third", FooterColumn.THIRD));
        add(new FooterLinks("fourth", FooterColumn.FOURTH));

        add(new FooterAddressPart("address"));
    }

    private class FooterAddressPart extends Fragment {

        public FooterAddressPart(String id) {
            super(id, "footerAddress", FooterPanel.this);

            final IModel<FooterAddress> footerAddress = new LoadableDetachableModel<FooterAddress>() {
                @Override
                protected FooterAddress load() {
                    List<FooterAddress> footerAddresses = footerAddressRepo.findAll();
                    return footerAddresses.isEmpty() ? new FooterAddress() : footerAddresses.get(0);
                }
            };
            WebMarkupContainer wmc = new WebMarkupContainer("addresses");
            wmc.setOutputMarkupId(true);
            wmc.add(new Label("title", new PropertyModel<String>(footerAddress, "title")));
            wmc.add(new Label("address", new PropertyModel<String>(footerAddress, "address")));
            add(wmc);
        }
    }

    private class FooterLinks extends Fragment {

        public FooterLinks(final String id, final FooterColumn column) {
            super(id, "footerLinks", FooterPanel.this);

            final IModel<List<? extends FooterLink>> links = new LoadableDetachableModel<List<? extends FooterLink>>() {
                @Override
                protected List<? extends FooterLink> load() {
                    return footerLinkService.getLinks(column);
                }
            };
            add(new AutoHidingListView<FooterLink>("links", links) {
                @Override
                protected void populateItem(final ListItem<FooterLink> item) {
                    item.add(new ExternalLink("link", new PropertyModel<String>(item.getModel(), "url"), new PropertyModel<String>(item.getModel(), "title")));
                }
            });
        }
    }
}
