package se.su.dsv.scipro.checklists;

import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklist.ChecklistQuestion;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.List;
import java.util.SortedSet;

public class ChecklistQuestionPanel extends GenericPanel<Checklist> {

    public static final String QUESTION_LABEL = "questionLabel";
    public static final String MEMBERS = "project.members";
    public static final String MEMBER_NAME = "name";
    public static final String QUESTIONS = "questions";
    public static final String QUESTION = "question";
    public static final String ANSWERS = "answers";

    @Inject
    private ActivityPlanFacade activityPlanFacade;
    @Inject
    private NotificationController notificationController;

    public ChecklistQuestionPanel(String id, IModel<Checklist> checklistModel) {
        super(id, checklistModel);
        activityPlanFacade.updateUserLastOpenDate(getModelObject(), SciProSession.get().getUser());

        WebMarkupContainer wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        wmc.add(new Label("title", new PropertyModel<>(getModel(), "name")));
        wmc.add(new MultiLineLabel("description", new PropertyModel<>(getModel(), "description")));
        wmc.add(new ListView<User>(MEMBERS, getAuthors()) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new UserLinkPanel(MEMBER_NAME, item.getModel()));
            }
        });
        wmc.add(new PropertyListView<ChecklistQuestion>(QUESTIONS) {
            @Override
            protected void populateItem(ListItem<ChecklistQuestion> item) {
                Label label = new Label(QUESTION_LABEL, new PropertyModel<String>(item.getModel(), QUESTION));
                label.add(new AttributeAppender("id", Model.of(item.getModelObject().getId())));
                item.add(label);
                item.add(new ChecklistAnswerCommentPanel(QUESTION, item.getModel()));
                item.add(new AnswerListView(ANSWERS, getAuthors(), item.getModel()));
            }
        });
    }

    private IModel<List<User>> getAuthors() {
        return new ListAdapterModel<>(new PropertyModel<SortedSet<User>>(getModel(), "project.projectParticipants"));
    }

    private class AnswerListView extends ListView<User> {
        private final IModel<ChecklistQuestion> question;

        public AnswerListView(String id, IModel<List<User>> model, IModel<ChecklistQuestion> question) {
            super(id, model);
            this.question = question;
            setOutputMarkupId(true);
        }

        @Override
        protected void populateItem(final ListItem<User> item) {
            item.add(new AnswerPanel("answer", question, item.getModel()) {
                @Override
                public void hook() {
                    if (activityPlanFacade.isCompletedByUser(SciProSession.get().getUser(), ChecklistQuestionPanel.this.getModelObject())) {
                        sendNotifications();
                    }
                }
            });
        }

        private void sendNotifications() {
            Checklist checklist = ChecklistQuestionPanel.this.getModelObject();

            PageParameters pp = new PageParameters();
            pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(checklist.getProject().getId()));
            pp.add(PageParameterKeys.MAP.get(Checklist.class), String.valueOf(checklist.getId()));

            NotificationSource source = new NotificationSource();
            source.setMessage(checklist.getName());

            notificationController.notifyProject(checklist.getProject(), ProjectEvent.Event.CHECKLIST_ANSWERED, source);
        }
    }
}