package se.su.dsv.scipro.checklists;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.checklist.ChecklistTemplateService;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.datatables.ClickableOrderColumn;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class ChecklistTemplateDataPanel extends Panel {

    @Inject
    public ChecklistTemplateService checklistTemplateService;

    ExportableDataPanel<ChecklistTemplate, String> dataTable;

    private IModel<String> titleString = Model.of("");

    public ChecklistTemplateDataPanel(String id) {
        super(id);
        AjaxLink<Void> newLink = new AjaxLink<Void>("newLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                setResponsePage(AdminEditChecklistTemplatePage.class);
            }
        };
        add(newLink);

        addTitleFilter();
        addTable();
    }

    private void addTable() {
        FilteredDataProvider<ChecklistTemplate, String> dataProvider = new FilteredDataProvider<>(checklistTemplateService, titleString);
        dataProvider.setSort("templateNumber", SortOrder.ASCENDING);
        dataTable = new ExportableDataPanel<>("dt", columns(), dataProvider);
        add(dataTable);
    }

    private List<IColumn<ChecklistTemplate, String>> columns() {
        List<IColumn<ChecklistTemplate, String>> columns = new ArrayList<>();

        columns.add(new ClickableOrderColumn<ChecklistTemplate>(Model.of("Order"), "templateNumber", "templateNumber") {
            @Override
            protected void onClick(IModel<ChecklistTemplate> clicked, AjaxRequestTarget target, boolean up) {
                if (up) {
                    checklistTemplateService.upChecklistTemplate(clicked.getObject());
                } else {
                    checklistTemplateService.downChecklistTemplate(clicked.getObject());
                }
                target.add(dataTable);
            }

            @Override
            public IModel<String> getDataModel(IModel<ChecklistTemplate> rowModel) {
                return Model.of(String.valueOf(rowModel.getObject().getTemplateNumber()));
            }
        });

        columns.add(new PropertyColumn<ChecklistTemplate, String>(Model.of("Title"), "name") {
            @Override
            public void populateItem(Item<ICellPopulator<ChecklistTemplate>> components, String s, final IModel<ChecklistTemplate> model) {
                components.add(new AjaxLinkPanel<ChecklistTemplate>(s, model, new PropertyModel(model, "name")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ChecklistTemplate> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(ChecklistTemplate.class), model.getObject().getId());
                        setResponsePage(AdminEditChecklistTemplatePage.class, pp);
                    }
                });
            }
        });
        columns.add(new AbstractExportableColumn<ChecklistTemplate, String, String>(Model.of("Types")) {
            @Override
            public void populateItem(Item<ICellPopulator<ChecklistTemplate>> item, String id, IModel<ChecklistTemplate> checklistTemplateIModel) {
                item.add(new Label(id, formatDegreeLevels(checklistTemplateIModel.getObject())));
            }

            @Override
            public IModel<String> getDataModel(IModel<ChecklistTemplate> rowModel) {
                return Model.of(formatDegreeLevels(rowModel.getObject()).toString());
            }
        });
        columns.add(new PropertyColumn<ChecklistTemplate, String>(Model.of("Categories"), "categoryNames"));

        columns.add(new ClickableIconColumn<ChecklistTemplate>(Model.of("Delete"), null, ClickableIconColumn.DELETE) {
            @Override
            protected void onClick(IModel<ChecklistTemplate> clicked, AjaxRequestTarget target) {
                checklistTemplateService.safeDeleteChecklistTemplate(clicked.getObject());
                target.add(dataTable);
            }
        });
        return columns;
    }

    private void addTitleFilter() {
        TextField<String> titleField = new TextField<>("titleFilter", titleString);
        titleField.add(new OnChangeAjaxBehavior() {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        });
        add(titleField);
    }

    private StringBuilder formatDegreeLevels(ChecklistTemplate checklistTemplate) {
        StringBuilder stringBuilder = new StringBuilder();
        for (Iterator<ProjectType> iterator = checklistTemplate.getProjectTypes().iterator(); iterator.hasNext(); ) {
            ProjectType next = iterator.next();
            stringBuilder.append(next.getName());
            if (iterator.hasNext()) {
                stringBuilder.append(", ");
            }
        }
        return stringBuilder;
    }
}
