package se.su.dsv.scipro.checklists;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.markup.html.image.ContextImage;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.checklist.ChecklistAnswerEnum;

public class TrafficLightPanel extends FormComponentPanel<ChecklistAnswerEnum> {

    private ContextImage greenImage, redImage, yellowImage, noAnswerImage;
    private ContextImage selectedGreen, selectedYellow, selectedRed, selectedNotApplicable;
    private ContextImage greenBackup, yellowBackup, redBackup, notApplicableBackup;
    private TrafficLightLink greenLink, redLink, yellowLink, noAnswerLink;
    private ChecklistAnswerEnum selected;

    public TrafficLightPanel(String id) {
        super(id);
    }

    public TrafficLightPanel(String id, IModel<ChecklistAnswerEnum> answerModel) {
        super(id, answerModel);
    }

    @Override
    protected void convertInput() {
        setConvertedInput(selected);
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        selected = getModelObject() != null ? getModelObject() : ChecklistAnswerEnum.NO_ANSWER;
        setUpImages();
        setUpLinks();
    }

    private void setUpImages() {
        greenImage = new ContextImage("greenImage", new Model<>("images/icons/bullet_ball_glass_green.png"));
        yellowImage = new ContextImage("yellowImage", new Model<>("images/icons/bullet_ball_glass_yellow.png"));
        redImage = new ContextImage("redImage", new Model<>("images/icons/bullet_ball_glass_red.png"));
        noAnswerImage = new ContextImage("noAnswerImage", new Model<>("images/icons/bullet_ball_glass_grey.png"));

        selectedGreen = new ContextImage("greenImage", new Model<>("images/icons/bullet_ball_glass_green_with_border.png"));
        selectedYellow = new ContextImage("yellowImage", new Model<>("images/icons/bullet_ball_glass_yellow_with_border.png"));
        selectedRed = new ContextImage("redImage", new Model<>("images/icons/bullet_ball_glass_red_with_border.png"));
        selectedNotApplicable = new ContextImage("noAnswerImage", new Model<>("images/icons/bullet_ball_glass_grey_with_border.png"));

        greenBackup = greenImage;
        yellowBackup = yellowImage;
        redBackup = redImage;
        notApplicableBackup = noAnswerImage;
    }

    private void setUpLinks() {
        greenLink = new TrafficLightLink("greenLink", ChecklistAnswerEnum.GREEN, selectedGreen, greenImage);
        redLink = new TrafficLightLink("redLink", ChecklistAnswerEnum.RED, selectedRed, redImage);
        yellowLink = new TrafficLightLink("yellowLink", ChecklistAnswerEnum.YELLOW, selectedYellow, yellowImage);
        noAnswerLink = new TrafficLightLink("noAnswerLink", ChecklistAnswerEnum.NOT_APLICABLE, selectedNotApplicable, noAnswerImage);
        add(greenLink);
        add(yellowLink);
        add(redLink);
        add(noAnswerLink);
    }

    public ChecklistAnswerEnum getSelected() {
        return selected;
    }

    private class TrafficLightLink extends AjaxLink<ChecklistAnswerEnum> {

        private final ContextImage selectedImage;

        public TrafficLightLink(String id, ChecklistAnswerEnum answer, ContextImage selectedImage, ContextImage defaultImage) {
            super(id, Model.of(answer));
            this.selectedImage = selectedImage;
            add(getImage(answer, selectedImage, defaultImage));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            resetImages();
            swapImages(getModelObject(), selectedImage, this);
            refreshImages(target);
        }

        private void resetImages() {
            greenImage = greenBackup;
            yellowImage = yellowBackup;
            redImage = redBackup;
            noAnswerImage = notApplicableBackup;
            greenLink.replace(greenImage);
            yellowLink.replace(yellowImage);
            redLink.replace(redImage);
            noAnswerLink.replace(noAnswerImage);
        }

        private void swapImages(ChecklistAnswerEnum answer, ContextImage selectedImage, TrafficLightLink link) {
            selected = answer;
            link.replace(selectedImage);
        }

        private void refreshImages(AjaxRequestTarget target) {
            target.add(greenLink);
            target.add(yellowLink);
            target.add(redLink);
            target.add(noAnswerLink);
        }

        private ContextImage getImage(ChecklistAnswerEnum answer, ContextImage selectedImage, ContextImage defaultImage) {
            return getSelected().equals(answer) ? selectedImage : defaultImage;
        }
    }
}
