package se.su.dsv.scipro.components;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.feedback.ComponentFeedbackMessageFilter;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.Set;

public class AddUsersPanel extends GenericPanel<Set<User>> {

    @Inject
    private UserService userService;

    private final ListView<User> users;

    public AddUsersPanel(String id, IModel<Set<User>> model, LookFor lookFor) {
        super(id, model);

        if (model.getObject()==null){
            model.setObject(new HashSet<User>());
        }

        final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        FencedFeedbackPanel feedbackPanel = new FencedFeedbackPanel("feedback", new ComponentFeedbackMessageFilter(AddUsersPanel.this));
        feedbackPanel.setOutputMarkupId(true);
        wmc.add(feedbackPanel);

        if (lookFor.equals(LookFor.AUTHORS)){
            wmc.add(new AuthorAutoComplete("add") {
                @Override
                protected void action(AjaxRequestTarget target, User object) {
                    if (object != null) {
                        AddUsersPanel.this.getModelObject().add(object);
                    }
                    refresh();
                    target.add(wmc);
                }
            });
        } else {
            wmc.add(new EmployeeAutoComplete("add") {
                @Override
                protected void action(AjaxRequestTarget target, User object) {
                    if (object != null) {
                        AddUsersPanel.this.getModelObject().add(object);
                    }
                    refresh();
                    target.add(wmc);
                }
            });
        }

        users = new ListView<User>("users", new ListAdapterModel<>(model)) {
            @Override
            protected void populateItem(final ListItem<User> item) {
                item.add(new UserLinkPanel("user", new DetachableServiceModel<>(userService, item.getModelObject())));
                item.add(new AjaxLink<User>("remove", item.getModel()) {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        AddUsersPanel.this.getModelObject().remove(item.getModelObject());
                        refresh();
                        target.add(wmc);
                    }
                });
            }
        };
        wmc.add(users);

        wmc.add(new Label("empty", Model.of("No users")) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(getModelObject()==null || getModelObject().isEmpty());
            }
        });
    }

    private void refresh(){
        users.detach();
    }

    public enum LookFor {
        AUTHORS, EMPLOYEES
    }
}
