package se.su.dsv.scipro.components;

import com.vaynberg.wicket.select2.Response;
import com.vaynberg.wicket.select2.TextChoiceProvider;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserRepo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class AutoCompleteRoleProvider extends TextChoiceProvider<User> {

    private static final int MAX_RESULTS = 6;

    private final UserRepo service;
    private final Roles role;

    protected AutoCompleteRoleProvider(UserRepo service, Roles role) {
        this.service = service;
        this.role = role;
    }

    @Override
    protected String getDisplayText(User choice) {
        StringBuilder sb = new StringBuilder();
        sb.append(choice.getFullName());
        sb.append(" <");
        sb.append(choice.getEmailAddress());
        sb.append(">");
        return sb.toString();
    }

    @Override
    protected Object getId(User choice) {
        return choice.getId();
    }

    @Override
    public void query(String term, int page, Response<User> response) {
        response.addAll(service.findByNameIncludes(term, role, new PageRequest(page, MAX_RESULTS)));
    }

    @Override
    public Collection<User> toChoices(Collection<String> ids) {
        List<User> list = new ArrayList<>(ids.size());
        for(String id : ids) {
            list.add(service.findOne(Long.valueOf(id)));
        }
        return list;
    }

}
