package se.su.dsv.scipro.components;

import org.apache.wicket.extensions.markup.html.form.DateTextField;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.Response;
import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;
import org.joda.time.LocalTime;

import java.util.Date;

public class BootstrapDateField extends DateTextField {
    private static final String DATE_PATTERN = "yyyy-MM-dd";

    public BootstrapDateField(final String id) {
        super(id, DATE_PATTERN);
    }

    public BootstrapDateField(final String id, final IModel<Date> model) {
        super(id, model, DATE_PATTERN);
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        setOutputMarkupId(true);
    }

    @Override
    protected void onAfterRender() {
        super.onAfterRender();
        Response response = getResponse();
        response.write("<script type=\"text/javascript\">");
        response.write("$(function() {");
        {
            response.write("$('#" + getMarkupId() + "').datetimepicker({");
            {
                response.write("format: 'yyyy-mm-dd',");
                response.write("weekStart: 1,");
                response.write("autoclose: true,");
                response.write("daysOfWeekDisabled: [0, 6],");
                response.write("minView: 'month',");
                response.write("todayHighlight: true");
            }
            response.write("})");
            response.write(".on('changeDate', function() { $('#" + getMarkupId() + "').trigger('inputchange'); });");
        }
        response.write("});");
        response.write("</script>");
    }

    @Override
    public void updateModel() {
        Date currentDate = getModelObject();
        if (currentDate == null || getConvertedInput() == null) {
            super.updateModel();
        }
        else {
            LocalDate date = new LocalDate(getConvertedInput());
            LocalTime time = new LocalTime(currentDate);
            LocalDateTime dateTime = date.toLocalDateTime(time);
            setModelObject(dateTime.toDate());
        }
    }
}
