package se.su.dsv.scipro.components;

import com.vaynberg.wicket.select2.TextChoiceProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.IValidator;
import org.apache.wicket.validation.ValidationError;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserRepo;

import javax.inject.Inject;

public class CoAuthorChoice extends AutoCompleteChoice<User> {

    @Inject
    private UserRepo studentService;
    @Inject
    private IdeaService ideaService;
    private User creator;
    private IModel<Idea> ideaModel;

    public CoAuthorChoice(String id, User creator, IModel<Idea> ideaModel) {
        super(id);
        this.creator = creator;
        this.ideaModel = ideaModel;
        setModel(new Model<User>());
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        add(new CoAuthorValidator());
    }

    @Override
    protected TextChoiceProvider<User> getChoiceProvider() {
        return new AutoCompleteRoleProvider(studentService, Roles.AUTHOR);
    }

    private class CoAuthorValidator implements IValidator<User> {
        @Override
        public void validate(IValidatable<User> coAuthor) {
            if (ideaModel.getObject().getProjectType() == null) {
                coAuthor.error(new ValidationError(this, "noProjectType"));
                return;
            }
            if(ideaModel.getObject().getProjectType().getMaxAuthors() == 1) {
                coAuthor.error(new ValidationError(this, "toManyAuthors"));
                return;
            }
            if(coAuthor.getValue().equals(creator)) {
                coAuthor.error(new ValidationError(this, "addedYourself"));
            }
            if(ideaService.authorParticipatingOnActiveIdea(coAuthor.getValue(), ideaModel.getObject().getApplicationPeriod())) {
                coAuthor.error(new ValidationError(this, "coAuthorParticipating"));
            }
        }
    }
}
