package se.su.dsv.scipro.components;

import org.apache.wicket.core.util.lang.PropertyResolver;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.data.enums.DateStyle;

import java.text.SimpleDateFormat;
import java.util.Date;

public class DateColumn<T> extends PropertyColumn<T, String> {

    private final DateStyle dateStyle;

    public DateColumn(IModel<String> displayModel, String propertyExpression) {
        this(displayModel, propertyExpression, DateStyle.DATE);
    }

    public DateColumn(IModel<String> displayModel, String propertyExpression, DateStyle dateStyle) {
        super(displayModel, propertyExpression);
        this.dateStyle = dateStyle;
    }

    public DateColumn(IModel<String> displayModel, String property, String sort) {
        this(displayModel, property, sort, DateStyle.DATE);
    }

    public DateColumn(IModel<String> displayModel, String property, String sort, DateStyle dateStyle) {
        super(displayModel, sort, property);
        this.dateStyle = dateStyle;
    }

    @Override
    public IModel<Object> getDataModel(final IModel<T> rowModel) {
        return new AbstractReadOnlyModel<Object>() {
            @Override
            public Object getObject() {
                final Date date = (Date) PropertyResolver.getValue(getPropertyExpression(), rowModel.getObject());
                if (date == null) {
                    return null;
                }
                else {
                    final SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dateStyle.getFormat());
                    return simpleDateFormat.format(date);
                }
            }
        };
    }

    @Override
    public void populateItem(Item<ICellPopulator<T>> cellItem, String componentId, IModel<T> rowModel) {
        cellItem.add(new DateLabel(componentId, new PropertyModel<Date>(rowModel, getPropertyExpression()), dateStyle));
    }
}