package se.su.dsv.scipro.components;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.IModel;
import org.apache.wicket.util.convert.IConverter;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;

import javax.inject.Inject;
import java.util.Date;
import java.util.Locale;

public class DateLabel extends Label {
    @Inject
    private DateService dateService;
    private final DateStyle dateStyle;

    public DateLabel(String id) {
        this(id, DateStyle.DATE);
    }

    public DateLabel(String id, DateStyle dateStyle) {
        super(id);
        this.dateStyle = dateStyle;
    }

    public DateLabel(String id, IModel<? extends Date> model) {
        this(id, model, DateStyle.DATE);
    }

    public DateLabel(String id, IModel<? extends Date> model, DateStyle dateStyle) {
        super(id, model);
        this.dateStyle = dateStyle;
    }

    @Override
    @SuppressWarnings("unchecked")
    public <C> IConverter<C> getConverter(Class<C> type) {
        if (Date.class.isAssignableFrom(type)) {
            return (IConverter<C>) new DateConverter();
        }
        return super.getConverter(type);
    }

    private class DateConverter implements IConverter<Date> {
        @Override
        public Date convertToObject(String value, Locale locale) {
            throw new UnsupportedOperationException("DateLabel should not be converting back to date");
        }

        @Override
        public String convertToString(Date value, Locale locale) {
            return dateService.format(value, dateStyle);
        }
    }
}
