package se.su.dsv.scipro.datatables.project;

import org.apache.wicket.Component;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.icons.ImageObject;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.StateOfMind;

public class ProjectStateColumn extends AbstractExportableColumn<Project, String, String> {

    static final String STATE_INFO = "Authors indicate if feedback is needed or not and how urgent it is. This alert function is solely controlled by the authors";
    public static final String TITLE = "title";
    public static final String IMG = "img";

    public ProjectStateColumn(IModel<String> displayModel, String sort) {
        super(displayModel, sort);
    }

    @Override
    public Component getHeader(String componentId) {
        return super.getHeader(componentId).add(new AttributeAppender(TITLE, Model.of(STATE_INFO), " "));
    }

    public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
        cellItem.add(new LinkPanel(componentId, rowModel));
    }

    @Override
    public IModel<String> getDataModel(IModel<Project> rowModel) {
        return Model.of(rowModel.getObject().getProjectStatus().toString());
    }

    private static class LinkPanel extends Panel {
        public LinkPanel(String id, IModel<Project> rowModel) {
            super(id);

            Project p = rowModel.getObject();

            switch (p.getStateOfMind()) {
                case FINE:
                    add(new ImageObject(IMG, ImageObject.TWENTYFOUR + ImageObject.LIGHT_GREEN));
                    add(new AttributeAppender(TITLE, Model.of(StateOfMind.FINE.getStatus()), " "));
                    break;
                case NEUTRAL:
                    add(new ImageObject(IMG, ImageObject.TWENTYFOUR + ImageObject.LIGHT_YELLOW));
                    add(new AttributeAppender(TITLE, Model.of(StateOfMind.NEUTRAL.getStatus()), " "));
                    break;
                case NEEDHELP:
                    add(new ImageObject(IMG, ImageObject.TWENTYFOUR + ImageObject.LIGHT_RED));
                    add(new AttributeAppender(TITLE, Model.of(StateOfMind.NEEDHELP.getStatus()), " "));
                    break;
                default:
                    add(new ImageObject(IMG, ImageObject.TWENTYFOUR + ImageObject.LIGHT_RED));
                    break;
            }
        }
    }
}