package se.su.dsv.scipro.date;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.border.Border;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

import java.util.Date;

public class DeactivatableDatePickerPanel extends Border {
    public static final String TOGGLE = "toggle";
    public static final String DATE_PICKER = "datePicker";

    private final DatePickerPanel datePicker;
    private final AjaxCheckBox active;
    private final IModel<Date> startDate;
    private final IModel<Date> endDate;

    public DeactivatableDatePickerPanel(String id, IModel<Date> startDate, IModel<Date> endDate) {
        this(id, startDate, endDate, Model.of(false));
    }

    public DeactivatableDatePickerPanel(String id, IModel<Date> startDate, IModel<Date> endDate, IModel<Boolean> activated) {
        super(id);
        this.startDate = startDate;
        this.endDate = endDate;

        active = new AjaxCheckBox(TOGGLE, activated) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(datePicker);
                if (getModelObject()) {
                    DeactivatableDatePickerPanel.this.startDate.setObject(datePicker.getStartDate());
                    DeactivatableDatePickerPanel.this.endDate.setObject(datePicker.getEndDate());
                }
                else {
                    DeactivatableDatePickerPanel.this.startDate.setObject(null);
                    DeactivatableDatePickerPanel.this.endDate.setObject(null);
                }
                datesChanged(target);
            }
        };
        addToBorder(active);

        datePicker = new DatePickerPanel(DATE_PICKER, startDate, endDate) {
            @Override
            public void updateStartDate(AjaxRequestTarget target) {
                DeactivatableDatePickerPanel.this.startDate.setObject(getStartDate());
                DeactivatableDatePickerPanel.this.datesChanged(target);
            }

            @Override
            public void updateEndDate(AjaxRequestTarget target) {
                DeactivatableDatePickerPanel.this.endDate.setObject(getEndDate());
                DeactivatableDatePickerPanel.this.datesChanged(target);
            }

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setEnabled(active.getModelObject());
            }
        };
        datePicker.setOutputMarkupId(true);
        addToBorder(datePicker);
    }

    protected void datesChanged(AjaxRequestTarget target) {}

    @Override
    protected void onDetach() {
        startDate.detach();
        endDate.detach();
        super.onDetach();
    }
}
