package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AjaxIconLinkPanel;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.reusable.SciProUtilities;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class AdminFinalSeminarDataPanel extends Panel {

    public static final String DATE_PICKER_PANEL = "datePickerPanel";
    public static final String DATA_TABEL = "dataTable";

    @Inject
    private FinalSeminarService finalSeminarService;

    private FinalSeminarService.Filter filter;
    private ExportableDataPanel<FinalSeminar, String> dataTable;
    private final FeedbackPanel feedback;

    public AdminFinalSeminarDataPanel(String id) {
        super(id);
        feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        addFiltering();
        addDataTable();
    }

    private void addFiltering() {
        filter = new FinalSeminarService.Filter();

        Date fromDate = SciProUtilities.setTimeOfDayTo0000(new Date());

        filter.setFromDate(fromDate);

        final DatePickerPanel datePanel = new DatePickerPanel(DATE_PICKER_PANEL, fromDate) {
            @Override
            public void updateStartDate(AjaxRequestTarget target) {
                filter.setFromDate(getStartDate());
                target.add(dataTable);
            }
            @Override
            public void updateEndDate(AjaxRequestTarget target) {
                filter.setToDate(getEndDate());
                target.add(dataTable);
            }
        };
        add(datePanel);
    }

    private void addDataTable() {
        SortableDataProvider<FinalSeminar, String> provider = new FilteredDataProvider<>(finalSeminarService, Model.of(filter), "startDate", SortOrder.ASCENDING);
        dataTable = new ExportableDataPanel<>(DATA_TABEL, createColumns(), provider);
        add(dataTable);
    }

    private List<IColumn<FinalSeminar, String>> createColumns() {
        List<IColumn<FinalSeminar, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<FinalSeminar>(Model.of("Date"), "startDate", "startDate", DateStyle.DATETIME));
        columns.add(new PropertyColumn<FinalSeminar, String>(Model.of("Type"), "project.projectType.name", "project.projectType.name"));

        columns.add(new PropertyColumn<FinalSeminar, String>(Model.of("Title"), "project.title") {
            @Override
            public void populateItem(Item<ICellPopulator<FinalSeminar>> components, String s, final IModel<FinalSeminar> model) {
                components.add(new AjaxLinkPanel<FinalSeminar>(s, model, new PropertyModel(model, "project.title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<FinalSeminar> model) {
                        setResponsePage(new SupervisorFinalSeminarDetailsPage(getModel()));
                    }
                });
            }
        });

        columns.add(new AbstractColumn<FinalSeminar, String>(Model.of("Cancel")) {
            @Override
            public void populateItem(Item<ICellPopulator<FinalSeminar>> cellItem, String componentId, final IModel<FinalSeminar> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<FinalSeminar>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), "Are you sure you want to cancel final seminar?") {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<FinalSeminar> model) {
                        setModelObject(finalSeminarService.cancel(getModelObject()));
                        success("Cancelled final seminar");
                        target.add(feedback);
                        target.add(dataTable);
                    }
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(rowModel.getObject().getStartDate().after(new Date()) && !rowModel.getObject().isCancelled());
                    }
                });
            }
        });

        return columns;
    }
}
