package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AutoSavingBehaviour;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorOpposition;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.grading.FillOutReportPanel;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.project.pages.ProjectDetailsPage;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.report.OppositionReportService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;

public class OppositionReportPage extends AbstractProjectDetailsPage implements MenuHighlightAuthorOpposition {

    public static final String THESIS_SUMMARY = "thesisSummary";
    public static final String FILL_OUT_REPORT = "fillOutReport";

    @Inject
    private FinalSeminarOppositionRepo finalSeminarOppositionRepo;
    @Inject
    private OppositionReportService oppositionReportService;

    public OppositionReportPage(PageParameters pp) {
        super(pp);

        if (pp.get("oid").isNull()) {
            throw new RestartResponseException(ProjectDetailsPage.class, pp);
        }

        final FinalSeminarOpposition opposition = finalSeminarOppositionRepo.findOne(pp.get("oid").toLong());

        if (opposition == null) {
            throw new RestartResponseException(ProjectDetailsPage.class, pp);
        }

        final IModel<OppositionReport> report = getOppositionReport(opposition);

        add(new ViewAttachmentPanel("thesisFile", new PropertyModel<FileDescription>(report, "finalSeminarOpposition.finalSeminar.document")));

        add(new FillOutReportPanel<OppositionReport>(FILL_OUT_REPORT, report) {
            {
                TextArea<String> textArea = new TextArea<>(THESIS_SUMMARY, new PropertyModel<String>(report, THESIS_SUMMARY));
                textArea.add(new AutoSavingBehaviour() {
                    @Override
                    protected void onUpdate(AjaxRequestTarget target) {
                        oppositionReportService.save(report.getObject());
                    }
                });
                add(textArea);
            }

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setEnabled(opposition.getUser().equals(SciProSession.get().getUser()));
            }
        });
    }

    private IModel<OppositionReport> getOppositionReport(final FinalSeminarOpposition opposition) {
        return new LoadableDetachableModel<OppositionReport>() {
            @Override
            protected OppositionReport load() {
                return oppositionReportService.findOrCreateReport(opposition);
            }
        };
    }
}
