package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.joda.time.DateTime;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.profile.UserLabel;

import javax.inject.Inject;
import java.util.Date;
import java.util.Set;

public class PublicFinalSeminarListingPage extends PublicPage {
    @Inject
    private FinalSeminarService finalSeminarService;

    public PublicFinalSeminarListingPage() {
        add(new PropertyListView<FinalSeminar>("finalSeminars", new ListAdapterModel<>(getSeminars())) {
            @Override
            protected void populateItem(final ListItem<FinalSeminar> item) {
                item.add(new Label("project.projectType.name"));
                item.add(new Label("project.title"));
                item.add(new UserLabel("project.headSupervisor"));
                item.add(new DateLabel("startDate", DateStyle.DATETIME));
                item.add(new Label("room"));

                IModel<Set<FinalSeminarRespondent>> respondents = new PropertyModel<>(item.getModel(), "respondents");
                item.add(new PropertyListView<FinalSeminarRespondent>("respondents", new ListAdapterModel<>(respondents)) {
                    @Override
                    protected void populateItem(final ListItem<FinalSeminarRespondent> item) {
                        item.add(new UserLabel("user"));
                    }
                });
            }
        });
    }

    private IModel<? extends Iterable<? extends FinalSeminar>> getSeminars() {
        return new LoadableDetachableModel<Iterable<? extends FinalSeminar>>() {
            @Override
            protected Iterable<? extends FinalSeminar> load() {
                return finalSeminarService.findAll(filterForToday(), new PageRequest(0, 20, new Sort(Sort.Direction.ASC, "startDate")));
            }
        };
    }

    private FinalSeminarService.Filter filterForToday() {
        FinalSeminarService.Filter filter = new FinalSeminarService.Filter();
        filter.setFromDate(startOfToday());
        return filter;
    }

    private Date startOfToday() {
        return new DateTime().withTimeAtStartOfDay().toDate();
    }
}
