package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.plagiarism.PlagiarismControl;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Date;

public class SeminarPanel extends Panel {

    public static final String PROJECT_TITLE = "projectTitle";
    static final String CANCELLED = "cancelled";
    static final String CRUD = "crud";
    static final String CRUD_NOT_ALLOWED = "noCrud";
    static final String PROJECT = "project";
    static final String THESIS = "thesis";
    static final String TURN_IT_IN = "turnItIn";

    @Inject
    FinalSeminarService finalSeminarService;
    @Inject
    ProjectService projectService;
    @Inject
    TurnItInCheckService turnItInService;
    @Inject
    PlagiarismControl plagiarismControl;
    @Inject
    FinalSeminarSettingsService finalSeminarSettingsService;

    private FinalSeminarSettings finalSeminarSettings;

    SeminarCancelledInfoPanel cancelledInfoPanel;

    private IModel<FinalSeminar> seminar;

    public SeminarPanel(String id, IModel<FinalSeminar> seminar) {
        super(id, seminar);
        this.seminar = seminar;

        finalSeminarSettings = finalSeminarSettingsService.getInstance();

        initComponents(seminar);
    }

    private boolean hasThesis() {
        return seminar.getObject().getDocument() != null;
    }

    private void initComponents(final IModel<FinalSeminar> seminar) {

        add(new ExternalLink("evaluation", finalSeminarSettings.getEvaluationURL(), finalSeminarSettings.getEvaluationURL()){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(seminar.getObject().getProject().isParticipant(SciProSession.get().getUser()) && seminar.getObject().getStartDate().before(new Date()) && finalSeminarSettings.getEvaluationURL() != null && !finalSeminarSettings.getEvaluationURL().isEmpty());
            }
        });

        add(new Label(PROJECT_TITLE, new PropertyModel<String>(seminar, "project.title")));
        add(new SeminarCRUDPanel(CRUD, seminar));

        add(new ProjectDetailsPanel(PROJECT, getProject()));
        add(new TurnItInPanel(TURN_IT_IN, seminar));
        add(new SeminarRespondentsPanel("testRespondents", seminar));
        add(new SeminarOppositionPanel("testOpposition", seminar));
        add(new SeminarActiveParticipantsPanel("testParticipants", seminar));
        initSeminarNonCRUDComponent(seminar);
        initThesisComponent(seminar);
        addCancelledInfo(seminar);
    }

    private IModel<Project> getProject() {
        return new LoadableDetachableModel<Project>() {
            @Override
            protected Project load() {
                return projectService.findOne(seminar.getObject().getProject().getId());
            }
        };
    }

    private void initSeminarNonCRUDComponent(final IModel<FinalSeminar> seminar) {
        add(new SeminarNonCRUDPanel(CRUD_NOT_ALLOWED, new CompoundPropertyModel<>(seminar)) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!isSupervisor(seminar));
            }
        });
    }

    private boolean isSupervisor(IModel<FinalSeminar> seminar) {
        return SciProSession.get().getUser().equals(seminar.getObject().getProject().getHeadSupervisor());
    }

    private void initThesisComponent(final IModel<FinalSeminar> seminar) {
        add(new SeminarThesisPanel(THESIS, seminar) {
            @Override
            protected void onDelete() {
                setResponsePage(getPage());
            }
        });
    }

    private void addCancelledInfo(final IModel<FinalSeminar> seminar) {
        cancelledInfoPanel = new SeminarCancelledInfoPanel(CANCELLED, seminar);
        cancelledInfoPanel.setOutputMarkupId(true);
        add(cancelledInfoPanel);
    }

}
