package se.su.dsv.scipro.finalseminar;

import com.google.common.eventbus.EventBus;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AjaxRadioChoice;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

public class SeminarRespondentsPanel extends Panel {

    public static final String RESPONDENTS = "respondents";
    public static final String USER = "user";
    public static final String RADIOS = "radios";
    public static final String FEEDBACK = "feedback";
    private final FeedbackPanel feedbackPanel;

    private IModel<FinalSeminar> seminar;

    @Inject
    private FinalSeminarRespondentService finalSeminarRespondentService;
    @Inject
    private EventBus eventBus;

    public SeminarRespondentsPanel(String id, IModel<FinalSeminar> seminar) {
        super(id, seminar);
        this.seminar = seminar;

        feedbackPanel = new FencedFeedbackPanel(FEEDBACK, this);
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        add(new ListView<FinalSeminarRespondent>(RESPONDENTS, getRespondents()) {
            @Override
            protected void populateItem(ListItem<FinalSeminarRespondent> item) {
                item.add(new UserLinkPanel(USER, new PropertyModel<User>(item.getModel(), "user")));
                item.add(new RespondentRadioChoice(RADIOS, item.getModel()));
            }
        });
    }

    private IModel<List<FinalSeminarRespondent>> getRespondents() {
        return new LoadableDetachableModel<List<FinalSeminarRespondent>>() {
            @Override
            protected List<FinalSeminarRespondent> load() {
                return finalSeminarRespondentService.findOrCreate(seminar.getObject());
            }
        };
    }

    private class RespondentRadioChoice extends AjaxRadioChoice<FinalSeminarGrade> {
        private IModel<FinalSeminarRespondent> respondent;

        public RespondentRadioChoice(String id, IModel<FinalSeminarRespondent> respondent) {
            super(id, new PropertyModel<FinalSeminarGrade>(respondent, "grade"), Arrays.asList(FinalSeminarGrade.values()),
                    new EnumChoiceRenderer<FinalSeminarGrade>(SeminarRespondentsPanel.this),
                    SeminarRespondentsPanel.this.getString("confirmation", respondent));
            this.respondent = respondent;
            setSuffix(" ");
        }

        @Override
        protected void update(AjaxRequestTarget target) {
            finalSeminarRespondentService.save(respondent.getObject());

            if (FinalSeminarGrade.APPROVED.equals(getModelObject())) {
                success(getString("feedback.respondent.approved", respondent));
                eventBus.post(new RespondentApprovedEvent(respondent.getObject()));
            } else if (FinalSeminarGrade.NOT_APPROVED.equals(getModelObject())) {
                success(getString("feedback.respondent.not.approved", respondent));
                eventBus.post(new RespondentFailedEvent(respondent.getObject()));
            }

            target.add(feedbackPanel);
        }

        @Override
        protected void onConfigure() {
            super.onConfigure();
            setVisibilityAllowed(isHeadSupervisor() && startDateHasPassed());
        }
    }

    private boolean startDateHasPassed() {
        return seminar.getObject().getStartDate().before(new Date());
    }

    private boolean isHeadSupervisor() {
        return SciProSession.get().getUser().equals(seminar.getObject().getProject().getHeadSupervisor());
    }
}
