package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightForum;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.forum.panels.threaded.ThreadsOverviewPanel;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

@Authorization(authorizedRoles = {Roles.AUTHOR})
@ProjectModuleComponent(ProjectModule.FORUM)
public class ProjectThreadedForumPage extends AbstractProjectDetailsPage implements MenuHighlightAuthorMyProjects, MenuHighlightForum {

    @Inject
    private ProjectForumService projectForumService;

    public ProjectThreadedForumPage(final PageParameters pp) {
        super(pp);

        add(new BookmarkablePageLink<Void>(CREATE_THREAD_LINK, ProjectCreateForumThreadPage.class, pp));
        add(new FeedbackPanel(FEEDBACK));
        add(new ThreadsOverviewPanel<>(FORUM, getThreads(), new ForumThreadDiscussable(),
                new ThreadsOverviewPanel.ThreadLinkSupplier<ProjectThread>() {
            @Override
            public BookmarkablePageLink<Void> newThreadLink(final String id, final IModel<ProjectThread> thread) {
                PageParameters parameters = new PageParameters(pp)
                        .set(PageParameterKeys.MAP.get(ForumThread.class), thread.getObject().getId());
                return new BookmarkablePageLink<>(id, ProjectViewForumThreadPage.class, parameters);
            }
        }));
    }

    private IModel<List<ProjectThread>> getThreads() {
        return new LoadableDetachableModel<List<ProjectThread>>() {
            @Override
            protected List<ProjectThread> load() {
                List<ProjectThread> threads = projectForumService.getThreads(projectModel.getObject());
                Collections.sort(threads, Collections.reverseOrder(new Comparator<ProjectThread>() {
                    @Override
                    public int compare(final ProjectThread o1, final ProjectThread o2) {
                        return o1.getForumThread().getLastModified().compareTo(o2.getForumThread().getLastModified());
                    }
                }));
                return threads;
            }
        };
    }

    static final String CREATE_THREAD_LINK = "createThread";
    static final String FEEDBACK = "feedback";
    static final String FORUM = "forum";
}
