package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightForum;
import se.su.dsv.scipro.exceptions.PageNotFoundException;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.forum.panels.threaded.ProjectForumThread;
import se.su.dsv.scipro.forum.panels.threaded.ViewProjectForumThreadPanel;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

@Authorization(authorizedRoles = {Roles.AUTHOR})
@ProjectModuleComponent(ProjectModule.FORUM)
public class ProjectViewForumThreadPage extends AbstractProjectDetailsPage implements MenuHighlightAuthorMyProjects, MenuHighlightForum {

    @Inject
    private ProjectForumService projectForumService;
    @Inject
    private BasicForumService basicForumService;
    @Inject
    private FileDescriptionService fileDescriptionService;

    public ProjectViewForumThreadPage(PageParameters pp) {
        super(pp);

        StringValue value = pp.get(PageParameterKeys.MAP.get(ForumThread.class));
        Long threadId = null;
        if(!value.isNull()) {
            threadId = value.toLongObject();
        }
        if (threadId == null) {
            throw new PageNotFoundException();
        }

        ProjectThread thread = projectForumService.findOne(threadId);
        if (thread == null) {
            throw new PageNotFoundException();
        }

        if (!getActiveProject().isParticipant(SciProSession.get().getUser())) {
            throw new RestartResponseException(AccessDeniedPage.class);
        }

        basicForumService.setThreadRead(SciProSession.get().getUser(), thread.getForumThread(), true);

        final Long finalThreadId = threadId;
        IModel<ProjectThread> threadModel = new LoadableDetachableModel<ProjectThread>() {
            @Override
            protected ProjectThread load() {
                return projectForumService.findOne(finalThreadId);
            }
        };
        add(new ViewProjectForumThreadPanel(FORUM_THREAD, threadModel, new ProjectForumThread(projectForumService, fileDescriptionService), projectModel));
    }

    static final String FORUM_THREAD = "forumThread";

}
