package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.Component;
import org.apache.wicket.extensions.markup.html.basic.SmartLinkMultiLineLabel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.DisplayMultiplesPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.system.User;

import java.util.Date;

public class ForumPostPanel extends Panel {

    public static final String POSTED_BY = "postedBy";
    public static final String DATE_CREATED = "dateCreated";
    public static final String CONTENT = "content";
    public static final String ATTACHMENT = "attachment";
    public static final String POST_NUMBER = "postNumber";

    public ForumPostPanel(String id, final IModel<ForumPost> model) {
        super(id);
        add(new UserLinkPanel(POSTED_BY, new PropertyModel<User>(model, POSTED_BY)));
        add(new DateLabel(DATE_CREATED, new PropertyModel<Date>(model, DATE_CREATED), DateStyle.DATETIME));
        add(new SmartLinkMultiLineLabel(CONTENT, new PropertyModel<String>(model, CONTENT)));

        add(new DisplayMultiplesPanel<FileDescription>(ATTACHMENT, new ListAdapterModel<>(new PropertyModel<Iterable<FileDescription>>(model, "attachments"))) {
            @Override
            public Component getComponent(String componentId, IModel<FileDescription> t) {
                return new ViewAttachmentPanel(componentId, t);
            }
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!model.getObject().getAttachments().isEmpty());
            }
        });

    }
}
