package se.su.dsv.scipro.grading;

import com.google.common.base.Function;
import lombok.SneakyThrows;
import net.sf.jasperreports.engine.JRException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.system.User;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

public class GradingReportPdfResourceFileUpload extends GradingReportPdfResource implements FileUpload {

    private static final Logger LOGGER = LoggerFactory.getLogger(GradingReportPdfResourceFileUpload.class);

    private final User user;

    public GradingReportPdfResourceFileUpload(GradingReportPdf gradingReportPdf, User user) {
        super(gradingReportPdf);
        this.user = user;
    }

    @Override
    public User getUploader() {
        return user;
    }

    @Override
    public long getSize() {
        try {
            return getExporterData().length;
        } catch (JRException e) {
            LOGGER.error("Couldn't get size from JasperReport", e);
        }
        return 0;
    }

    private byte[] getExporterData() throws JRException {
        return getExporterData(newJasperPrint(), newExporter());
    }

    @Override
    @SneakyThrows({IOException.class, JRException.class})
    public <T> T handleData(Function<InputStream, T> handler) {
        try (InputStream is = new ByteArrayInputStream(getExporterData())) {
            return handler.apply(is);
        }
    }
}
