package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Date;

public class ThesisApprovedPanel extends GenericPanel<Project> {
    public static final String DATE_LABEL = "dateLabel";
    public static final String REQUIRES_GRADE_REPORT_PANEL = "requiresGradePanel";
    public static final String GRADE_REPORT_FINISHED_PANEL = "gradeReportFinishedPanel";
    public static final String MESSAGE = "message";
    private static final String FILE_PANEL = "filePanel";

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private GradingReportService gradingReportService;

    public ThesisApprovedPanel(String id, final IModel<Project> project) {
        super(id, project);
        add(new ViewAttachmentPanel(FILE_PANEL, getFileDescription(), DateStyle.DATETIME));
        add(new RequiresGradeReportPanel(REQUIRES_GRADE_REPORT_PANEL));
        add(new GradeReportFinishedPanel(GRADE_REPORT_FINISHED_PANEL));
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(gradingReportTemplateIsAvailable());
    }

    private boolean gradingReportTemplateIsAvailable() {
        return gradingReportService.getTemplate(getModelObject()) != null;
    }

    private class RequiresGradeReportPanel extends Panel {
        public RequiresGradeReportPanel(String id) {
            super(id);

            add(new DateLabel(DATE_LABEL, new PropertyModel<Date>(getFinalThesis(), "dateApproved")));
            Label approvedLabel = new Label(MESSAGE, new ResourceModel("approved"));
            if (getModelObject().isReviewer(SciProSession.get().getUser())) {
                approvedLabel.setDefaultModel(new ResourceModel("reviewer.approved"));
            }
            add(approvedLabel);
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(gradingReportTemplateIsAvailable() && !gradingReportService.hasFinishedGradingReports(getModelObject(), !getModelObject().isSupervisor(SciProSession.get().getUser())));
        }
    }

    private LoadableDetachableModel<FileDescription> getFileDescription() {
        return new LoadableDetachableModel<FileDescription>() {
            @Override
            protected FileDescription load() {
                return finalThesisService.getFinalThesisFileDescription(getModelObject());
            }
        };
    }

    private class GradeReportFinishedPanel extends Panel {

        public GradeReportFinishedPanel(String id) {
            super(id);
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(gradingReportTemplateIsAvailable() && gradingReportService.hasFinishedGradingReports(getModelObject(), !getModelObject().isSupervisor(SciProSession.get().getUser())));
        }
    }

    private LoadableDetachableModel<FinalThesis> getFinalThesis() {
        return new LoadableDetachableModel<FinalThesis>() {
            @Override
            protected FinalThesis load() {
                return finalThesisService.findByProject(getModelObject());
            }
        };
    }
}