package se.su.dsv.scipro.group;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.cookies.CookieUtils;
import org.apache.wicket.util.string.StringValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyGroups;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.pages.ProjectPage;
import se.su.dsv.scipro.security.auth.SystemModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import javax.servlet.http.Cookie;

@SystemModuleComponent(SystemModule.GROUP)
public abstract class AbstractAuthorGroupPage extends ProjectPage implements MenuHighlightAuthorMyGroups {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractAuthorGroupPage.class);
    public static final String COOKIE_KEY = "authorLastVisitedGID";
    public static final int MAX_AGE = 2592000;

    @Inject
    protected GroupService groupService;
    @Inject
    protected GroupFacade groupFacade;

    private final IModel<Group> groupModel;

    public AbstractAuthorGroupPage(PageParameters pp) {
        super(pp);
        groupModel = new DetachableServiceModel<>(groupService);
        setDefaultModel(groupModel);
        initializeGroup(pp);

        add(new GroupDetailsPanel("groupDetails", groupModel));
    }

    protected IModel<Group> getGroup() {
        return groupModel;
    }

    private void initializeGroup(PageParameters pp) {
        StringValue value = pp.get(PageParameterKeys.MAP.get(Group.class));
        Long groupId = null;
        if(!value.isNull()) {
            groupId = value.toLongObject();
        }
        if (groupId == null) {
            try {
                String cookieValue = new CookieUtils().load(COOKIE_KEY);
                groupId = Long.parseLong(cookieValue);
            } catch (NumberFormatException e) {/* Ignore */
                LOGGER.error("Couldnt initialize group", e);
            }
        } else {
            /*
             * Cookie is set when the link to visit this details page is clicked but for it to work
			 * if a URL is entered directly we set it here too
			 */
            Cookie c = new Cookie(COOKIE_KEY, groupId.toString());
            c.setMaxAge(MAX_AGE);
            c.setPath("/");
            ((WebResponse) getRequestCycle().getResponse()).addCookie(c);
        }

        if (groupId == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        Group group = groupService.findOne(groupId);

        if (group == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        if (!group.isAuthor(SciProSession.get().getUser())) {
            throw new RestartResponseException(AccessDeniedPage.class);
        } else {
            groupModel.setObject(group);
        }
    }
}
