package se.su.dsv.scipro.group;

import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyGroups;
import se.su.dsv.scipro.forum.GroupForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.GroupThread;
import se.su.dsv.scipro.forum.panels.threaded.ThreadsOverviewPanel;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class AuthorGroupPage extends AbstractAuthorGroupPage implements MenuHighlightAuthorMyGroups {

    @Inject
    private GroupForumService groupForumService;

    public AuthorGroupPage(final PageParameters pp) {
        super(pp);
        add(new FeedbackPanel("feedback"));
        add(new BookmarkablePageLink<Void>("createThread", CreateThreadPage.class, pp));
        add(new ThreadsOverviewPanel<>("threads", getThreads(), new GroupThreadDiscussable(),
                new ThreadsOverviewPanel.ThreadLinkSupplier<GroupThread>() {
            @Override
            public BookmarkablePageLink<Void> newThreadLink(final String id, final IModel<GroupThread> thread) {
                PageParameters parameters = new PageParameters(pp)
                        .set(PageParameterKeys.MAP.get(ForumThread.class), thread.getObject().getId());
                return new BookmarkablePageLink<>(id, ViewThreadPage.class, parameters);
            }
        }));
    }

    private IModel<List<GroupThread>> getThreads() {
        return new LoadableDetachableModel<List<GroupThread>>() {
            @Override
            protected List<GroupThread> load() {
                List<GroupThread> threads = groupForumService.getThreads(getGroup().getObject());
                Collections.sort(threads, Collections.reverseOrder(new Comparator<GroupThread>() {
                    @Override
                    public int compare(final GroupThread o1, final GroupThread o2) {
                        return o1.getForumThread().getLastModified().compareTo(o2.getForumThread().getLastModified());
                    }
                }));
                return threads;
            }
        };
    }
}
