package se.su.dsv.scipro.group;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.Option;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.notifications.dataobject.GroupEvent;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryMethod;
import se.su.dsv.scipro.notifications.settings.service.DeliveryConfigurationService;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationService;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.SupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Set;

public class ViewGroupProjectsPanel extends GenericPanel<Group> {

    @Inject
    ReceiverConfigurationService receiverConfigurationService;
    @Inject
    DeliveryConfigurationService deliveryConfigurationService;

    public ViewGroupProjectsPanel(String id, final IModel<Group> model) {
        super(id, model);

        add(new Label("authorReceiving", mark(isReceiving(Member.Type.AUTHOR))));
        add(new Label("headSupervisorReceiving", mark(isReceiving(Member.Type.SUPERVISOR))));
        add(new Label("coSupervisorReceiving", mark(isReceiving(Member.Type.CO_SUPERVISOR))));
        add(new Label("reviewerReceiving", mark(isReceiving(Member.Type.REVIEWER))));

        add(new ListView<Project>("projects", new ListAdapterModel<>(new PropertyModel<Set<Project>>(model, "projects"))) {
            @Override
            protected void populateItem(final ListItem<Project> item) {
                item.add(new Label("type", new PropertyModel<>(item.getModel(), "projectTypeName")));
                item.add(projectLink(item.getModel()));
                item.add(new ListView<User>("authors", new ListAdapterModel<>(new PropertyModel<Set<User>>(item.getModel(), "projectParticipants"))) {
                    @Override
                    public void populateItem(ListItem<User> item) {
                        item.add(new UserLinkPanel("author", item.getModel()));
                    }
                });
                item.add(new UserLinkPanel("headSupervisor", new PropertyModel<User>(item.getModel(), "headSupervisor")));
                item.add(new ListView<User>("coSupervisors", new ListAdapterModel<>(new PropertyModel<Set<User>>(item.getModel(), "coSupervisors"))) {
                    @Override
                    public void populateItem(ListItem<User> item) {
                        item.add(new UserLinkPanel("coSupervisor", item.getModel()));
                    }
                });
                item.add(new UserLinkPanel("reviewer", new PropertyModel<User>(item.getModel(), "reviewer")));
            }
        });

        add(new Label("noProjects", "None") {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(ViewGroupProjectsPanel.this.getModelObject().getProjects() == null || ViewGroupProjectsPanel.this.getModelObject().getProjects().isEmpty());
            }
        });
    }

    private BookmarkablePageLink<Object> projectLink(IModel<Project> model) {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), model.getObject().getId());
        BookmarkablePageLink<Object> link = new BookmarkablePageLink<>("titleLink", SupervisorProjectDetailsPage.class, pp);
        link.setEnabled(allowedAccess(model));
        link.add(new Label("title", new PropertyModel<>(model, "title")));
        return link;
    }

    private boolean allowedAccess(IModel<Project> model) {
        Project project = model.getObject();
        User loggedIn = SciProSession.get().getUser();
        return project.isSupervisor(loggedIn)
                || project.isCoSupervisor(loggedIn)
                || project.isReviewer(loggedIn);
    }

    private boolean isReceiving(final Member.Type member){
        return deliveryConfigurationService.isDelivery(Notification.Type.GROUP, GroupEvent.Event.MESSAGE_THREAD_CREATED, DeliveryMethod.WEB, Option.<User>empty()) &&
                deliveryConfigurationService.isDelivery(Notification.Type.GROUP, GroupEvent.Event.MESSAGE_THREAD_REPLY, DeliveryMethod.WEB, Option.<User>empty()) &&
                receiverConfigurationService.isReceiving(Notification.Type.GROUP, GroupEvent.Event.MESSAGE_THREAD_CREATED, member) &&
                receiverConfigurationService.isReceiving(Notification.Type.GROUP, GroupEvent.Event.MESSAGE_THREAD_REPLY, member);
    }

    private String mark(final boolean boo){
        return boo ? "*" : "";
    }
}

