package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.Url;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.EmailAddressValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.HashService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;

public class ForgotPasswordPage extends PublicPage {

    private static final Logger LOGGER = LoggerFactory.getLogger(ForgotPasswordPage.class);
    @Inject
    private UserService userService;
    @Inject
    private HashService hashService;
    @Inject
    private MailEventService mailEventService;

    public ForgotPasswordPage() {
        add(new ForgotPasswordForm("form", Model.of("")));
	}

    private class ForgotPasswordForm extends StatelessForm<String> {
        private RequiredTextField<String> email;

        public ForgotPasswordForm(String id, IModel<String> model) {
            super(id, model);

            email = new RequiredTextField<>("email", getModel());
            email.add(EmailAddressValidator.getInstance());
            add(email);
            add(new ComponentFeedbackPanel("emailFeedback", email));

            add(new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            }));
        }

        @Override
        protected void onSubmit() {
            try {
                User user = userService.findByEmailAddress(getModelObject());
                Hash hash = hashService.createForUser(user);
                sendResetPasswordMail(user, hash);
                success(getString("instructions.sent"));
            } catch (Exception e){
                success(getString("instructions.sent"));
                LOGGER.info("Problem with sending reset-password-mail to email-address: " + email, e);
            }
        }
    }

    private void sendResetPasswordMail(final User user, final Hash hash) {
        final PageParameters pp = new PageParameters();
        pp.add("hash", hash.getHash());
        final String resetPasswordURL = RequestCycle.get().getUrlRenderer().renderFullUrl(Url.parse(urlFor(SetPasswordPage.class, pp)));
        final String subject = getString("reset.password.subject");
        final String body = getString("reset.password.content") + " " + resetPasswordURL;

        final MailEvent mailEvent = new MailEvent();
        mailEvent.getRecipients().add(user);
        mailEvent.setSubject(subject);
        mailEvent.setMessageBody(body);

        mailEventService.save(mailEvent);
    }
}
