package se.su.dsv.scipro.loginlogout.panels;

import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.loginlogout.pages.ForgotPasswordPage;
import se.su.dsv.scipro.security.auth.AuthenticationService;
import se.su.dsv.scipro.security.auth.Credentials;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;

public class LoginPanel extends Panel {

    @Inject
    private AuthenticationService authenticationService;
    @Inject
    private UserService userService;

    public LoginPanel(final String id) {
        super(id);
        add(new FeedbackPanel("feedback"));
        add(new SignInForm("signInForm"));
    }

    public final class SignInForm extends StatelessForm<Void> {
        private IModel<String> usernameModel = new Model<>();
        private IModel<String> passwordModel = new Model<>();

        public SignInForm(final String id) {
            super(id);
            add(new RequiredTextField<>("username", usernameModel, String.class));
            add(new PasswordTextField("password", passwordModel));
            add(new BookmarkablePageLink<>("forgotPassword", ForgotPasswordPage.class));
        }

        @Override
        public void onSubmit() {
            String username = usernameModel.getObject();
            String password = passwordModel.getObject();
            Credentials credentials = new Credentials(username, password.toCharArray());

            if (authenticationService.authenticate(credentials)) {
                SciProSession.get().setUser(userService.findByUsername(username));
                setResponsePage(getApplication().getHomePage());
                continueToOriginalDestination();
            }
            else {
                error(getString("loginFailed"));
            }
        }
    }
}