package se.su.dsv.scipro.match;

import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.datatables.UserColumn;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AdminIdeaHistoryPanel extends Panel {

    public static final String DATA_TABLE = "dataTable";
    @Inject
	private MatchService matchService;

    public AdminIdeaHistoryPanel(String id, IModel<Idea> model) {
		super(id);
		addDataTable(model);
	}

	private void addDataTable(final IModel<Idea> model) {
        SortableDataProvider<Match, String> provider = new SortableDataProvider<Match, String>() {
            @Override
            public Iterator<? extends Match> iterator(long first, long count) {
                return matchService.getMatchHistory(model.getObject(), new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return matchService.countMatchHistory(model.getObject());
            }

            @Override
            public IModel<Match> model(Match object) {
                return new DetachableServiceModel<>(matchService, object);
            }
        };
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider));
	}

    private List<IColumn<Match, String>> createColumns() {
        List<IColumn<Match,String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Match>(Model.of("Date"), "dateCreated", "dateCreated", DateStyle.DATETIME));
        columns.add(new PropertyColumn<Match, String>(Model.of("Status"), "status", "status"));
        columns.add(new AbstractColumn<Match, String>(Model.of("Supervisor"), "supervisor.firstName") {
            @Override
            public void populateItem(Item<ICellPopulator<Match>> item, String componentId, IModel<Match> rowModel) {
                if (rowModel.getObject().getSupervisor() != null) {
                    item.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "supervisor")));
                } else {
                    item.add(new Label(componentId, "None"));
                }

            }
        });
        columns.add(new UserColumn<Match, String>(Model.of("Change made by")) {
            @Override
            protected IModel<User> getUser(final IModel<Match> rowModel) {
                return new PropertyModel<>(rowModel, "changedBy");
            }
        });
        return columns;
    }
}
