package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.statistics.AbstractAdminStatisticsPage;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

@Authorization(authorizedRoles={Roles.ADMIN})
public class AdminIdeaStatusStatisticsPage extends AbstractAdminStatisticsPage {

    @Inject
    private IdeaService ideaService;
    @Inject
    private ProjectTypeService projectTypeService;

	public AdminIdeaStatusStatisticsPage() {
        setUpStatusList();
	}

    private void setUpStatusList() {
        List<Idea.Status> statuses = Arrays.asList(Idea.Status.values());
        add(new PropertyListView<ProjectType>("projectTypeHeaders", getProjectTypes()) {
            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                item.add(new Label("name"));
            }
        });
        ListView<Idea.Status> listView = new ListView<Idea.Status>("statusList", statuses) {

            @Override
            protected void populateItem(final ListItem<Idea.Status> item) {
                item.add(new Label("status", item.getModelObject().toString()));
                item.add(new Label("total", String.valueOf(ideaService.countIdeas(item.getModelObject(), null))));
                item.add(new ListView<ProjectType>("projectTypes", getProjectTypes()) {
                    @Override
                    protected void populateItem(ListItem<ProjectType> projectType) {
                        projectType.add(new Label("value",
                                String.valueOf(ideaService.countIdeas(item.getModelObject(), projectType.getModelObject()))));
                    }
                });

            }
        };
        add(listView);
    }

    private IModel<List<ProjectType>> getProjectTypes() {
        return new LoadableDetachableModel<List<ProjectType>>() {
            @Override
            protected List<ProjectType> load() {
                return projectTypeService.findWithModule(ProjectModule.MATCH);
            }
        };
    }

}
