package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.KeywordAutoComplete;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.List;

public abstract class AdminResearchAreaEditPanel extends GenericPanel<ResearchArea> {

    @Inject
	private KeywordService keywordService;

	public AdminResearchAreaEditPanel(String id, IModel<ResearchArea> model) {
		super(id, model);
		FeedbackPanel feedback = new FeedbackPanel("feedback");
		feedback.setOutputMarkupId(true);
		add(feedback);
		WebMarkupContainer container = new WebMarkupContainer("container");
		container.setOutputMarkupId(true);
		add(container);

		addInputField(container, feedback);
		addKeywordList(container, feedback);
	}

	private void addInputField(final WebMarkupContainer container, final FeedbackPanel feedback) {
        KeywordAutoComplete autoComplete = new KeywordAutoComplete("inputField") {
            @Override
            protected void action(AjaxRequestTarget pTarget, Keyword newSelection) {
                keywordService.addConnection(newSelection, AdminResearchAreaEditPanel.this.getModelObject());
                info("Relation added, Keyword: " + newSelection.getKeyword() + ", Research area: "+
                        AdminResearchAreaEditPanel.this.getModelObject().getTitle());
                pTarget.add(feedback);
                pTarget.add(container);
                updateTarget(pTarget);
            }
        };
        container.add(autoComplete);
	}

	private void addKeywordList(final WebMarkupContainer container, final FeedbackPanel feedback) {
		container.add(new ListView<Keyword>("listView", getKeywords()){
            @Override
			protected void populateItem(ListItem<Keyword> item) {
				final Keyword keyword = item.getModelObject();
				item.add(new Label("keywordName", new PropertyModel<>(keyword, "keyword")));
				item.add(new AjaxConfirmationLink<Void>("removeLink", "Are you sure you want to delete the relationship to "+keyword.getKeyword()+"?") {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
						keywordService.deleteConnection(keyword, AdminResearchAreaEditPanel.this.getModelObject());
						info("Relationship between Keyword: "+ keyword.getKeyword() + " and Research area: " +
                                AdminResearchAreaEditPanel.this.getModelObject().getTitle()+ " removed");
						target.add(feedback);
						target.add(container);
						updateTarget(target);
					}
                });
			}
		});
	}

    private IModel<List<Keyword>> getKeywords() {
        return new LoadableDetachableModel<List<Keyword>>() {
            @Override
            protected List<Keyword> load() {
                return keywordService.findByResearchArea(getModelObject());
            }
        };
    }

	protected abstract void updateTarget(AjaxRequestTarget target);
}
