package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.system.Program;

import javax.inject.Inject;

public abstract class EditProgramPanel extends Panel {

    @Inject
    private ProgramService programService;

    public abstract void onSave(AjaxRequestTarget target);

    public EditProgramPanel(String id, final IModel<Program> model) {
        super(id, model);
        add(new ProgramForm("form", model));
    }

    private class ProgramForm extends Form<Program> {

        public ProgramForm(String id, IModel<Program> model) {
            super(id, model);

            final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);

            RequiredTextField<String> name = new RequiredTextField<>("name", new PropertyModel<String>(model, "name"));
            wmc.add(name);
            wmc.add(new ComponentFeedbackPanel("nameFeedback", name));

            RequiredTextField<String> nameEn = new RequiredTextField<>("nameEn", new PropertyModel<String>(model, "nameEn"));
            wmc.add(nameEn);
            wmc.add(new ComponentFeedbackPanel("nameEnFeedback", nameEn));

            RequiredTextField<String> code = new RequiredTextField<>("code", new PropertyModel<String>(model, "code"));
            wmc.add(code);
            wmc.add(new ComponentFeedbackPanel("codeFeedback", code));

            FeedbackPanel feedback = new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            });
            feedback.setOutputMarkupId(true);
            wmc.add(feedback);

            add(wmc);

            add(new AjaxButton("save") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    super.onSubmit(target, form);
                    ProgramForm.this.setModelObject(programService.save(ProgramForm.this.getModelObject()));
                    info("Saved");
                    target.add(wmc);
                    onSave(target);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    super.onError(target, form);
                    target.add(wmc);
                }
            });
        }
    }
}