package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.EmailTextField;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.validation.validator.EmailAddressValidator;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.system.Unit;

import javax.inject.Inject;

public abstract class EditUnitPanel extends Panel {

    @Inject
    private UnitService unitService;

    public abstract void onSave(AjaxRequestTarget target);

    public EditUnitPanel(String id, final IModel<Unit> model) {
        super(id, model);
        add(new UnitForm("form", model));
    }

    private class UnitForm extends Form<Unit> {

        public UnitForm(String id, final IModel<Unit> model) {
            super(id, model);

            final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);

            RequiredTextField<String> unit = new RequiredTextField<>("unit", new PropertyModel<String>(model, "title"));
            wmc.add(unit);
            wmc.add(new ComponentFeedbackPanel("unitFeedback", unit));

            FormComponent<String> matchResponsible = new EmailTextField("matchResponsible", new PropertyModel<String>(model, "matchResponsible"));
            matchResponsible.add(EmailAddressValidator.getInstance());
            wmc.add(matchResponsible);
            wmc.add(new ComponentFeedbackPanel("matchResponsibleFeedback", matchResponsible));

            FeedbackPanel feedback = new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            });
            feedback.setOutputMarkupId(true);
            wmc.add(feedback);

            add(wmc);

            add(new AjaxButton("save") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    super.onSubmit(target, form);
                    UnitForm.this.setModelObject(unitService.save(UnitForm.this.getModelObject()));
                    info("Saved");
                    target.add(wmc);
                    onSave(target);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    super.onError(target, form);
                    target.add(wmc);
                }
            });
        }
    }
}