package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.extensions.markup.html.form.palette.Palette;
import org.apache.wicket.extensions.markup.html.form.palette.component.Recorder;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.ResourceReference;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class KeywordFilterPanel extends GenericPanel<Set<Keyword>> {

    public static final String PALETTE = "palette";
    public static final String FORM = "form";
    private static final int DEFAULT_ROWS = 15;
    private List<Keyword> selection = new ArrayList<>();

    public KeywordFilterPanel(String id, IModel<Set<Keyword>> model) {
        this(id, model, null);
    }

    public KeywordFilterPanel(String id, IModel<Set<Keyword>> selectable, IModel<List<Keyword>> selected) {
        super(id, selectable);

        if (selected != null && selected.getObject() != null) {
            this.selection = selected.getObject();
        }

        add(new PaletteForm(FORM, selectable, new PropertyModel<List<Keyword>>(this, "selection")));
    }

    public List<Keyword> getSelection() {
        return this.selection;
    }

    protected void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> keywords) {

    }

    private class PaletteForm extends Form<Set<Keyword>> {

        public PaletteForm(String id, IModel<Set<Keyword>> model, IModel<List<Keyword>> selection) {
            super(id, model);
            add(getPalette(model, selection));
        }

        private Palette<Keyword> getPalette(IModel<Set<Keyword>> model, IModel<List<Keyword>> selection) {
            return new Palette<Keyword>(PALETTE, selection, model, getRenderer(), DEFAULT_ROWS, false) {
                @Override
                protected Recorder<Keyword> newRecorderComponent() {
                    Recorder<Keyword> recorder = super.newRecorderComponent();
                    recorder.add(new AjaxFormComponentUpdatingBehavior("change") {
                        @Override
                        protected void onUpdate(AjaxRequestTarget target) {
                            onKeywordsSelected(target, new HashSet<>(getSelection()));
                        }
                    });
                    return recorder;
                }

                @Override
                protected ResourceReference getCSS() {
                    return new CssResourceReference(KeywordFilterPanel.class,
                            "custompalette.css");
                }
            };
        }

        private IChoiceRenderer<Keyword> getRenderer() {
            return new ChoiceRenderer<>("keyword", "id");
        }

    }

}