package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.firstmeeting.FirstMeetingColumnPanel;
import se.su.dsv.scipro.firstmeeting.FirstMeetingPanel;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.profile.StudentProfileProgramPanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class ProjectMyIdeasPanel extends Panel {

    public static final String DATA_TABLE = "dataTable";
    @Inject
    private IdeaService ideaService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;

    private ModalWindowPlus modalWindowPlus;

    public ProjectMyIdeasPanel(String id, User author) {
        super(id);
        addDialog();
        addDataTable(author);
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("modalWindowPlus");
        add(modalWindowPlus);
    }

    private void addDataTable(final User author) {
        SortableDataProvider<Idea, String> provider = new SortableDataProvider<Idea, String>() {
            @Override
            public Iterator<? extends Idea> iterator(long first, long count) {
                return ideaService.findAuthorIdeas(author, new PageRequest(first, count,  new Sort(Sort.Direction.DESC, "dateCreated"))).iterator();
            }

            @Override
            public long size() {
                return ideaService.countAuthorIdeas(author);
            }

            @Override
            public IModel<Idea> model(Idea object) {
                return new DetachableServiceModel<>(ideaService, object);
            }
        };
        add(new ExportableDataPanel<>(DATA_TABLE, createColumns(), provider));
    }

    private List<IColumn<Idea, String>> createColumns() {

        List<IColumn<Idea, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"));

        columns.add(new AbstractColumn<Idea, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, final IModel<Idea> model) {
                cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                    @Override
                    public AbstractLink apply(final String linkId) {
                        PageParameters pageParameters = new PageParameters();
                        pageParameters.set(PageParameterKeys.MAP.get(Idea.class), model.getObject().getId());
                        AbstractLink link = new BookmarkablePageLink<>(linkId, ProjectIdeaDetailsPage.class, pageParameters);
                        link.setBody(new PropertyModel<>(model, "title"));
                        return link;
                    }
                }));
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Authors")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<IdeaParticipation>(componentId, new ListAdapterModel<>(new PropertyModel<Set<IdeaParticipation>>(rowModel, "ideaParticipations"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<IdeaParticipation> t) {
                        return new StudentProfileProgramPanel(componentId, t, false);
                    }
                });
            }
        });
        columns.add(new AbstractColumn<Idea, String>(Model.of("Status"), "match.status") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                item.add(new StudentIdeaStatusColumnPanel(id, model));
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Supervisor")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                Idea idea = model.getObject();
                if ((idea.getType().equals(Type.SUPERVISOR) || applicationPeriodService.courseStartHasPassed(idea.getApplicationPeriod())) && idea.getMatch().getSupervisor() != null) {
                    item.add(new Label(id, idea.getMatch().getSupervisor().getFullName()));
                } else {
                    item.add(new Label(id, "Visible on course start date"));
                }
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("First meeting")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                item.add(new FirstMeetingColumnPanel(id, model) {
                    @Override
                    protected void onColumnClick(AjaxRequestTarget target, IModel<Idea> model) {
                        openFirstMeetingDialog(target, model);
                    }
                });
            }
        });

        return columns;
    }


    private void openFirstMeetingDialog(AjaxRequestTarget target, IModel<Idea> model) {
        modalWindowPlus.setContent(new FirstMeetingPanel(modalWindowPlus.getContentId(), model));
        modalWindowPlus.setTitle("Idea title: " + model.getObject().getTitle());
        modalWindowPlus.show(target);
    }

}
