package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.match.Idea.Status;

import javax.inject.Inject;

/**
 * 
 * Panel to display in Student datatables where the status of a submitted idea is needed.
 *
 */
public class StudentIdeaStatusColumnPanel extends Panel {

    public static final String LABEL = "label";
    @Inject
	private ApplicationPeriodService periodService;
    @Inject
    private DateService dateService;

	public StudentIdeaStatusColumnPanel(String id, IModel<Idea> ideaModel) {
		super(id, ideaModel);
		Idea idea = ideaModel.getObject();
		Status currentStatus = idea.getMatchStatus();
		switch (currentStatus) {
			case UNMATCHED:
				add(new Label(LABEL, "Unmatched, waiting for supervisor"));
				break;
			case COMPLETED:
				add(new Label(LABEL, "Matched, project started"));
				break;
            case INACTIVE:
                add(new Label(LABEL, "Inactive"));
                break;
			case MATCHED:
                if(periodService.courseStartHasPassed(idea.getApplicationPeriod())){
					add(new Label(LABEL, "Matched, course started, awaiting project creation"));
				} else {
					add(new Label(LABEL, "Matched, course starting " + dateService.format(idea.getApplicationPeriod().getCourseStartDate(), DateStyle.DATE)));
				}
				break;
            default:
                throw new IllegalStateException("Incorrect status");
		}
	}

}
