package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;

public class SupervisorAcceptIdeaDialogPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private TargetService targetService;

    public SupervisorAcceptIdeaDialogPanel(String id, IModel<Idea> model) {
        super(id, model);
        FeedbackPanel feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        add(new AcceptIdeaForm("form", model, feedback));
    }


    private class AcceptIdeaForm extends Form<Idea> {


        public AcceptIdeaForm(String id, final IModel<Idea> model, FeedbackPanel feedback) {
            super(id, model);
            add(new IdeaAuthorInfoPanel("authors", model));
            add(new IdeaDetailsPanel("ideaDetails", model, false));
            add(new IdeaWatsonInfoPanel("watson", model));
            addAcceptButton(model, feedback);
        }

        private void addAcceptButton(final IModel<Idea> model, final FeedbackPanel feedback) {
            AjaxConfirmationLink<Void> accept = new AjaxConfirmationLink<Void>("accept", "Are you sure you want to accept to supervise this idea?") {

                @Override
                public void onClick(AjaxRequestTarget target) {
                    User supervisor = SciProSession.get().getUser();
                    boolean hasTargetsLeft = targetService.hasTargetsLeft(model.getObject().getApplicationPeriod(), supervisor, model.getObject().getProjectType());
                    Pair<Boolean, String> acceptance = ideaService.validateSupervisorAcceptance(model.getObject(), hasTargetsLeft);
                    if (acceptance.getHead()) {
                        ideaService.confirmSupervision(model.getObject(), supervisor);
                        getSession().info(acceptance.getTail());
                        setResponsePage(SupervisorIdeaStartPage.class);
                    } else {
                        error(acceptance.getTail());
                        target.add(feedback);
                    }
                }
            };
            add(accept);
        }
    }
}
