package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;
import se.su.dsv.scipro.system.EventService;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Arrays;

public class EditMilestoneActivityPanel extends Panel {

    static final String EDIT_FORM = "editForm";
    static final String FEEDBACK = "feedback";
    static final String TITLE = "title";
    static final String DESCRIPTION = "description";
    static final String PROJECT_TYPES = "projectTypes";
    static final String EDITABLE_BY_AUTHORS = "editableByAuthors";
    static final String TYPE = "type";
    static final String MILE_STONE_PHASE_TEMPLATE = "milestonePhaseTemplate";
    static final String SAVE = "save";
    static final String SORT_ORDER = "sortOrder";
    static final String EVENT = "activatedBy";

    @Inject
    private MilestoneActivityTemplateService activityService;
    @Inject
    private MilestonePhaseTemplateService phaseService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private EventService eventService;

    private FeedbackPanel feedback;

    public EditMilestoneActivityPanel(String id, IModel<MilestoneActivityTemplate> model) {
        super(id);

        feedback = new FeedbackPanel(FEEDBACK);
        add(feedback.setOutputMarkupId(true));

        add(new EditForm(EDIT_FORM, new CompoundPropertyModel<>(model)));

    }

    private class EditForm extends Form<MilestoneActivityTemplate> {

        private MilestonePhaseTemplate oldPhase;

        public EditForm(String id, IModel<MilestoneActivityTemplate> model) {
            super(id, model);

            oldPhase = model.getObject().getMilestonePhaseTemplate();

            add(new RequiredTextField<String>(TITLE));
            add(new TextArea<String>(DESCRIPTION).setRequired(true));
            add(new RadioChoice<>(TYPE, Arrays.asList(MilestoneActivityTemplate.Type.values()))
                    .setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER)
                    .setRequired(true));
            add(new CheckBoxMultipleChoice<>(PROJECT_TYPES, projectTypeService.findAllActive(), new ChoiceRenderer<ProjectType>("name", "id"))
                    .setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER)
                    .setRequired(true));
            add(new CheckBox(EDITABLE_BY_AUTHORS, new PropertyModel<Boolean>(model, "editableByStudents")));
            add(new DropDownChoice<>(MILE_STONE_PHASE_TEMPLATE, phaseService.findAll(new Sort(Sort.Direction.ASC, SORT_ORDER))).setRequired(true));
            add(new DropDownChoice<>(EVENT, eventService.findAll(), new ChoiceRenderer<>("description", "name")).setNullValid(true));
            add(new AjaxSubmitLink(SAVE) {

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedback);
                }

                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    activitySaved(target);
                }
            });
        }

        @Override
        protected void onSubmit() {
            activityService.save(getModelObject(), oldPhase);
        }
    }

    protected void activitySaved(AjaxRequestTarget target) {
    }
}
