package se.su.dsv.scipro.milestones;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.components.DisplayMultiplesPanel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.components.SortOrderPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.List;

public class MileStoneActivityAdminPanel extends Panel {

    static final String LIST_CONTAINER = "listContainer";
    static final String FEEDBACK = "feedback";
    static final String DIALOG = "dialog";
    static final String CREATE_LINK = "createLink";
    static final String PHASE_LIST = "phaseList";
    static final String TITLE = "title";
    static final String MILESTONE_LIST = "milestoneList";
    static final String MILESTONE_TITLE = "milestoneTitle";
    static final String TYPE = "type";
    static final String PROJECT_TYPES = "projectTypes";
    static final String EDIT = "edit";

    static final String STATE_SWITCH = "stateSwitch";
    static final String MOVE = "move";
    static final String SORT_ORDER = "sortOrder";

    @Inject
    private MilestonePhaseTemplateService phaseService;
    @Inject
    private MilestoneActivityTemplateService activityService;

    private final WebMarkupContainer container;
    private final ModalWindowPlus dialog;
    private final FeedbackPanel feedback;

    public MileStoneActivityAdminPanel(String id) {
        super(id);

        this.container = new WebMarkupContainer(LIST_CONTAINER);
        this.feedback = new FeedbackPanel(FEEDBACK);
        this.dialog = new ModalWindowPlus(DIALOG);

        add(container.setOutputMarkupId(true));
        add(feedback.setOutputMarkupId(true));
        add(dialog);
        addCreateNewLink();
        addPhaseList();
    }

    private void addCreateNewLink() {
        add(new AjaxLink<MilestoneActivityTemplate>(CREATE_LINK) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                openDialog(target, Model.of(new MilestoneActivityTemplate()));
            }
        });

    }

    private void addPhaseList() {
        container.add(new ListView<MilestonePhaseTemplate>(PHASE_LIST, getPhaseListModel()) {
            @Override
            protected void populateItem(ListItem<MilestonePhaseTemplate> item) {
                item.add(new Label(TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
                addMileStoneList(item);
            }
        });
    }

    private void addMileStoneList(ListItem<MilestonePhaseTemplate> item) {
        item.add(new ListView<MilestoneActivityTemplate>(MILESTONE_LIST, getActivityListModel(item.getModel())) {

            @Override
            protected void populateItem(ListItem<MilestoneActivityTemplate> item) {
                item.add(new SortOrderPanel<>(MOVE, item.getModel(), activityService));
                item.add(new Label(MILESTONE_TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
                item.add(new Label(TYPE, new PropertyModel<String>(item.getModel(), TYPE)));
                item.add(new DisplayMultiplesPanel<ProjectType>(PROJECT_TYPES, new ListAdapterModel<>(new PropertyModel<Iterable<ProjectType>>(item.getModel(), "projectTypes"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<ProjectType> model) {
                        return new Label(componentId, new PropertyModel<>(model, "name"));
                    }
                });
                item.add(newEditLink(EDIT, item.getModel()));
                item.add(newActiveCheckBox(STATE_SWITCH, item.getModel()));
            }
        });
    }

    private AjaxFallbackLink<MilestoneActivityTemplate> newEditLink(String id, final IModel<MilestoneActivityTemplate> phaseModel) {
        return new AjaxFallbackLink<MilestoneActivityTemplate>(id, phaseModel) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                openDialog(target, getModel());
            }
        };
    }

    private AjaxCheckBox newActiveCheckBox(String id, final IModel<MilestoneActivityTemplate> activityModel) {
        return new AjaxCheckBox(id, Model.of(!activityModel.getObject().isDeleted())) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                activityModel.getObject().setDeleted(!getModelObject());
                activityService.save(activityModel.getObject(), activityModel.getObject().getMilestonePhaseTemplate());
                target.add(container);
            }
        };
    }

    private LoadableDetachableModel<List<MilestonePhaseTemplate>> getPhaseListModel() {
        return new LoadableDetachableModel<List<MilestonePhaseTemplate>>() {
            @Override
            protected List<MilestonePhaseTemplate> load() {
                return phaseService.findAll(new Sort(Sort.Direction.ASC, SORT_ORDER));
            }
        };
    }

    private LoadableDetachableModel<List<MilestoneActivityTemplate>> getActivityListModel(final IModel<MilestonePhaseTemplate> phaseModel) {
        return new LoadableDetachableModel<List<MilestoneActivityTemplate>>() {
            @Override
            protected List<MilestoneActivityTemplate> load() {
                return activityService.findByMileStonePhase(phaseModel.getObject());
            }
        };
    }

    private void openDialog(AjaxRequestTarget target, IModel<MilestoneActivityTemplate> model) {
        dialog.setTitle(model.getObject().getTitle() == null ? "New milestone" : "Edit milestone");
        dialog.setContent(new EditMilestoneActivityPanel(dialog.getContentId(), model) {
            @Override
            protected void activitySaved(AjaxRequestTarget target) {
                info("Milestone saved");
                target.add(feedback);
                target.add(container);
                dialog.close(target);
            }
        });
        target.add(dialog);
        dialog.show(target);
    }
}
