package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.components.SortOrderPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;

import javax.inject.Inject;
import java.util.List;

public class MileStonePhaseAdminPanel extends Panel {

    @Inject
    private MilestonePhaseTemplateService phaseService;

    private final WebMarkupContainer container;
    private final ModalWindowPlus dialog;
    private final FeedbackPanel feedback;

    public MileStonePhaseAdminPanel(String id) {
        super(id);

        this.container = new WebMarkupContainer(LIST_CONTAINER);
        this.feedback = new FeedbackPanel(FEEDBACK);
        this.dialog = new ModalWindowPlus(DIALOG);

        add(container.setOutputMarkupId(true));
        add(feedback.setOutputMarkupId(true));
        add(dialog);
        addCreateNewLink();
        addPhaseList();
    }

    private void addCreateNewLink() {
        add(new AjaxLink<MilestonePhaseTemplate>(CREATE_LINK) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                openDialog(target, Model.of(new MilestonePhaseTemplate()));
            }
        });

    }

    private void addPhaseList() {
        container.add(new ListView<MilestonePhaseTemplate>(PHASE_LIST, getListModel()) {
            @Override
            protected void populateItem(ListItem<MilestonePhaseTemplate> item) {
                item.add(new SortOrderPanel<>(MOVE, item.getModel(), phaseService));
                item.add(new Label(TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
                item.add(newActiveCheckBox(STATE_SWITCH, item.getModel()));
                item.add(newEditLink(EDIT, item.getModel()));
            }
        });
    }

    private AjaxCheckBox newActiveCheckBox(String id, final IModel<MilestonePhaseTemplate> phase) {
        return new AjaxCheckBox(id, Model.of(!phase.getObject().isDeleted())) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                phase.getObject().setDeleted(!getModelObject());
                phaseService.save(phase.getObject());
                target.add(container);
            }
        };
    }

    private AjaxLink<MilestonePhaseTemplate> newEditLink(String id, final IModel<MilestonePhaseTemplate> phaseModel) {
        return new AjaxLink<MilestonePhaseTemplate>(id, phaseModel) {
            @Override
            public void onClick(final AjaxRequestTarget target) {
                openDialog(target, getModel());
            }
        };
    }

    private LoadableDetachableModel<List<MilestonePhaseTemplate>> getListModel() {
        return new LoadableDetachableModel<List<MilestonePhaseTemplate>>() {
            @Override
            protected List<MilestonePhaseTemplate> load() {
                return phaseService.findAll(new Sort(Sort.Direction.ASC, SORT_ORDER));
            }
        };
    }

    private void openDialog(AjaxRequestTarget target, IModel<MilestonePhaseTemplate> model) {
        dialog.setTitle(model.getObject().getTitle() == null ? "New phase" : "Edit phase");
        dialog.setContent(new EditMileStonePhasePanel(dialog.getContentId(), model) {
            @Override
            protected void activitySaved(AjaxRequestTarget target) {
                info("Phase saved");
                target.add(feedback);
                target.add(container);
                dialog.close(target);
            }
        });
        dialog.show(target);
    }

    static final String LIST_CONTAINER = "listContainer";
    static final String FEEDBACK = "feedback";
    static final String DIALOG = "dialog";
    static final String CREATE_LINK = "createLink";
    static final String PHASE_LIST = "phaseList";
    static final String TITLE = "title";
    static final String STATE_SWITCH = "stateSwitch";
    static final String EDIT = "edit";
    static final String SORT_ORDER = "sortOrder";
    static final String MOVE = "move";

}
