package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.StringResourceModel;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

public class MileStoneStatusPanel extends Panel {

    @Inject
    private MileStoneService mileStoneService;

    private final IModel<Project> projectModel;
    private final IModel<MilestoneActivityTemplate> activityModel;
    private final MilestoneActivityTemplate.Type activityType;

    public MileStoneStatusPanel(String id, IModel<Project> projectModel, IModel<MilestoneActivityTemplate> activityModel, MilestoneActivityTemplate.Type type) {
        this(id, projectModel, activityModel, null, type);
    }

    public MileStoneStatusPanel(String id, IModel<Project> projectModel, final IModel<MilestoneActivityTemplate> activityModel, IModel<User> studentModel, MilestoneActivityTemplate.Type type) {
        super(id);
        this.projectModel = projectModel;
        this.activityModel = activityModel;
        this.activityType = type;

        final boolean isProjectMileStone = studentModel == null;
        final Milestone milestone = isProjectMileStone ? getProjectMileStone() : getStudentMileStone(studentModel);

        add(new WebComponent(CHECK_ICON) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(milestone.isConfirmed());
            }
        });

        add(new WebMarkupContainer(NOT_CHECKED) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(!milestone.isConfirmed() && activityType.equals(activityModel.getObject().getType()));
            }
        }.add(new AttributeAppender("title", getNotCompletedModel(), "")));
    }

    private Milestone getProjectMileStone() {
        return mileStoneService.getMileStone(projectModel.getObject(), activityModel.getObject());
    }

    private Milestone getStudentMileStone(IModel<User> studentModel) {
        return mileStoneService.getMileStone(projectModel.getObject(), studentModel.getObject(), activityModel.getObject());
    }

    private StringResourceModel getNotCompletedModel() {
        return new StringResourceModel("notCompleted", MileStoneStatusPanel.this, null);
    }

    static final String CHECK_ICON = "checkIcon";
    static final String NOT_CHECKED = "notChecked";
}
