package se.su.dsv.scipro.notes;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.basepages.MenuPage;
import se.su.dsv.scipro.components.AutoSavingBehaviour;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Date;

public class NotesPage extends MenuPage {

    @Inject
    private NoteService noteService;
    @Inject
    private DateService dateService;

    private final FeedbackPanel feedback;

    public NotesPage(){
        feedback = new FeedbackPanel("feedback");
        feedback.setOutputMarkupId(true);
        add(feedback);
        add(new NoteForm("form", new DetachableServiceModel<>(noteService, noteService.getNote(SciProSession.get().getUser()))));
    }

    private class NoteForm extends Form<Note> {
        public NoteForm(String id, IModel<Note> model) {
            super(id, model);
            TextArea<String> contents = new TextArea<>("content", new PropertyModel<String>(model, "content"));
            contents.add(new AutoSavingBehaviour() {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    noteService.save(getModelObject());
                    showSavedNotesFeedback();
                    target.add(feedback);
                }
            });
            add(contents);
        }

        @Override
        protected void onSubmit() {
            noteService.save(getModelObject());
            showSavedNotesFeedback();
        }

        private void showSavedNotesFeedback() {
            success("Saved notes at " + dateService.format(new Date(), DateStyle.DATETIME));
        }
    }
}
