package se.su.dsv.scipro.peer;

import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;

import java.util.List;

public class DisplayReviewPanel extends GenericPanel<PeerReview> {

    private final IModel<List<Answer>> answersModel;

    public DisplayReviewPanel(String id, IModel<PeerReview> reviewModel) {
        super(id, reviewModel);

        answersModel = new PropertyModel<>(reviewModel, "answers");

        add(new FencedFeedbackPanel("feedback", this));
        add(new WebMarkupContainer("questionsHeading") {

            @Override
            public boolean isVisible() {
                return !answersModel.getObject().isEmpty();
            }
        });

        ListView<Answer> questionList = new ListView<Answer>("questions", answersModel) {

            @Override
            protected void populateItem(ListItem<Answer> item) {
                int index = item.getIndex();

                DisplayQuestionPanel dqPanel = new DisplayQuestionPanel("questionPanel", item.getModel(), index);
                dqPanel.setOutputMarkupId(true);
                item.add(dqPanel);
            }
        };
        add(questionList);

        add(new WebMarkupContainer("commentsHeading") {

            @Override
            public boolean isVisible() {
                return getModelObject().getComment() != null;
            }
        });

        final MultiLineLabel reviewComment = new MultiLineLabel("reviewComment", new PropertyModel<String>(reviewModel, "comment"));
        add(reviewComment);

        add(new ViewAttachmentPanel("file", new PropertyModel<FileDescription>(reviewModel, "file")));


    }

    @Override
    protected void onConfigure() {
        super.onConfigure();
        setVisible(getModelObject().isCompleted());
    }
}
