package se.su.dsv.scipro.peer;

import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Arrays;

public class PeerPortalHeader extends GenericPanel<Project> {

    @Inject
    private PeerReviewService peerReviewService;
    @Inject
    private PeerRequestService peerRequestService;

    public PeerPortalHeader(String id, final IModel<Project> project) {
        super(id, project);

        add(new WebMarkupContainer("review") {
            {
                add(new BookmarkablePageLink<Void>("perform", PeerPortalPage.class, PeerPortalPage.getPageParameters(getModelObject()))
                        .setAutoEnable(true));
            }

            @Override
            protected void onConfigure() {
                setVisible(hasMoreRequestsThanReviews() && !hasReviewInProgress());
            }
        });

        add(new WebMarkupContainer("request") {
            @Override
            protected void onConfigure() {
                setVisible(!hasMoreRequestsThanReviews() && !hasReviewInProgress());
            }
        });

        add(new WebMarkupContainer("reviewInProgress") {
            @Override
            protected void onConfigure() {
                setVisible(hasReviewInProgress());
            }
        });

        add(new PeerRequestSubmissionPanel("submit", project) {
            @Override
            protected void onComponentTag(ComponentTag tag) {
                super.onComponentTag(tag);
                if (submissionStartsCollapsed()) {
                    tag.append("style", "display: none;", ";");
                }
            }
        });
    }

    private boolean hasReviewInProgress() {
        return !peerReviewService.findInProgressReviews(SciProSession.get().getUser(), getModelObject()).isEmpty();
    }

    protected boolean submissionStartsCollapsed() {
        return hasMoreRequestsThanReviews() || hasReviewInProgress();
    }

    private boolean hasMoreRequestsThanReviews() {
        return countRequests() > countReviews();
    }

    private long countReviews() {
        PeerReviewService.Filter reviewParams = new PeerReviewService.Filter();
        reviewParams.setAuthor(SciProSession.get().getUser());
        reviewParams.setAuthorProject(getModelObject());
        reviewParams.setStatuses(Arrays.asList(PeerReview.ReviewStatus.COMPLETED, PeerReview.ReviewStatus.IN_PROGRESS));
        return peerReviewService.count(reviewParams);
    }

    private long countRequests() {
        PeerRequestService.Filter requestParams = getRequestParams();
        return peerRequestService.count(requestParams);
    }

    private PeerRequestService.Filter getRequestParams() {
        PeerRequestService.Filter requestParams = new PeerRequestService.Filter();
        requestParams.setAuthor(SciProSession.get().getUser());
        requestParams.setStatuses(Arrays.asList(RequestStatus.WAITING, RequestStatus.FINISHED, RequestStatus.TAKEN));
        requestParams.setProject(getModelObject());
        return requestParams;
    }
}
