package se.su.dsv.scipro.peer;

import com.google.inject.Inject;
import com.googlecode.wickedcharts.highcharts.options.series.PointSeries;
import com.googlecode.wickedcharts.wicket6.highcharts.Chart;
import org.apache.wicket.markup.html.panel.Panel;
import se.su.dsv.scipro.statistics.BarColumnOptions;
import se.su.dsv.scipro.system.ProjectType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PeerReviewChartPanel extends Panel {

    private PeerReviewService.Filter filter;
    @Inject
    private PeerReviewService peerReviewService;

    public PeerReviewChartPanel(String id, PeerReviewService.Filter filter) {
        super(id);
        this.filter = filter;
        add(new PeerReviewFilterDetailsPanel("filterDetails", filter));
        addCharts();
    }
    private void addCharts() {
        BarColumnOptions peerReviews = new BarColumnOptions("Peer reviews", getCategories());
        for (ProjectType projectType : filter.getProjectTypes()) {
            peerReviews.addSeries(new PointSeries().addNumbers(getPointsForProjectType(projectType))
                    .setName(projectType.getName()));

        }
        add(new Chart("projectGraphics", peerReviews));
    }

    private List<Number> getPointsForProjectType(ProjectType projectType) {
        List<Number> result = new ArrayList<>();
        for (PeerReview.ReviewStatus status: Arrays.asList(PeerReview.ReviewStatus.values())) {
            result.add(peerReviewService.countByStatus(status, projectType, filter));
        }
        return result;
    }

    private List<String> getCategories() {
        List<String> categories = new ArrayList<>();
        for (PeerReview.ReviewStatus status: Arrays.asList(PeerReview.ReviewStatus.values())) {
            categories.add(status.name());
        }
        return categories;
    }


}
