package se.su.dsv.scipro.peer;

import com.google.common.base.Joiner;
import com.google.inject.Inject;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.StringResourceModel;
import org.apache.wicket.util.io.IClusterable;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.List;

public class PeerReviewFilterDetailsPanel extends Panel {
    private final PeerReviewService.Filter filter;
    @Inject
    DateService dateService;

    public PeerReviewFilterDetailsPanel(String id, PeerReviewService.Filter filter) {
        super(id);
        this.filter = filter;
        add(new Label("filterInfo",
                new StringResourceModel("filterInfo",
                        Model.of(new FilterStringFormatter()))).setEscapeModelStrings(false));
    }

    protected class FilterStringFormatter implements IClusterable {

        public String projectTypeNames() {
            List<String> projectTypes = new ArrayList<>();
            for (ProjectType projectType : filter.getProjectTypes()) {
                projectTypes.add(projectType.getName());
            }
            return Joiner.on(", ").join(projectTypes);
        }

        public String supervisorName() {
            User supervisor = filter.getSupervisor();
            if (supervisor == null) {
                return getString("notAvailable");
            }
            return supervisor.getFullName();
        }

        public String reviewerName() {
            User author = filter.getAuthor();
            if (author == null) {
                return getString("notAvailable");
            }
            return author.getFullName();

        }

        public String language() {
            Language language = filter.getLanguage();
            if (language == null) {
                return getString("notAvailable");
            }
            return language.getName();
        }

        public String startedBetween() {
            StringBuilder sb = new StringBuilder();

            if (filter.getCreatedAfter() == null) {
                sb.append(getString("notAvailable"));
            } else {
                String createdAfter = dateService.format(filter.getCreatedAfter(), DateStyle.DATE);
                sb.append(createdAfter);
            }

            sb.append(" and ");

            if (filter.getCreatedBefore() == null) {
                sb.append(getString("notAvailable"));
            } else {
                String createdBefore = dateService.format(filter.getCreatedBefore(), DateStyle.DATE);
                sb.append(createdBefore);
            }
            return sb.toString();
        }
    }
}

