package se.su.dsv.scipro.peer;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

@ProjectModuleComponent(ProjectModule.PEER)
public class PeerSummaryPanel extends GenericPanel<Project> {

    public static final String RECEIVED_REVIEWS = "requests";
    public static final String PERFORMED_REVIEWS = "reviews";
    public static final String REVIEW_LINK = "reviewLink";
    public static final String DATE_CREATED = "dateCreated";
    public static final String REVIEWER = "reviewer";
    public static final String NO_RECEIVED = "noReceived";
    public static final String NO_PERFORMED = "noPerformed";

    @Inject
    private PeerReviewService peerReviewService;

    public PeerSummaryPanel(String id, IModel<Project> model) {
        super(id, model);

        add(new PeerReviewList(RECEIVED_REVIEWS, getReceivedReviews()));
        add(new PeerReviewList(PERFORMED_REVIEWS, getPerformedReviews()));
        add(new NoReviewsLabel(NO_RECEIVED, getReceivedReviews()));
        add(new NoReviewsLabel(NO_PERFORMED, getPerformedReviews()));
    }

    private LoadableDetachableModel<List<PeerReview>> getReceivedReviews() {
        return new LoadableDetachableModel<List<PeerReview>>() {
            @Override
            protected List<PeerReview> load() {
                return peerReviewService.findReceivedCompletedReviews(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<List<PeerReview>> getPerformedReviews() {
        return new LoadableDetachableModel<List<PeerReview>>() {
            @Override
            protected List<PeerReview> load() {
                return peerReviewService.findCompletedReviews(getModelObject());
            }
        };
    }

    private class PeerReviewList extends ListView<PeerReview> {

        public PeerReviewList(String id, IModel<List<PeerReview>> model) {
            super(id, model);
        }

        @Override
        protected void populateItem(ListItem<PeerReview> item) {
            PageParameters parameters = new PageParameters();
            parameters.add(PageParameterKeys.MAP.get(PeerReview.class), item.getModelObject().getId());
            BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(REVIEW_LINK, getReviewPage(), parameters);
            link.add(new DateLabel(DATE_CREATED, new PropertyModel<Date>(item.getModel(), DATE_CREATED)));
            item.add(link);
            item.add(new Label(REVIEWER, new PropertyModel<>(item.getModel(), "reviewer.fullName")));
        }

    }

    private Class<? extends Page> getReviewPage() {
        return getModelObject().isParticipant(SciProSession.get().getUser()) ?
                ViewPeerReviewPage.class : SupervisorViewPeerReviewPage.class;
    }

    private static class NoReviewsLabel extends Label {

        private final IModel<List<PeerReview>> list;

        public NoReviewsLabel(String id, IModel<List<PeerReview>> list) {
            super(id, new ResourceModel("noReviews"));
            this.list = list;
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(list.getObject().isEmpty());
        }
    }
}
