package se.su.dsv.scipro.peer;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.checklists.ViewChecklistTemplatePanelPeer;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Date;

public class RequestDetailsPanel extends Panel {

    public static final String TEMPLATE_LINK = "templateLink";
    public static final String DIALOG = "dialog";
    public static final String DATE_REQUESTED = "dateRequested";
    public static final String REQUESTER = "requester";

    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    private ModalWindowPlus modalWindowPlus;

    public RequestDetailsPanel(String id, IModel<PeerRequest> peerRequest) {
        super(id);

        modalWindowPlus = new ModalWindowPlus(DIALOG);
        add(modalWindowPlus);

        add(new DateLabel(DATE_REQUESTED, new PropertyModel<Date>(peerRequest, "dateCreated"), DateStyle.DATETIME));
        add(new UserLinkPanel(REQUESTER, new PropertyModel<User>(peerRequest, "requester")));

        add(new AjaxLink<ChecklistTemplate>(TEMPLATE_LINK, new PropertyModel<ChecklistTemplate>(peerRequest, "checklistTemplate")) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                ViewChecklistTemplatePanelPeer vclPanel =
                        new ViewChecklistTemplatePanelPeer(modalWindowPlus.getContentId(), getModel());
                modalWindowPlus.setContent(vclPanel);
                modalWindowPlus.setTitle("Preview of chosen checklist template");
                modalWindowPlus.show(target);
            }

            @Override
            protected void onConfigure() {
                setEnabled(getModelObject() != null);
                setBody(Model.of(getModelObject() != null ? getModelObject().getName() : "No template attached"));
            }
        });

        IModel<Language> languageModel = new PropertyModel<>(peerRequest, "language");
        if (peerRequest.getObject().getLanguage() != null) {
            add(new Label("language", new PropertyModel<String>(languageModel, "name")));
        } else {
            add(new Label("language", "No language provided"));
        }


        if (peerRequest.getObject().getComment() != null) {
            add(new MultiLineLabel("requestComment", new PropertyModel<>(peerRequest, "comment")));
        } else {
            add(new Label("requestComment", "No comment provided"));
        }

        IModel<FileDescription> fileModel = new PropertyModel<>(peerRequest, "file");
        if (fileModel.getObject() != null) {
            add(new Label("fileName", new PropertyModel<String>(fileModel, "name")));
        } else {
            add(new Label("fileName", "No file provided"));
        }

        add(new ViewAttachmentPanel("download", fileModel) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisible(generalSystemSettingsService.getGeneralSystemSettingsInstance().isPeerDownloadEnabled());
            }
        });
    }
}
