package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.navigation.paging.AjaxPagingNavigator;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.FilteredListModel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ProjectPartnerPanel extends Panel {

    public static final int ITEMS_PER_PAGE = 5;

    @Inject
    private ProjectPartnerService projectPartnerService;
    @Inject
    private ProjectTypeService projectTypeService;

    public ProjectPartnerPanel(String id) {
        super(id);

        final IModel<? extends List<? extends ProjectType>> matchableTypes = getMatchableTypes();

        add(new ListView<ProjectType>("tabs", matchableTypes) {
            @Override
            protected void populateItem(final ListItem<ProjectType> item) {
                final String href = "#" + item.getModelObject().getName().toLowerCase().replaceAll("[^a-z]", "");
                final ExternalLink tab = new ExternalLink("tabLink", Model.of(href), new PropertyModel<>(item.getModel(), "name"));
                if (item.getIndex() == 0) {
                    item.add(AttributeAppender.append("class", "active"));
                }
                item.add(tab);
            }
        });

        add(new ListView<ProjectType>("types", matchableTypes) {
            @Override
            protected void populateItem(final ListItem<ProjectType> item2) {
                ProjectPartnerDataProvider projectPartnerDataProvider = new ProjectPartnerDataProvider(projectPartnerService, 0, item2.getModel());

                final WebMarkupContainer wmc = new WebMarkupContainer("listContainer");

                final DataView<ProjectPartner> dataView = new DataView<ProjectPartner>("list", projectPartnerDataProvider, ITEMS_PER_PAGE) {
                    @Override
                    protected void populateItem(final Item<ProjectPartner> item) {
                        item.add(new Label("exname", new PropertyModel<String>(item.getModel(), "user.fullName")));
                        item.add(new Label("exmail", new PropertyModel<String>(item.getModel(), "user.emailAddress")));
                        item.add(new DateLabel("exdate", new PropertyModel<Date>(item.getModel(), "dateCreated")));
                        item.add(new MultiLineLabel("exinfoText", new PropertyModel<String>(item.getModel(), "infotext")));
                        item.add(new AjaxConfirmationLink<Void>("delete", "Are you sure you want to remove this project partner application?") {
                            @Override
                            public void onClick(AjaxRequestTarget target) {
                                ProjectPartner temp = item.getModelObject();
                                projectPartnerService.delete(temp.getId());
                                target.add(wmc);

                            }
                        });
                    }
                };

                wmc.setOutputMarkupId(true);
                wmc.add(dataView);
                wmc.add(new AjaxPagingNavigator("navigator", dataView));
                item2.add(wmc);
                final String href = item2.getModelObject().getName().toLowerCase().replaceAll("[^a-z]", "");
                item2.add(AttributeModifier.replace("id", href));
                if (item2.getIndex() == 0) {
                    item2.add(AttributeAppender.append("class", "active"));
                }
            }
        });
    }

    private IModel<? extends List<? extends ProjectType>> getMatchableTypes() {
        final IModel<List<? extends ProjectType>> matchableTypes = new LoadableDetachableModel<List<? extends ProjectType>>() {
            @Override
            protected List<? extends ProjectType> load() {
                return projectTypeService.findWithModule(ProjectModule.MATCH);
            }
        };
        return new FilteredListModel<ProjectType>(matchableTypes) {
            @Override
            protected boolean accept(final ProjectType projectType) {
                return projectType.getMaxAuthors() > 1;
            }
        };
    }
}
