package se.su.dsv.scipro.reviewer;

import org.apache.wicket.util.io.IClusterable;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.ReviewerInteractionService;
import se.su.dsv.scipro.system.User;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ReviewerInteractionForumThread implements ForumThread<Project>, IClusterable {
    private final ReviewerInteractionService reviewerInteractionService;
    private final FileDescriptionService fileDescriptionService;

    public ReviewerInteractionForumThread(final ReviewerInteractionService reviewerInteractionService, final FileDescriptionService fileDescriptionService) {
        this.reviewerInteractionService = reviewerInteractionService;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public List<ForumPost> getPosts(final Project project) {
        return reviewerInteractionService.getPosts(project);
    }

    @Override
    public ForumPost reply(final Project project, final User poster, final String content, final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = store(attachments);
        return reviewerInteractionService.reply(project, poster, content, storedAttachments);
    }

    private Set<FileDescription> store(final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = new HashSet<>(attachments.size());
        for (FileUpload attachment : attachments) {
            storedAttachments.add(fileDescriptionService.storeFile(attachment));
        }
        return storedAttachments;
    }

    @Override
    public String getSubject(final Project project) {
        return "";
    }

    @Override
    public void setRead(User user, Project project) {
        reviewerInteractionService.markAllRead(user, project);
    }
}
