package se.su.dsv.scipro.reviewer.timeline

import java.util.Date

import org.apache.wicket.Component
import org.apache.wicket.model.{IModel, PropertyModel}
import se.su.dsv.scipro.forum.dataobjects.ForumPost
import se.su.dsv.scipro.forum.panels.threaded.ForumPostPanel
import se.su.dsv.scipro.reviewing.Decision

sealed trait Event {
  def when: Date

  def component(id: String, model: IModel[Event]): Component
}

object Event {

  case class Message(forumPost: ForumPost) extends Event {
    override def component(id: String, model: IModel[Event]): Component = new ForumPostPanel(id, new PropertyModel[ForumPost](model, "forumPost"))

    override def when: Date = forumPost.getDateCreated
  }

  case class Requested(decision: Decision) extends Event {
    override def component(id: String, model: IModel[Event]): Component = new RequestedPanel(id, new PropertyModel[Decision](model, "decision"))

    override def when: Date = decision.getRequested
  }

  case class Denied(decision: Decision) extends Event {
    override def component(id: String, model: IModel[Event]): Component = new DeniedPanel(id, new PropertyModel[Decision](model, "decision"))

    override def when: Date = decision.getDecisionDate
  }

  case class Approved(decision: Decision) extends Event {
    override def component(id: String, model: IModel[Event]): Component = new ApprovedPanel(id, new PropertyModel[Decision](model, "decision"))

    override def when: Date = decision.getDecisionDate
  }

}
