package se.su.dsv.scipro.security.auth;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.io.IOException;

/**
 * Throw-away implementation of a servlet filter, main task is to fake the getRemoteUser() call for the request chain.
 */
public final class MockRemoteUserFilter implements Filter {
	private static final Logger LOGGER = LoggerFactory.getLogger(MockRemoteUserFilter.class);
	//Default value unless supplied via init parameter
	private String fakedUser = "SOME_GUY";
	private FilterConfig cfg = null;
    /**
     * Default constructor. 
     */
    public MockRemoteUserFilter() {
    }
	/**
	 * @see Filter#destroy()
	 */
	public void destroy() {
		cfg = null;
	}
	/**
	 * Wraps the passed request and alters the behavior of getRemoteUser() for later links of the chain.
	 * @see Filter#doFilter(ServletRequest, ServletResponse, FilterChain)
	 */
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
		LOGGER.debug("Faking external authentication user: " + fakedUser);
		if(cfg != null){
			HttpServletRequestWrapper wrapper = new ModifiedRemoteUserRequestWrapper((HttpServletRequest)request,fakedUser);
			// pass the request along the filter chain
			chain.doFilter(wrapper, response);
			return;
		}
		chain.doFilter(request, response);
	}
	/**
	 * @see Filter#init(FilterConfig)
	 */
	public void init(FilterConfig fConfig) throws ServletException {
		cfg = fConfig;
		if(cfg!=null){
			fakedUser = cfg.getInitParameter("fakedUser");
		}
	}
	/**
	 * Private RequestWrapper, of no interest to anyone outside of this class.
	 */
	static class ModifiedRemoteUserRequestWrapper extends HttpServletRequestWrapper{
		private final String fakedUser;
		ModifiedRemoteUserRequestWrapper(final HttpServletRequest request,final String fakedUser){
			super(request);
			this.fakedUser = fakedUser; 
		}
		@Override
		public String getRemoteUser(){
			return fakedUser;
		}
	}
}
