package se.su.dsv.scipro.session;

import org.apache.wicket.Session;
import org.apache.wicket.injection.Injector;
import org.apache.wicket.protocol.http.WebSession;
import org.apache.wicket.request.Request;
import se.su.dsv.scipro.security.auth.roles.IRole;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.CurrentUser;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Collections;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

public class SciProSession extends WebSession {

    @Inject
    private CurrentUser currentUser;

    private Set<ProjectModule> projectModules = new HashSet<>();
    private Set<SystemModule> systemModules = new HashSet<>();

    public SciProSession(Request request) {
        super(request);
        Injector.get().inject(this);
    }

    public static SciProSession get() {
        return (SciProSession) Session.get();
    }

    public synchronized void setUser(User user) {
        currentUser.set(user);
    }

    public synchronized User getUser() {
        return currentUser.get();
    }

    public synchronized boolean isLoggedIn() {
        return currentUser.exists();
    }

    public synchronized boolean authorizedForRole(Roles role) {
        for (IRole existingRole : getRoles()) {
            if (existingRole.authorizedForRole(role)) {
                return true;
            }
        }
        return false;
    }

    private Set<Roles> getRoles() {
        return currentUser.exists() ? currentUser.get().getRoles() : Collections.<Roles>emptySet();
    }

    public boolean hasActualRole(Roles role) {
        for (IRole existingRole : getRoles()) {
            if (existingRole.isActualRole(role)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public Locale getLocale() {
        return Locale.ENGLISH;
    }

    public Set<ProjectModule> getProjectModules() {
        return Collections.unmodifiableSet(projectModules);
    }

    public void setProjectModules(Set<ProjectModule> projectModules) {
        this.projectModules.clear();
        this.projectModules.addAll(projectModules);
    }

    public Set<SystemModule> getSystemModules() {
        return Collections.unmodifiableSet(systemModules);
    }

    public void setSystemModules(Set<SystemModule> modules) {
        this.systemModules.clear();
        this.systemModules.addAll(modules);
    }
}
