package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.io.IClusterable;
import scala.Option;
import scala.collection.JavaConversions;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.grading.GradeStatisticsService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class AdminGradeStatisticsPage extends AbstractAdminStatisticsPage {

    @Inject
    private GradeStatisticsService gradeStatisticsService;

    public AdminGradeStatisticsPage() {

        SortableDataProvider<Grade, String> provider = new SortableDataProvider<Grade, String>() {
            @Override
            public Iterator<? extends Grade> iterator(long first, long count) {
                scala.collection.Iterable<Grade> all = gradeStatisticsService.findAll(new PageAdapter(first, count, getSort()));
                return JavaConversions.asJavaIterable(all).iterator();
            }

            @Override
            public long size() {
                return gradeStatisticsService.count();
            }

            @Override
            public IModel<Grade> model(Grade object) {
                return new DetachableModel<>(object, new GradePersister());
            }
        };

        add(new ExportableDataPanel<>("dp", getColumns(), provider));
    }

    private List<IColumn<Grade,String>> getColumns() {
        List<IColumn<Grade, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Grade>(Model.of("Final grade date"), "reportedOn", "reportOn", DateStyle.DATE));
        columns.add(new PropertyColumn<Grade, String>(Model.of("Final grade"), "value", "value"));
        columns.add(new MultipleUsersColumn<Grade, String>(Model.of("Authors")) {
            @Override
            public IModel<? extends List<? extends User>> getUsers(final IModel<Grade> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<Set<User>>(rowModel, "project.projectParticipants"));
            }
        });
        columns.add(new PropertyColumn<Grade, String>(Model.of("Supervisor"), "project.headSupervisor.fullName"));
        columns.add(new PropertyColumn<Grade, String>(Model.of("Title"), "project.title"));
        return columns;
    }

    private class GradePersister implements Persister<Grade>, IClusterable {
        @Override
        public Option<Grade> load(long id) {
            return gradeStatisticsService.findOne(id);
        }

        @Override
        public Option<Long> id(Grade grade) {
            return Option.apply(grade.getId());
        }
    }
}