package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.joda.time.DateTime;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import static java.util.Arrays.asList;

public class AdminUnfinishedFinalSeminarsPage extends AbstractAdminStatisticsPage {
    private static Date SIX_MONTHS_AGO = new DateTime().plusMonths(-6).toDate();
    private static Date SEVEN_DAYS_AGO = new DateTime().plusDays(-7).toDate();
    @Inject
    private FinalSeminarService finalSeminarService;

    public AdminUnfinishedFinalSeminarsPage() {
        final IModel<Date> seminarAfter = Model.of(SIX_MONTHS_AGO);
        final IModel<Date> seminarBefore = Model.of(SEVEN_DAYS_AGO);
        add(new Form<Void>("form") { {
            add(new DatePickerPanel("seminarDate", seminarAfter, seminarBefore));
        }});
        add(new ExportableDataPanel<>("table", columns(), new UnfinishedSeminarProvider(seminarAfter, seminarBefore)));
    }

    private List<? extends IColumn<FinalSeminar, String>> columns() {
        return asList(
                new PropertyColumn<FinalSeminar, String>(Model.of("Title"), "project.title", "project.title"),
                new SupervisorColumn(Model.of("Head supervisor"), "project.headSupervisor.firstName"),
                new DateColumn<FinalSeminar>(Model.of("Seminar date"), "startDate", "startDate", DateStyle.DATETIME),
                new ParticipationColumn(Model.of("Not graded respondents"), "notGradedRespondents"),
                new ParticipationColumn(Model.of("Not graded opponents"), "notGradedOpponents"),
                new ParticipationColumn(Model.of("Not graded active participants"), "notGradedActiveParticipations")
        );
    }

    private static class SupervisorColumn extends AbstractExportableColumn<FinalSeminar, String, String> {

        public SupervisorColumn(IModel<String> displayModel, String sortProperty) {
            super(displayModel, sortProperty);
        }

        @Override
        public IModel<String> getDataModel(IModel<FinalSeminar> model) {
            User supervisor = model.getObject().getProject().getHeadSupervisor();
            return Model.of(supervisor.getFullName() + " <" + supervisor.getEmailAddress() + ">");
        }

        @Override
        public void populateItem(Item<ICellPopulator<FinalSeminar>> item, String id, IModel<FinalSeminar> model) {
            item.add(new UserLinkPanel(id, new PropertyModel<User>(model, "project.headSupervisor")));
        }
    }
    private static class ParticipationColumn extends MultipleUsersColumn<FinalSeminar, String> {

        private String property;

        public ParticipationColumn(IModel<String> header, String property) {
            super(header);
            this.property = property;
        }

        @Override
        public IModel<? extends List<? extends User>> getUsers(final IModel<FinalSeminar> rowModel) {
            return new PropertyModel<>(rowModel, property);
        }
    }

    private final class UnfinishedSeminarProvider extends SortableDataProvider<FinalSeminar, String> {

        private final IModel<Date> seminarAfter;
        private final IModel<Date> seminarBefore;

        private UnfinishedSeminarProvider(IModel<Date> seminarAfter, IModel<Date> seminarBefore) {
            this.seminarAfter = seminarAfter;
            this.seminarBefore = seminarBefore;
        }

        @Override
        public Iterator<? extends FinalSeminar> iterator(long first, long count) {
            return finalSeminarService.findUnfinishedSeminars(seminarAfter.getObject(), seminarBefore.getObject(), new PageAdapter(first, count, getSort())).iterator();
        }

        @Override
        public long size() {
            return finalSeminarService.countUnfinishedSeminars(seminarAfter.getObject(), seminarBefore.getObject());
        }

        @Override
        public IModel<FinalSeminar> model(FinalSeminar object) {
            return new DetachableServiceModel<>(finalSeminarService, object);
        }

    }
}
