package se.su.dsv.scipro.statistics;

import com.googlecode.wickedcharts.highcharts.options.series.PointSeries;
import com.googlecode.wickedcharts.wicket6.highcharts.Chart;
import org.apache.wicket.markup.html.panel.Panel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class MilestoneChartPanel extends Panel {

    @Inject
    private MilestoneActivityTemplateService templateService;
    @Inject
    private MileStoneService mileStoneService;
    private MileStoneService.Filter filter;

    public MilestoneChartPanel(String id, MileStoneService.Filter filter) {
        super(id);
        this.filter = filter;
        add(new MilestoneFilterDetailsPanel("filterDetails", filter));
        addCharts();
    }

    private void addCharts() {
        BarColumnOptions projectOptions = new BarColumnOptions("Projects with project milestone confirmed", getCategories(MilestoneActivityTemplate.Type.PROJECT));
        for (ProjectType projectType : filter.getProjectTypes()) {
            projectOptions.addSeries(new PointSeries().addNumbers(getPointsForProjectType(projectType, MilestoneActivityTemplate.Type.PROJECT))
                    .setName(projectType.getName()));

        }
        add(new Chart("projectGraphics", projectOptions));

        BarColumnOptions individualOptions = new BarColumnOptions("Projects with individual milestone confirmed", getCategories(MilestoneActivityTemplate.Type.STUDENT));
        for (ProjectType projectType : filter.getProjectTypes()) {
            individualOptions.addSeries(new PointSeries().addNumbers(getPointsForProjectType(projectType, MilestoneActivityTemplate.Type.STUDENT))
                    .setName(projectType.getName()));

        }
        add(new Chart("individualGraphics", individualOptions));
    }

    private List<Number> getPointsForProjectType(ProjectType projectType, MilestoneActivityTemplate.Type type) {
        List<Number> result = new ArrayList<>();
        for (MilestoneActivityTemplate template : getAllActivities(type)) {
            result.add(mileStoneService.countCompletedMileStones(template, filter, projectType));
        }
        return result;
    }

    private List<String> getCategories(MilestoneActivityTemplate.Type type) {
        List<String> categories = new ArrayList<>();
        for (MilestoneActivityTemplate template : getAllActivities(type)) {
            categories.add(template.getTitle());
        }
        return categories;
    }

    private List<MilestoneActivityTemplate> getAllActivities(MilestoneActivityTemplate.Type type) {
        return templateService.getAllActivities(type, filter.getProjectTypes());
    }
}
