package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestoneStatisticsService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class MilestoneStatisticsProgressPanel extends Panel {
    @Inject
    private MilestoneActivityTemplateService milestoneActivityTemplateService;
    @Inject
    private MilestoneStatisticsService milestoneStatisticsService;
    @Inject
    private ProjectService projectService;

    private IModel<MilestoneActivityTemplate> completed = new Model<>();
    private IModel<MilestoneActivityTemplate> notCompleted = new Model<>();

    public MilestoneStatisticsProgressPanel(final String id) {
        super(id);

        add(new Form<Void>("selection") {
            {
                IModel<? extends List<? extends MilestoneActivityTemplate>> templates = getTemplates();
                add(new DropDownChoice<>("completed", completed, templates, new MilestoneActivityTemplateChoiceRenderer()).setRequired(true));
                add(new DropDownChoice<>("notCompleted", notCompleted, templates, new MilestoneActivityTemplateChoiceRenderer()).setRequired(true));
            }
        });

        add(new ExportableDataPanel<>("projects", columns(), new ProgressedProjectsProvider()));
    }

    private List<? extends IColumn<Project, String>> columns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Project>(Model.of("Project started"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Title"), "title", "title"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Authors"), "authorNames"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Status"), "projectStatus"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Supervisor"), "headSupervisor.lastName", "headSupervisor.fullName"));
        return columns;
    }

    private IModel<? extends List<? extends MilestoneActivityTemplate>> getTemplates() {
        return new LoadableDetachableModel<List<? extends MilestoneActivityTemplate>>() {
            @Override
            protected List<? extends MilestoneActivityTemplate> load() {
                List<MilestoneActivityTemplate> all = milestoneActivityTemplateService.findAll();
                Collections.sort(all, new MilestoneActivityTemplate.BySortOrderComparator());
                return all;
            }
        };
    }

    private static class MilestoneActivityTemplateChoiceRenderer extends ChoiceRenderer<MilestoneActivityTemplate> {
        public MilestoneActivityTemplateChoiceRenderer() {
            super("title", "id");
        }

        @Override
        public Object getDisplayValue(final MilestoneActivityTemplate object) {
            return String.format("[%s] %s", object.getMilestonePhaseTemplate().getTitle(), object.getTitle());
        }
    }

    private class ProgressedProjectsProvider extends SortableDataProvider<Project, String> {
        @Override
        public Iterator<? extends Project> iterator(final long first, final long count) {
            return milestoneStatisticsService.getProgressedProjects(completed.getObject(), notCompleted.getObject(), new PageAdapter(first, count, getSort())).iterator();
        }

        @Override
        public long size() {
            if (completed.getObject() == null || notCompleted.getObject() == null) { return 0; }
            return milestoneStatisticsService.countProgressedProjects(completed.getObject(), notCompleted.getObject());
        }

        @Override
        public IModel<Project> model(final Project object) {
            return new DetachableServiceModel<>(projectService, object);
        }
    }
}
