package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.*;

public class SupervisorProjectStatisticsPanel extends Panel {

    @Inject
    private ProjectService projectService;
    private ProjectService.Filter filter;

    public SupervisorProjectStatisticsPanel(String id) {
        super(id);
        addFilters();
        add(new Label("title", PROJECTS_LABEL));
        addProjectsTable();
    }

    private void addFilters() {
        Set<ProjectStatus> statuses = new HashSet<>();
        statuses.add(ProjectStatus.ACTIVE);
        filter = new ProjectService.Filter();
        filter.setSupervisor(null);
        filter.setStatuses(statuses);
        filter.setRoles(EnumSet.allOf(ProjectTeamMemberRoles.class));
    }

    private void addProjectsTable() {
        SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(filter));
        provider.setSort("title", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>(PROJECTS, createColumns(), provider));
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project,String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Title"), "title", "title"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Authors"), "authorNames"));
        columns.add(new AbstractColumn<Project, String>(Model.of("Role")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> item, String id, IModel<Project> model) {
                Project p = model.getObject();
                item.add(new Label(id, getRoleString(p)));
            }
        });
        return columns;
    }

    @Override
    protected void onConfigure() {
        setVisible(filter.getSupervisor() != null);
    }

    protected void updateSupervisor(User newSupervisor) {
        filter.setSupervisor(newSupervisor);
    }

    private String getRoleString(Project p) {
        if (p.getHeadSupervisor().equals(filter.getSupervisor())) {
            return HEAD_SUPERVISOR;
        } else {
            for (User user : p.getCoSupervisors()) {
                if (user.equals(filter.getSupervisor())) {
                    return CO_SUPERVISOR;
                }
            }
            return REVIEWER;
        }
    }

    static final String PROJECTS = "projectTable";
    static final String HEAD_SUPERVISOR = "Head supervisor";
    static final String REVIEWER = "Reviewer";
    static final String CO_SUPERVISOR = "Co-supervisor";
    static final String PROJECTS_LABEL = "Active projects";
}
