package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.SupervisorActivityPlanTemplatesPage;
import se.su.dsv.scipro.components.AbstractMenuPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorActivityTemplates;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorAllPeers;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMilestoneStatistics;
import se.su.dsv.scipro.finalseminar.SupervisorFinalSeminarListingPage;
import se.su.dsv.scipro.finalthesis.SupervisorFinalThesisListingPage;
import se.su.dsv.scipro.peer.SupervisorPeersPage;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public abstract class AbstractSupervisorAllProjectsPage extends AbstractSupervisorPage {

    @Inject
    private ProjectTypeService projectTypeService;

    protected AbstractSupervisorAllProjectsPage() {
        initComponents();
    }

    public AbstractSupervisorAllProjectsPage(PageParameters pp) {
        super(pp);
        initComponents();
    }

    private void initComponents() {
        FeedbackPanel feedbackPanel = new FeedbackPanel("feedbackPanel");
        add(feedbackPanel);
        feedbackPanel.setOutputMarkupId(true);
        addSubMenu();
    }

    private void addSubMenu() {
        add(new AbstractMenuPanel("menuPanel", AbstractSupervisorAllProjectsPage.class, this.getClass()) {

            @Override
            protected MenuType getMenuType() {
                return MenuType.HORIZONTAL;
            }

            @Override
            protected List<MenuItem> getItemList() {
                final List<MenuItem> items = new ArrayList<>();
                items.add(new MenuItem("Start", SupervisorAllProjectsPage.class));
                if (moduleIsEnabled(ProjectModule.FINAL_SEMINAR)) {
                    items.add(new MenuItem("Final seminars", SupervisorFinalSeminarListingPage.class));
                }
                if (moduleIsEnabled(ProjectModule.GRADING)) {
                    items.add(new MenuItem("Final theses", SupervisorFinalThesisListingPage.class));
                }
                if (moduleIsEnabled(ProjectModule.PEER)) {
                    items.add(new MenuItem("Peers", SupervisorPeersPage.class, MenuHighlightSupervisorAllPeers.class));
                }
                if (moduleIsEnabled(ProjectModule.ACTIVITY_PLAN)) {
                    items.add(new MenuItem("Activity plan templates", SupervisorActivityPlanTemplatesPage.class, MenuHighlightSupervisorActivityTemplates.class));
                }
                if (moduleIsEnabled(ProjectModule.MILESTONES)) {
                    items.add(new MenuItem("Milestone statistics", SupervisorMilestoneStatisticsPage.class, MenuHighlightSupervisorMilestoneStatistics.class));
                }
                return items;
            }
        });
    }

    private boolean moduleIsEnabled(ProjectModule module) {
        return projectTypeService.projectTypeWithModuleExists(module);
    }
}
