package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.feedback.ContainerFeedbackMessageFilter;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.project.ExternalOrganization;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;

import javax.inject.Inject;

public class SupervisorExternalProjectPanel extends Panel {

    @Inject
    ProjectService projectService;

    private WebMarkupContainer enclosure;

    public SupervisorExternalProjectPanel(final String id, final IModel<Project> model) {
        super(id, model);

        createEnclosure();
        createForm();
    }

    private void createEnclosure() {
        TextField<String> textField = new TextField<>(EXTERNAL_FIELD, new PropertyModel<String>(getDefaultModel(), "externalOrganization.name"));

        InfoPanel helpIcon = new InfoPanel(HELP_TEXT, new ResourceModel("helpText"));

        enclosure = new WebMarkupContainer(ENCLOSURE) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getProject().hasExternalOrganization());
            }
        };
        enclosure.setOutputMarkupPlaceholderTag(true);
        enclosure.add(textField);
        enclosure.add(helpIcon);
    }

    private void createForm() {
        final Form<Void> form = new Form<Void>(FORM){
            protected void onSubmit(){
                projectService.save(getProject());
                success(getString("external.organization.changed"));
            }
        };
        form.setOutputMarkupId(true);

        form.add(new AjaxCheckBox(EXTERNAL_CHECKBOX, Model.of(getProject().hasExternalOrganization())) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                getProject().setExternalOrganization(getModelObject() ? new ExternalOrganization() : null);
                projectService.save(getProject());
                target.add(form);

                if (!getModelObject()) {
                    info(getString("external.organization.removed"));
                }
            }
        });
        form.add(enclosure);
        form.add(new FeedbackPanel(FEEDBACK, new ContainerFeedbackMessageFilter(form)));

        add(form);
    }

    private Project getProject() {
        return (Project) getDefaultModelObject();
    }

    static final String FORM = "form";
    static final String EXTERNAL_CHECKBOX = "checkbox";
    static final String ENCLOSURE = "enclosure";
    static final String HELP_TEXT = "help";
    static final String EXTERNAL_FIELD = "textField";
    static final String FEEDBACK = "feedback";
}
