package se.su.dsv.scipro.user;

import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.validation.EqualPasswordInputValidator;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminInternalUsers;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.PasswordFacade;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class AdminChangePasswordPage extends AbstractAdminUsersPage implements MenuHighlightAdminInternalUsers {

    @Inject
    private UserService userService;
    @Inject
    private PasswordFacade passwordFacade;

    public AdminChangePasswordPage(final PageParameters pp) {
        StringValue userId = pp.get(PageParameterKeys.MAP.get(User.class));
        User user;
        if (userId.isNull() || userId.isEmpty()){
            user = new User();
        } else {
            user = userService.findOne(userId.toLong());
        }
        IModel<User> userModel = new DetachableServiceModel<>(userService, user);
        add(new ChangePasswordForm("form", userModel));
    }

    private class ChangePasswordForm extends Form<User> {
        private IModel<String> passwordModel = new Model<>();

        public ChangePasswordForm(final String id, final IModel<User> userModel) {
            super(id, userModel);
            add(new FencedFeedbackPanel("feedback", this));
            PasswordTextField password1 = new PasswordTextField("password1", passwordModel);
            add(password1);
            PasswordTextField password2 = new PasswordTextField("password2", new Model<String>());
            add(password2);
            add(new EqualPasswordInputValidator(password1, password2));
        }

        @Override
        protected void onSubmit() {
            passwordFacade.setPassword(getModelObject(), passwordModel.getObject());
            success(getString("password.changed"));
        }
    }
}
