package se.su.dsv.scipro.util;

import org.apache.wicket.authorization.AuthorizationException;
import org.apache.wicket.core.request.mapper.StalePageException;
import org.apache.wicket.protocol.http.PageExpiredException;
import org.apache.wicket.protocol.http.request.WebClientInfo;
import org.apache.wicket.protocol.http.servlet.ResponseIOException;
import org.apache.wicket.protocol.http.servlet.ServletWebRequest;
import org.apache.wicket.request.IRequestHandler;
import org.apache.wicket.request.IRequestParameters;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.cycle.AbstractRequestCycleListener;
import org.apache.wicket.request.cycle.RequestCycle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

public class AdditionalExceptionLogger extends AbstractRequestCycleListener {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdditionalExceptionLogger.class);

    @Override
    public IRequestHandler onException(RequestCycle cycle, Exception ex) {
        if (!isWicketHandledException(ex)) {
            logAdditionalInformation(cycle);
        }
        return super.onException(cycle, ex);
    }

    private void logAdditionalInformation(RequestCycle cycle) {
        logCurrentUser();
        logBrowserInformation(cycle);
        logRequestInformation(cycle.getRequest());
    }

    private void logRequestInformation(Request request) {
        logRequestUrl(request);
        logRequestParameters(request);
    }

    private void logBrowserInformation(RequestCycle cycle) {
        WebClientInfo webClientInfo = new WebClientInfo(cycle);
        LOGGER.info("Browser: {}", webClientInfo.getUserAgent());
    }

    private void logRequestParameters(Request request) {
        IRequestParameters requestParameters = request.getRequestParameters();
        for (String parameterName : requestParameters.getParameterNames()) {
            if (!parameterName.contains("password")){
                LOGGER.info("Request parameter: {}={}", parameterName, requestParameters.getParameterValues(parameterName));
            }
        }
    }

    private void logRequestUrl(Request request) {
        LOGGER.info("Request url: {} {}", ((ServletWebRequest) request).getContainerRequest().getMethod(), request.getUrl());
    }

    private void logCurrentUser() {
        User user = SciProSession.get().getUser();
        if (user != null) {
            LOGGER.info("Current user: {} ({})", user, user.getId());
        }
        else {
            LOGGER.info("Current user: not logged in");
        }
    }

    private boolean isWicketHandledException(Exception ex) {
        return ex instanceof StalePageException || ex instanceof PageExpiredException || ex instanceof AuthorizationException || ex instanceof ResponseIOException;
    }
}
