package se.su.dsv.scipro.forum.mail

import javax.inject.Inject

import org.apache.wicket.markup.html.WebPage
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.StringValueConversions
import se.su.dsv.scipro.forum.dataobjects.ForumThread
import se.su.dsv.scipro.forummail.ForumMailGroupRepository
import se.su.dsv.scipro.group.{Group, ViewThreadPage}
import se.su.dsv.scipro.security.auth.Authorization
import se.su.dsv.scipro.security.auth.roles.Roles._
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.supervisor.pages.SupervisorViewGroupThreadPage
import se.su.dsv.scipro.util.PageParameterKeys

import scala.collection.JavaConverters.iterableAsScalaIterableConverter

@Authorization(requiresLoggedInUser = true, authorizedRoles = Array(AUTHOR, SUPERVISOR, REVIEWER))
class ForumMailGroupPage(val pp: PageParameters) extends WebPage(pp) {
  @Inject private var forumMailGroupRepository: ForumMailGroupRepository = _

  val currentUser = SciProSession.get().getUser

  val optParameters = for {
    id <- pp.get("id").asLong
    forumMail <- forumMailGroupRepository.findByThreadId(id)
  } yield {
    val group = forumMail.getGroupThread.getGroup
    val pp = new PageParameters
    pp.set(PageParameterKeys.MAP.get(classOf[ForumThread]), forumMail.getGroupThread.getId)
    pp.set(PageParameterKeys.MAP.get(classOf[Group]), group.getId)
    (group, pp)
  }
  
  val supervisor = for {
    (group, pp) <- optParameters
    if group.getProjects.asScala.exists(project =>
      project.isReviewer(currentUser) || project.isCoSupervisor(currentUser) || project.isSupervisor(currentUser))
  } yield setResponsePage(classOf[SupervisorViewGroupThreadPage], pp)

  val author = for {
    (group, pp) <- optParameters
    if group.getProjects.asScala.exists(_.isParticipant(currentUser))
  } yield setResponsePage(classOf[ViewThreadPage], pp)

  supervisor.orElse(author).getOrElse(setResponsePage(getApplication.getHomePage))
}
