package se.su.dsv.scipro.forum.mail

import javax.inject.Inject

import org.apache.wicket.markup.html.WebPage
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.StringValueConversions
import se.su.dsv.scipro.forum.dataobjects.ForumThread
import se.su.dsv.scipro.forum.pages.threaded.{ProjectViewForumThreadPage, SupervisorViewForumThreadPage}
import se.su.dsv.scipro.forummail.ForumMailProjectRepository
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.security.auth.Authorization
import se.su.dsv.scipro.security.auth.roles.Roles._
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.util.PageParameterKeys

@Authorization(requiresLoggedInUser = true, authorizedRoles = Array(AUTHOR, SUPERVISOR, REVIEWER))
class ForumMailProjectPage(val pp: PageParameters) extends WebPage(pp) {
  @Inject private var forumMailProjectRepository: ForumMailProjectRepository = _

  val currentUser = SciProSession.get().getUser

  val optParameters = for {
    id <- pp.get("id").asLong
    forumMail <- forumMailProjectRepository.findByThreadId(id)
  } yield {
    val project = forumMail.getProjectThread.getProject
    val pp = new PageParameters
    pp.set(PageParameterKeys.MAP.get(classOf[ForumThread]), forumMail.getProjectThread.getId)
    pp.set(PageParameterKeys.MAP.get(classOf[Project]), project.getId)
    (project, pp)
  }
  
  val supervisor = for {
    (project, pp) <- optParameters
    if project.isReviewer(currentUser) || project.isCoSupervisor(currentUser) || project.isSupervisor(currentUser)
  } yield setResponsePage(classOf[SupervisorViewForumThreadPage], pp)

  val author = for {
    (project, pp) <- optParameters
    if project.isParticipant(currentUser)
  } yield setResponsePage(classOf[ProjectViewForumThreadPage], pp)

  supervisor.orElse(author).getOrElse(setResponsePage(getApplication.getHomePage))
}
