package se.su.dsv.scipro.notifications.pages

import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.finalseminar.{FinalSeminar, ProjectFinalSeminarDetailsPage, ProjectFinalSeminarPage, SupervisorFinalSeminarPage}
import se.su.dsv.scipro.notifications.dataobject.SeminarEvent
import se.su.dsv.scipro.notifications.pages.FinalSeminarRedirection._
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.system.User

import scala.collection.JavaConverters.iterableAsScalaIterableConverter

private[pages] trait FinalSeminarRedirection {
  def finalSeminarLocation(event: SeminarEvent): UserBasedLocation = {
    val finalSeminar = event.getSeminar
    val project = finalSeminar.getProject

    val pp = new PageParameters
    pp.set(parameter[Project], project.getId)

    {
      case user if project.isParticipant(user) => (classOf[ProjectFinalSeminarPage], pp)
      case user if project.hasSupervisingRole(user) => (classOf[SupervisorFinalSeminarPage], pp)
      case user if finalSeminar.isOpponent(user) => (classOf[ProjectFinalSeminarDetailsPage], pp)
      case user if finalSeminar.isActiveParticipant(user) => (classOf[ProjectFinalSeminarDetailsPage], pp)
    }
  }
}

object FinalSeminarRedirection {

  implicit class FinalSeminarEnhancement(val finalSeminar: FinalSeminar) extends AnyVal {
    def isOpponent(user: User) = finalSeminar.getOppositions.asScala.exists(_.getUser == user)

    def isActiveParticipant(user: User) = finalSeminar.getActiveParticipations.asScala.exists(_.getUser == user)
  }

}