package se.su.dsv.scipro.admin.panels;

import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.FooterColumn;
import se.su.dsv.scipro.system.FooterLink;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AdminEditFooterColumnPanelTest extends SciProTest {

    private AdminEditFooterColumnPanel panel;
    private FooterLink footerLink;

    @Before
    public void setUp() throws Exception {
        footerLink = newFooterLink();
        when(footerLinkService.getLinks(FooterColumn.FIRST)).thenReturn(Collections.singletonList(footerLink));
        panel = tester.startComponentInPage(new AdminEditFooterColumnPanel("id", FooterColumn.FIRST));
    }

    @Test
    public void saves() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, "new"));
        formTester.setValue("title", "fin title");
        formTester.setValue("url", "http://localhost");
        formTester.submit();
        verify(footerLinkService).save(isA(FooterLink.class));
        tester.assertInfoMessages(panel.getString("added"));
    }

    @Test
    public void no_title() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, "new"));
        formTester.setValue("url", "http://localhost");
        formTester.submit();
        tester.assertErrorMessages("'Title' is required.");
    }

    @Test
    public void no_url() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, "new"));
        formTester.setValue("title", "fin title");
        formTester.submit();
        tester.assertErrorMessages("'url' is required.");
    }

    @Test
    public void lists_all_current_links() throws Exception {
        tester.assertListView(path(panel, "current"), footerLinkService.getLinks(FooterColumn.FIRST));
    }

    @Test
    public void lists_each_current_link_as_an_external_link() throws Exception {
        List<FooterLink> links = footerLinkService.getLinks(FooterColumn.FIRST);
        for (int i = 0; i < links.size(); i++) {
            String linkPath = path(panel, "current", i, "link");
            tester.assertComponent(linkPath, ExternalLink.class);
            tester.assertModelValue(linkPath, links.get(i).getUrl());
        }
    }

    @Test
    public void delete_footer_links() throws Exception {
        tester.newFormTester(path(panel, "current", 0, "delete")).submit();
        FooterLink footerLink = footerLinkService.getLinks(FooterColumn.FIRST).get(0);
        verify(footerLinkService).delete(footerLink);
    }

    @Test
    public void edit_link_updates_form_model() throws Exception {
        tester.newFormTester(path(panel, "current", 0, "edit")).submit();
        tester.assertModelValue(path(panel, "new"), footerLink);
    }

    @Test
    public void mailto_links_should_be_allowed() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, "new"));
        formTester.setValue("title", "My title");
        formTester.setValue("url", "mailto:thesissupport@dsv.su.se");
        formTester.submit();

        verify(footerLinkService).save(isA(FooterLink.class));
    }

    @Test
    public void move_up() throws Exception {
        tester.newFormTester(path(panel, "current", 0, "up")).submit();
        verify(footerLinkService).moveUp(isA(FooterLink.class));
    }

    @Test
    public void move_down() throws Exception {
        tester.newFormTester(path(panel, "current", 0, "down")).submit();
        verify(footerLinkService).moveDown(isA(FooterLink.class));
    }

    private FooterLink newFooterLink() {
        FooterLink footerLink = new FooterLink();
        footerLink.setFooterColumn(FooterColumn.FIRST);
        footerLink.setTitle("CMS");
        footerLink.setUrl("http://scipro.dsv.su.se");
        footerLink.setId(45L);
        return footerLink;
    }
}