package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodExemption;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.text.SimpleDateFormat;
import java.util.Date;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.applicationperiod.AdminEditApplicationPeriodExemptionsPage.*;

public class AdminEditApplicationPeriodExemptionsPageTest extends SciProTest {

    ApplicationPeriod applicationPeriod;

    @Before
    public void setUp() throws Exception {
        applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setId(1L);

        ApplicationPeriodExemption exemption = new ApplicationPeriodExemption();
        User user = new UserBuilder().setFirstName("Bill").setLastName("Gates").setEmail("billgates@example.com").create();
        exemption.setUser(user);
        exemption.setComment("Bla bla bla");
        exemption.setStart(new Date());
        exemption.setUntil(new Date());
        exemption.setGrantedOn(new Date());
        exemption.setGrantedBy(user);
        exemption.setApplicationPeriod(applicationPeriod);
        applicationPeriod.addApplicationPeriodExemption(exemption);

        when(applicationPeriodService.findOne(isA(Long.class))).thenReturn(applicationPeriod);
    }

    @Test
    public void errors() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester(ADD_FORM);
        formTester.submit();
        tester.assertErrorMessages("Field 'author' is required", "Field 'start' is required", "Field 'until' is required", "Field 'comment' is required");
    }

    @Test
    public void saves() throws Exception {
        startPage();
        FormTester formTester = tester.newFormTester(ADD_FORM);
        formTester.setValue(AUTHOR, String.valueOf(1L));

        String dateString = new SimpleDateFormat(dateService.getFormat(DateStyle.DATE)).format(new Date());
        formTester.setValue(START, dateString);
        formTester.setValue(UNTIL, dateString);

        formTester.setValue(COMMENT, "bla bla bla bla bla");
        formTester.submit(ADD_EXEMPTION);
        tester.assertNoErrorMessage();
        verify(applicationPeriodService).save(isA(ApplicationPeriod.class));
    }

    @Test
    public void deletes() throws Exception {
        startPage();
        tester.clickLink(path(EXEMPTIONS, 0, DELETE));
        verify(applicationPeriodService).save(isA(ApplicationPeriod.class));
    }

    private AdminEditApplicationPeriodExemptionsPage startPage() {
        return tester.startPage(AdminEditApplicationPeriodExemptionsPage.class, getPageParameters());
    }

    private PageParameters getPageParameters() {
        PageParameters parameters = new PageParameters();
        parameters.set(PageParameterKeys.MAP.get(ApplicationPeriod.class), 1L);
        return parameters;
    }
}