package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListView;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Target;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@RunWith(MockitoJUnitRunner.class)
public class ApplicationPeriodInfoPanelTest extends SciProTest {

    ApplicationPeriodInfoPanel panel;
    List<ApplicationPeriod> currentPeriods;

    @Before
    public void setUp() throws Exception {

        currentPeriods = new ArrayList<>();
        ApplicationPeriod bachelor = new ApplicationPeriod("Bachelor");
        bachelor.setStartDate(new Date(0));
        bachelor.setEndDate(new Date(86400));
        bachelor.setCourseStartDate(new Date(86400 * 2));
        bachelor.setId(1L);
        ApplicationPeriod master = new ApplicationPeriod("Master");
        master.setStartDate(new Date(0));
        master.setEndDate(new Date(86400));
        master.setCourseStartDate(new Date(86400 * 2));
        master.setId(2L);
        currentPeriods.add(bachelor);
        currentPeriods.add(master);
        Mockito.when(applicationPeriodService.getCurrentPeriods()).thenReturn(currentPeriods);

        ProjectType pc = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        Mockito.when(targetService.findOne(new ApplicationPeriod(), new User(), pc)).thenReturn(new Target(new User(), new ApplicationPeriod(), pc));

        panel = tester.startComponentInPage(new ApplicationPeriodInfoPanel("panel", true));
    }

    @Test
    public void testRenders() throws Exception {
        tester.assertNoErrorMessage();
    }

    @Test
    public void testContainsListView() throws Exception {
        tester.assertComponent(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST), ListView.class);
    }

    @Test
    public void testListViewContainsPeriods() throws Exception {
        tester.assertListView(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST), currentPeriods);
    }

    @Test
    public void testLabelForEachPeriod() throws Exception {
        for (int i = 0; i < currentPeriods.size(); i++) {
            tester.assertComponent(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST, i, ApplicationPeriodInfoPanel.TITLE), Label.class);
        }
    }

    @Test
    public void testStartDateLabelForEachPeriod() throws Exception {
        for (int i = 0; i < currentPeriods.size(); i++) {
            tester.assertComponent(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST, i, ApplicationPeriodInfoPanel.START_DATE), Label.class);
        }
    }

    @Test
    public void testEndDateLabelForEachPeriod() throws Exception {
        for (int i = 0; i < currentPeriods.size(); i++) {
            tester.assertComponent(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST, i, ApplicationPeriodInfoPanel.END_DATE), Label.class);
        }
    }

    @Test
    public void testCourseStartLabelForEachPeriod() throws Exception {
        for (int i = 0; i < currentPeriods.size(); i++) {
            tester.assertComponent(path(panel.getId(), ApplicationPeriodInfoPanel.PERIOD_LIST, i, ApplicationPeriodInfoPanel.COURSE_START_DATE), Label.class);
        }
    }

    @Test
    public void testEmptyLabelIsNotVisible() throws Exception {
        Assert.assertFalse(panel.get(ApplicationPeriodInfoPanel.EMPTY).isVisibleInHierarchy());
    }

}
