package se.su.dsv.scipro.checklists;

import com.google.common.base.Joiner;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.ChecklistAnswer;
import se.su.dsv.scipro.checklist.ChecklistQuestion;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static se.su.dsv.scipro.checklists.ChecklistAnswerCommentPanel.*;

public class ChecklistAnswerCommentPanelTest extends SciProTest {

    ChecklistAnswerCommentPanel panel;
    ChecklistQuestion question;

    @Before
    public void setUp() {
        question = new ChecklistQuestion("", 1);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ChecklistAnswerCommentPanel("panel", Model.of(question)));
    }

    @Test
    public void testHideTableIfEmptyAnswerList() {
        question.setAnswers(new ArrayList<ChecklistAnswer>());

        startPanel();

        Assert.assertFalse(panel.get(TABLE).determineVisibility());
    }

    @Test
    public void testShowTableIfHasAnswers() {
        List<ChecklistAnswer> answers = Collections.singletonList(new ChecklistAnswer(null));
        question.setAnswers(answers);

        startPanel();

        Assert.assertTrue(panel.get(TABLE).determineVisibility());
    }

    @Test
    public void testCorrectNumberOfAnswers() {
        List<ChecklistAnswer> answers = Arrays.asList(new ChecklistAnswer(null), new ChecklistAnswer(null));
        question.setAnswers(answers);

        startPanel();

        MarkupContainer component = (MarkupContainer) panel.get(path(TABLE, ChecklistAnswerCommentPanel.ANSWERS));
        Assert.assertEquals(2, component.size());
    }

    @Test
    public void comment_shown_if_comment_is_not_empty() throws Exception {
        ChecklistAnswer checklistAnswer = new ChecklistAnswer(null);
        checklistAnswer.setComment("comment");
        List<ChecklistAnswer> answers = Collections.singletonList(checklistAnswer);
        question.setAnswers(answers);

        startPanel();

        tester.assertVisible(path(panel, TABLE, ANSWERS, 0, NAME));
        tester.assertVisible(path(panel, TABLE, ANSWERS, 0, COMMENT));
    }

    @Test
    public void comment_hidden_if_comment_is_null() throws Exception {
        ChecklistAnswer checklistAnswer = new ChecklistAnswer(null);
        List<ChecklistAnswer> answers = Collections.singletonList(checklistAnswer);
        question.setAnswers(answers);

        startPanel();

        tester.assertInvisible(path(panel, TABLE, ANSWERS, 0, NAME));
        tester.assertInvisible(path(panel, TABLE, ANSWERS, 0, COMMENT));
    }

    @Test
    public void comment_hidden_if_comment_is_empty() throws Exception {
        ChecklistAnswer checklistAnswer = new ChecklistAnswer(null);
        checklistAnswer.setComment("");
        List<ChecklistAnswer> answers = Collections.singletonList(checklistAnswer);
        question.setAnswers(answers);

        startPanel();

        tester.assertInvisible(path(panel, TABLE, ANSWERS, 0, NAME));
        tester.assertInvisible(path(panel, TABLE, ANSWERS, 0, COMMENT));
    }

    private String path(String... ids) {
        return Joiner.on(":").join(ids);
    }
}
