package se.su.dsv.scipro.checklists;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.activityplan.ProjectActivityPlanPage;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

public class ProjectViewChecklistPageTest extends PageTest {

    private Checklist checklist;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);

        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description");
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(projectType)
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        Mockito.when(projectService.isPartOfProject(project, user)).thenReturn(true);
        checklist = Checklist.builder().name("name").project(project).description("description").build();
        mockChecklist(checklist);
    }

    @Test
    public void redirect_without_checklist_parameters() throws Exception {
        tester.startPage(ProjectViewChecklistPage.class, getInvalidParameters());
        tester.assertRenderedPage(ProjectActivityPlanPage.class);
    }

    @Test
    public void redirects_if_no_checklist_found_with_the_given_id() throws Exception {
        mockChecklist(null);
        tester.startPage(getPage(), getPageParameters());
        tester.assertRenderedPage(ProjectActivityPlanPage.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectViewChecklistPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), "1");
        pageParameters.set(PageParameterKeys.MAP.get(Checklist.class), "2");
        return pageParameters;
    }

    protected PageParameters getInvalidParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), "1");
        return pageParameters;
    }

    private void mockChecklist(Checklist value) {
        Mockito.when(checklistService.findOne(Matchers.anyLong())).thenReturn(value);
    }
}
