package se.su.dsv.scipro.checklists;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklist.ChecklistQuestion;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.ArrayList;
import java.util.List;

public class SupervisorViewChecklistPageTest extends PageTest {

    private static final String SOME_NAME = "someName";
    private Checklist checklist;

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorViewChecklistPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), "1");
        pp.add(PageParameterKeys.MAP.get(Checklist.class), "175");
        return pp;
    }

    @Before
    public void setUp() {


        checklist = Checklist.builder().name(SOME_NAME).project(ObjectMother._Project.create(1L)).description("description!").build();
        checklist.setId(2L);
        Mockito.when(checklistService.findOne(Matchers.isA(Long.class))).thenReturn(checklist);
        Mockito.when(projectService.findOne(Matchers.isA(Long.class))).thenReturn(checklist.getProject());
        Mockito.when(projectService.isSupervisorOrFollower(Matchers.isA(Project.class), Matchers.isA(User.class))).thenReturn(true);
        Mockito.when(dateService.getFormat(Matchers.isA(DateStyle.class))).thenReturn("yyyy-mm-dd");

        ChecklistQuestion question = new ChecklistQuestion("", 1);
        List<ChecklistQuestion> questions = new ArrayList<>();
        questions.add(question);
        checklist.setQuestions(questions);
    }

    @Test
    public void renders() throws Exception {
        startPageWithParams();
    }

    private SupervisorViewChecklistPage startPageWithParams() {
        return tester.startPage(new SupervisorViewChecklistPage(getPageParameters()));
    }
}
