package se.su.dsv.scipro.dataproviders;

import org.apache.wicket.model.IModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.GenericService;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class GenericDataProviderTest {
    @Mock
    private GenericService<DomainObject, Long> genericService;

    private GenericDataProvider<DomainObject> provider;

    @Before
    public void setUp() throws Exception {
        provider = new GenericDataProvider<>(genericService);
    }

    @Test
    public void size() throws Exception {
        long count = 17L;
        when(genericService.count()).thenReturn(count);

        assertEquals(count, provider.size());
    }

    @Test
    public void iterator() throws Exception {
        DomainObject domainObject = mock(DomainObject.class);
        when(genericService.findAll(isA(Pageable.class))).thenReturn(Collections.singletonList(domainObject));

        Iterator<? extends DomainObject> iterator = provider.iterator(0, Integer.MAX_VALUE);
        assertEquals(domainObject, iterator.next());
        assertFalse(iterator.hasNext());
    }

    @Test
    public void model() throws Exception {
        DomainObject domainObject = mock(DomainObject.class);

        IModel<DomainObject> model = provider.model(domainObject);

        assertEquals(domainObject, model.getObject());
    }
}
