package se.su.dsv.scipro.files;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.files.FileDescriptionPanel.FILE;

@RunWith(MockitoJUnitRunner.class)
public class FileDescriptionPanelTest extends SciProTest {

    @Mock
    Project project;

    private FileDescription fileDescription;

    FileDescriptionPanel panel;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().create();
        fileDescription = new FileDescription();
        fileDescription.setIdentifier("1");
        fileDescription.setId(1L);
        fileDescription.setUploader(user);

        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        when(userService.findOne(isA(Long.class))).thenReturn(user);

        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testUploader() throws Exception {
        tester.assertComponent(path(panel, FileDescriptionPanel.UPLOADER), UserLinkPanel.class);
    }

    @Test
    public void testDelete() throws Exception {
        tester.assertInvisible(path(panel, FileDescriptionPanel.DELETE));
    }

    @Test
    public void shows_correct_file_description() throws Exception {
        tester.assertComponent(path(panel, FILE), ViewAttachmentPanel.class);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new FileDescriptionPanel("panel", Model.of(fileDescription)));
    }
}
