package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ResourceLink;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.grading.OppositionReportPdfResource;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.SeminarBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.List;

public class DownloadPdfReportPanelTest extends SciProTest {

    private static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);
    private FinalSeminar seminar;
    private User uploader;
    private OppositionReport gradingReport;
    private DownloadPdfReportPanel panel;

    @Before
    public void setUp() throws Exception {
        uploader = new UserBuilder().create();
        setLoggedInAs(uploader);
        seminar = new SeminarBuilder().create();
        seminar.setStartDate(new DateTime().plusDays(1).toDate());
        List<FinalSeminarOpposition> oppositions = new ArrayList<>();
        FinalSeminarOpposition e = new FinalSeminarOpposition();
        e.setUser(ObjectMother.SOME_USER);
        oppositions.add(e);
        seminar.setOppositions(oppositions);

        prepareReports();

        Mockito.when(oppositionReportService.findOrCreateReport(Matchers.eq(seminar.getOppositions().iterator().next()))).thenReturn(gradingReport);
        startPanel(seminar);
    }

    @Test
    public void has_resource_link() throws Exception {
        submitAndStartPanel();
        tester.assertComponent(path(panel, DownloadPdfReportPanel.DOWNLOAD_PDF), ResourceLink.class);
    }

    @Test
    public void hides_resource_link_if_report_is_not_submitted() throws Exception {
        tester.assertInvisible(path(panel, DownloadPdfReportPanel.DOWNLOAD_PDF));
    }

    @Test
    public void has_label_for_pdf() throws Exception {
        submitAndStartPanel();
        tester.assertComponent(path(panel, DownloadPdfReportPanel.DOWNLOAD_PDF, DownloadPdfReportPanel.PDF_LABEL), Label.class);
    }

    @Test
    public void shows_date_for_report_creation() throws Exception {
        submitAndStartPanel();
        tester.assertModelValue(path(panel, DownloadPdfReportPanel.DOWNLOAD_PDF, DownloadPdfReportPanel.PDF_UPLOAD_DATE), gradingReport.getDateCreated());
    }

    private void submitAndStartPanel() {
        gradingReport.submit();
        Mockito.when(oppositionReportService.findOrCreateReport(Matchers.eq(seminar.getOppositions().iterator().next()))).thenReturn(gradingReport);
        startPanel(seminar);
    }

    private void prepareReports() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(BACHELOR, 30);
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(0).description("Description").build());
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(1).description("Description").build());
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(2).description("Description").build());
        gradingReportTemplate.addProjectCriterion("title", 0, gradingCriterionPointTemplates);
        gradingReport = gradingReportTemplate.createOppositionReport(seminar.getOppositions().iterator().next());
        gradingReport.setThesisSummary("some summary");
        for (Criterion criterion : gradingReport.getCriteria()) {
            criterion.setFeedback("Feedback");
        }
    }

    private void startPanel(final FinalSeminar seminar) {
        panel = tester.startComponentInPage(new DownloadPdfReportPanel("panel", Model.of(seminar), Model.of(gradingReport), new OppositionReportPdfResource(Model.of(gradingReport))));
    }
}
