package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import scala.util.Right;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.SeminarEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Date;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.finalseminar.MoveFinalSeminarPanel.DATE;
import static se.su.dsv.scipro.finalseminar.MoveFinalSeminarPanel.MOVE_FORM;

public class MoveFinalSeminarPanelTest extends SciProTest {
    public static final String panel = "id";
    private FinalSeminar seminar;

    @Before
    public void setUp() throws Exception {
        prepareFinalSeminar();
        when(finalSeminarSettingsService.getInstance()).thenReturn(new FinalSeminarSettings());
        tester.startComponentInPage(new MoveFinalSeminarPanel(panel, Model.of(seminar)));
        when(finalSeminarService.move(any(FinalSeminar.class), any(Date.class)))
                .thenReturn(Right.<SchedulingError, FinalSeminar>apply(seminar));
    }

    private void prepareFinalSeminar() {
        ProjectType bachelor = new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor");
        bachelor.getProjectTypeSettings().setMinOpponentsOnFinalSeminar(2);
        User supervisor = User.builder().firstName("Hasse").lastName("Handledare").emailAddress("hasse@example.com").build();
        DomainObjects.injectId(supervisor, 234L);
        Project project = new Project();
        project.setId(1L);
        project.setProjectType(bachelor);
        project.setHeadSupervisor(supervisor);
        seminar = new FinalSeminar();
        seminar.setProject(project);
        seminar.setStartDate(calculateDateByInterval(365).toDate());
        seminar.setRoom("6405");
        seminar.setPresentationLanguage(new Language());
        seminar.setReportLanguage(new Language());
    }

    @Test
    public void changing_date_should_send_notification() throws Exception {
        DomainObjects.injectId(seminar, 3232L);
        FormTester formTester = tester.newFormTester(path(panel, MOVE_FORM));
        formTester.setValue(DATE, calculateDateByInterval(100).toString(DateStyle.DATETIME.getFormat()));
        formTester.submit();
        tester.assertNoErrorMessage();
        verifyNotificationSent(SeminarEvent.Event.DATE_CHANGED);
    }

    private void verifyNotificationSent(SeminarEvent.Event event) {
        verify(notificationController).notifySeminar(Matchers.eq(seminar), Matchers.eq(event), any(NotificationSource.class));
    }

    private DateTime calculateDateByInterval(int numberOfDays) {
        DateTime startDate = new DateTime();
        startDate.plusDays(numberOfDays);
        return startDate;
    }
}