package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.plagiarism.TurnitinSettings;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

public class SeminarPanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
    @Mock
    FinalSeminarSettings finalSeminarSettings = new FinalSeminarSettings();

    private FinalSeminar SOME_FINAL_SEMINAR;
    private Project someProject;
    private SeminarPanel panel;
    private FinalSeminarOpposition opposition;
    private User opponentUser;

    @Before
    public void before() throws Exception {
        User supervisorUser = new UserBuilder().create();
        opponentUser = new UserBuilder().create();
        setLoggedInAs(supervisorUser);

        User authorUser = new UserBuilder().create();

        someProject = Project.builder()
                .title("some title")
                .projectType(BACHELOR)
                .headSupervisor(supervisorUser)
                .build();
        someProject.setId(238942L);

        addCoSupervisorToProject();

        SOME_FINAL_SEMINAR = mockSeminar(new DateTime().minusDays(1).toDate(), null);
        SOME_FINAL_SEMINAR.setDocument(addThesis());
        Mockito.when(finalSeminarSettingsService.getInstance()).thenReturn(finalSeminarSettings);
        Mockito.when(finalSeminarSettings.getDaysAheadToCreate()).thenReturn(1);
        OppositionReport oppositionReport = Mockito.mock(OppositionReport.class);
        Mockito.when(oppositionReportService.findOrCreateReport(opposition)).thenReturn(oppositionReport);
        FinalSeminarRespondent finalSeminarRespondent = new FinalSeminarRespondent(authorUser, SOME_FINAL_SEMINAR);
        List<FinalSeminarRespondent> respondents = Collections.singletonList(finalSeminarRespondent);
        Mockito.when(finalSeminarRespondentService.findOrCreate(SOME_FINAL_SEMINAR)).thenReturn(respondents);
        Mockito.when(projectService.findOne(someProject.getId())).thenReturn(someProject);
        TurnitinSettings turnitinSettings = new TurnitinSettings();
        turnitinSettings.setMaxFileSize(20);
        Mockito.when(turnitinSettingsService.getSettings()).thenReturn(turnitinSettings);
    }

    private void addCoSupervisorToProject() {
        User coSupervisorUser = new UserBuilder().create();
        someProject.addCoSupervisor(coSupervisorUser);
    }

    @Test
    public void shows_correct_project_title() throws Exception {
        startPanel();
        tester.assertModelValue(path(panel, SeminarPanel.PROJECT_TITLE), someProject.getTitle());
    }

    private FinalSeminar mockSeminar(Date startDate, FinalSeminarGrade grade) {
        FinalSeminar seminar = new FinalSeminar();
        DomainObjects.injectId(seminar, 1L);
        seminar.setStartDate(startDate);
        seminar.setProject(someProject);

        opposition = new FinalSeminarOpposition();
        DomainObjects.injectId(opposition, 42L);
        opposition.setUser(opponentUser);
        Project opponentProject = Project.builder().title("opponentProject").projectType(BACHELOR).build();
        opposition.setProject(opponentProject);
        opposition.setGrade(grade);
        seminar.addOpposition(opposition);

        FinalSeminarActiveParticipation activeParticipation = new FinalSeminarActiveParticipation();
        User activeParticipantUser = new UserBuilder().create();
        activeParticipation.setUser(activeParticipantUser);
        seminar.setActiveParticipations(Collections.singletonList(activeParticipation));

        return seminar;
    }

    private FileDescription addThesis() {
        FileDescription fileDescription = new FileDescription();
        fileDescription.setId(55L);
        fileDescription.setSize(10L);
        return fileDescription;
    }

    private void startPanel() {
        startPanel(SOME_FINAL_SEMINAR);
    }

    private void startPanel(FinalSeminar seminar) {
        panel = tester.startComponentInPage(new SeminarPanel("id", Model.of(seminar)));
    }
}
