package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static se.su.dsv.scipro.finalseminar.SeminarRespondentsPanel.*;

public class SeminarRespondentsPanelTest extends SciProTest {

    private FinalSeminar finalSeminar;
    private SeminarRespondentsPanel panel;
    private List<FinalSeminarRespondent> respondents;

    @Before
    public void setUp() throws Exception {
        finalSeminar = new FinalSeminar();
        finalSeminar.setStartDate(new DateTime().minusDays(1).toDate());
        User supervisorUser = new UserBuilder().create();
        finalSeminar.setProject(Project.builder()
                .title("title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING))
                .headSupervisor(supervisorUser)
                .build());
        respondents = Collections.singletonList(new FinalSeminarRespondent(new UserBuilder().create(), finalSeminar));

        setLoggedInAs(supervisorUser);

        Mockito.when(finalSeminarRespondentService.findOrCreate(finalSeminar)).thenReturn(respondents);
    }

    @Test
    public void has_list_of_respondents() throws Exception {
        startPanel();

        tester.assertModelValue(path(panel, RESPONDENTS), respondents);
    }

    @Test
    public void has_user_link_panel() throws Exception {
        startPanel();

        tester.assertComponent(path(panel, RESPONDENTS, 0, USER), UserLinkPanel.class);
    }

    @Test
    public void clicking_radio_choice_saves_respondent() throws Exception {
        startPanel();

        tester.executeAjaxEvent(path(panel, RESPONDENTS, 0, RADIOS), "onclick");

        Mockito.verify(finalSeminarRespondentService).save(Matchers.any(FinalSeminarRespondent.class));
    }

    @Test
    public void has_fenced_feedback_panel() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, FEEDBACK), FencedFeedbackPanel.class);
    }

    @Test
    public void repaints_feedback() throws Exception {
        startPanel();
        tester.executeAjaxEvent(path(panel, RESPONDENTS, 0, RADIOS), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel, FEEDBACK));
    }

    @Test
    public void radios_invisible_if_not_supervisor() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        startPanel();
        tester.assertInvisible(path(panel, RESPONDENTS, 0, RADIOS));
    }

    @Test
    public void radios_invisible_if_seminar_is_in_the_future() throws Exception {
        finalSeminar.setStartDate(new DateTime().plusDays(1).toDate());
        startPanel();
        tester.assertInvisible(path(panel, RESPONDENTS, 0, RADIOS));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SeminarRespondentsPanel("id", Model.of(finalSeminar)));
    }
}
