package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.plagiarism.TurnitinSettings;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

public class TurnItInPanelTest extends SciProTest {

    private TurnItInPanel panel;
    private FinalSeminar seminar;

    @Before
    public void setUp() throws Exception {
        FileDescription document = new FileDescription();
        document.setSize(10L * 1024L * 1024L); //10 MB
        seminar = new FinalSeminar();
        seminar.setDocument(document);
        User supervisorUser = new UserBuilder().create();
        seminar.setProject(Project.builder()
                .title("title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING))
                .headSupervisor(supervisorUser)
                .build());
        setLoggedInAs(supervisorUser);
    }

    @Test
    public void too_big_file_label_is_visible_if_file_is_to_big() throws Exception {
        mockTurnItInSettings(1);
        startPanel();
        tester.assertVisible(path(panel, TurnItInPanel.TOO_BIG_FILE_LABEL));
    }

    @Test
    public void too_big_file_label_is_invisible_if_not_supervisor() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        mockTurnItInSettings(1);
        startPanel();
        tester.assertInvisible(path(panel, TurnItInPanel.TOO_BIG_FILE_LABEL));
    }

    @Test
    public void too_big_file_label_is_invisible_if_file_below_max_size() throws Exception {
        mockTurnItInSettings(99);
        startPanel();
        tester.assertInvisible(path(panel, TurnItInPanel.TOO_BIG_FILE_LABEL));
    }

    @Test
    public void does_not_crash_without_document() throws Exception {
        seminar.setDocument(null);
        startPanel();
    }

    private void mockTurnItInSettings(int maxFileSize) {
        TurnitinSettings turnitinSettings = new TurnitinSettings();
        turnitinSettings.setMaxFileSize(maxFileSize);
        Mockito.when(turnitinSettingsService.getSettings()).thenReturn(turnitinSettings);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new TurnItInPanel("id", Model.of(seminar)));
    }
}
