package se.su.dsv.scipro.forum.pages;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.pages.threaded.SupervisorThreadedForumPage;
import se.su.dsv.scipro.forum.pages.threaded.SupervisorViewForumThreadPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorForumBasePageTest extends SciProTest {

    Project project;

    @Before
    public void setUp() throws Exception {
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        setRoles(Roles.values());
        setLoggedIn(true);
    }

    @Test
    public void testThreadedForumIsDefault() throws Exception {
        tester.startPage(getPage(), getPageParameters());
        tester.assertRenderedPage(SupervisorThreadedForumPage.class);
    }

    @Test
    public void testThreadIdSendsUserToViewThreadPage() throws Exception {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), "1");
        pp.add(PageParameterKeys.MAP.get(ForumThread.class), "2");
        when(projectForumService.findOne(isA(Long.class))).thenReturn(new ForumBuilder().createThread());
        tester.startPage(getPage(), pp);
        tester.assertRenderedPage(SupervisorViewForumThreadPage.class);
    }

    protected Class<? extends Page> getPage() {
        return SupervisorForumBasePage.class;
    }

    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), "1");
        return pp;
    }
}
