package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;


public class ProjectCreateForumThreadPageTest extends PageTest {

    ProjectCreateForumThreadPage page;
    Project project;

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectCreateForumThreadPage.class;
    }

    @Before
    public void setUp() throws Exception {
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        when(projectService.isPartOfProject(isA(Project.class), isA(User.class))).thenReturn(true);

        page = tester.startPage(ProjectCreateForumThreadPage.class, getPageParameters());
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1);
        return pageParameters;
    }

    @Test
    public void testPageContainsBackLink() throws Exception {
        tester.assertComponent(ProjectCreateForumThreadPage.BACK_LINK, BookmarkablePageLink.class);
    }

    @Test
    public void testClickLinkTakesUserBackToForumPage() throws Exception {
        tester.clickLink(ProjectCreateForumThreadPage.BACK_LINK, false);
        tester.assertRenderedPage(ProjectThreadedForumPage.class);
    }
}
