package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.TreeSet;

import static org.mockito.Mockito.*;

public class ProjectThreadedForumPageTest extends PageTest {

    Project project;

    @Before
    public void setUp() throws Exception {
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        project.setDateCreated(new Date());

        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        when(projectService.isPartOfProject(isA(Project.class), isA(User.class))).thenReturn(true);
        tester.startPage(ProjectThreadedForumPage.class, getPageParameters());
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectThreadedForumPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1);
        return pageParameters;
    }

    @Test
    public void testClickCreateLinkTakesUserToCreatePage() throws Exception {
        tester.assertBookmarkablePageLink(ProjectThreadedForumPage.CREATE_THREAD_LINK, ProjectCreateForumThreadPage.class, getPageParameters());
    }

}
