package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.model.Model;
import org.apache.wicket.protocol.http.mock.MockHttpServletRequest;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ForumBuilder;

import java.io.Serializable;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.internal.verification.VerificationModeFactory.times;

public class SubmitForumReplyPanelTest extends SciProTest {

    SubmitForumReplyPanel panel;

    @Before
    public void setUp() throws Exception {
        ProjectThread thread = mockForumThread();
        startPanel(thread, new ProjectForumThread(projectForumService, fileDescriptionService));
    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testSubmitReply() throws Exception {
        submit();

        tester.assertNoErrorMessage();
        verify(projectForumService, times(1)).createReply(isA(ProjectThread.class), isA(User.class), isA(String.class),
                anySetOf(FileDescription.class));
        tester.assertModelValue(path(panel.getId(), "form", "content"), "");
    }

    private ProjectThread mockForumThread() {
        ProjectThread thread = new ForumBuilder().createThread();
        when(projectForumService.createReply(
                any(ProjectThread.class), any(User.class), any(String.class), anySetOf(FileDescription.class)))
                .thenReturn(new ForumBuilder().createPost());
        return thread;
    }

    private void submit() {
        FormTester formTester = tester.newFormTester(path(panel.getId(), "form"));
        formTester.setValue("content", "content");

        MockHttpServletRequest servletRequest = tester.getRequest();
        servletRequest.setUseMultiPartContentType(true);

        formTester.submit();
    }

    private <A extends Serializable> void startPanel(A thread, se.su.dsv.scipro.forum.ForumThread<A> forumThread) {
        panel = tester.startComponentInPage(new SubmitForumReplyPanel<>("panel", Model.of(thread), forumThread));
    }
}
