package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.panels.AbstractReadStatePanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import static junit.framework.TestCase.assertNotNull;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

public class ThreadReadStatePanelTest extends SciProTest {

    ThreadReadStatePanel panel;
    User user;
    ForumThread forumThread;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().create();
        setLoggedIn(true);
        setLoggedInAs(user);

        forumThread = new ForumBuilder().createThread().getForumThread();
        forumThread.addPost(new ForumBuilder().createPost(forumThread));
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assertNotNull(panel);
    }

    @Test
    public void testTogglingCallsCorrectService() throws Exception {
        // given
        when(basicForumService.isThreadRead(eq(user), eq(forumThread))).thenReturn(true);

        // when
        startPanel();
        tester.executeAjaxEvent(path(panel.getId(), AbstractReadStatePanel.TOGGLE), "onclick");

        verify(basicForumService, times(1)).setThreadRead(eq(user), eq(forumThread), eq(false));

        // then
        ArgumentCaptor<Boolean> captor = ArgumentCaptor.forClass(Boolean.class);
        verify(basicForumService).setThreadRead(eq(user), eq(forumThread), captor.capture());

        assertEquals("Does not toggle to the correct state", false, captor.getValue());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ThreadReadStatePanel("panel", Model.of(forumThread)));
    }

}
