package se.su.dsv.scipro.grading;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.grading.GradingReportPointsPanel.GRADE;
import static se.su.dsv.scipro.grading.GradingReportPointsPanel.POINTS_LABEL;

public class GradingReportPointsPanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
    public static final Project SOME_PROJECT = Project.builder().title("Grading report project").projectType(BACHELOR).build();
    public static final User SOME_USER = new UserBuilder().setFirstName("Author").setLastName("One").create();

    @Mock
    private GradeCalculator gradeCalculator;

    private GradingReport gradingReport;
    private GradingReportPointsPanel panel;

    @Before
    public void setUp() throws Exception {
        prepareReports();
    }

    @Test
    public void shows_report_grade() throws Exception {
        GradingReport.Grade grade = GradingReport.Grade.A;
        when(gradeCalculator.getGrade(any(GradingReport.class))).thenReturn(grade);
        startPanel();

        tester.assertLabel(path(panel, GRADE), grade.toString());
    }

    @Test
    public void shows_total_points() throws Exception {
        long points = 42;
        when(gradeCalculator.getPoints(any(GradingReport.class))).thenReturn(points);
        startPanel();

        tester.assertLabel(path(panel, POINTS_LABEL), Long.toString(points));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new GradingReportPointsPanel("id", Model.of(gradingReport), gradeCalculator));
    }

    private void prepareReports() {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(BACHELOR, 30);
        gradingReportTemplate.addProjectCriterion("title", 0, getPointTemplates(2));
        gradingReport = gradingReportTemplate.createSupervisorReport(SOME_PROJECT, SOME_USER);
        for (GradingCriterion gradingCriterion : gradingReport.getGradingCriteria()) {
            gradingCriterion.setPoints(1);
            gradingCriterion.setFeedback("Feedback");
        }
    }

    private List<GradingCriterionPointTemplate> getPointTemplates(int maxPoint) {
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        for (int i = 0; i <= maxPoint; i++) {
            gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(i).description("Description").build());
        }
        return gradingCriterionPointTemplates;
    }
}
