package se.su.dsv.scipro.grading;

import org.apache.wicket.model.Model;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Date;

public class ThesisApprovedPanelTest extends SciProTest {

    public static Project SOME_PROJECT = Project.builder().title("Title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
    public static final FinalThesis FINAL_THESIS = new FinalThesis();
    private ThesisApprovedPanel panel;

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void displays_date() throws Exception {
        startPanel();

        tester.assertModelValue(path(panel, ThesisApprovedPanel.REQUIRES_GRADE_REPORT_PANEL, ThesisApprovedPanel.DATE_LABEL), FINAL_THESIS.getDateApproved());
    }

    @Test
    public void says_that_you_approved_thesis_if_supervisor() throws Exception {
        startPanel();

        tester.assertModelValue(path(panel, ThesisApprovedPanel.REQUIRES_GRADE_REPORT_PANEL, ThesisApprovedPanel.MESSAGE), "You approved the final thesis.");
    }

    @Test
    public void says_that_supervisor_approved_thesis_if_reviewer() throws Exception {
        User user = new UserBuilder().create();
        setLoggedInAs(user);
        SOME_PROJECT = createProject();
        SOME_PROJECT.addReviewer(user);


        startPanel();

        tester.assertModelValue(path(panel, ThesisApprovedPanel.REQUIRES_GRADE_REPORT_PANEL, ThesisApprovedPanel.MESSAGE), "Supervisor approved the final thesis.");
    }

    private void startPanel() {
        FINAL_THESIS.setDateCreated(new Date());
        FINAL_THESIS.setDateApproved(new Date());

        Mockito.when(finalThesisService.findByProject(SOME_PROJECT)).thenReturn(FINAL_THESIS);

        ProjectType projectType = new ProjectType(ProjectType.MASTER, "Master", "Master");
        projectType.setId(1L);

        Mockito.when(gradingReportService.getTemplate(SOME_PROJECT)).thenReturn(new GradingReportTemplate(projectType, 30));
        panel = tester.startComponentInPage(new ThesisApprovedPanel("panel", Model.of(SOME_PROJECT)));
    }

    protected Project createProject() {
        return Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(new User())
                .build();
    }
}