package se.su.dsv.scipro.group;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Collections;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class AuthorGroupPageTest extends PageTest {

    private User student;
    private Group group;

    @Before
    public void setUp() throws Exception {
        final User supervisor = User.builder().firstName("John").lastName("Smith").emailAddress("john@example.com").build();
        supervisor.setId(1L);
        student = User.builder().firstName("Stina").lastName("Student").emailAddress("stina@example.com").build();
        student.setId(2L);
        final Project project = Project.builder().title("Some project").projectType(ObjectMother.BACHELOR).build();
        project.addProjectParticipant(student);
        group = new Group();
        group.setId(1L);
        group.setUser(supervisor);
        group.setProjects(Collections.singleton(project));
        when(groupService.findOne(isA(Long.class))).thenReturn(group);
        setLoggedInAs(student);
    }

    @Test
    public void links_to_create_thread() throws Exception {
        startPage();
        tester.clickLink("createThread");
        tester.assertRenderedPage(CreateThreadPage.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AuthorGroupPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Group.class), 1);
        return pageParameters;
    }
}