package se.su.dsv.scipro.group;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class EditGroupPanelTest extends SciProTest {

    EditGroupPanel panel;
    Group group;

    @Before
    public void setUp() throws Exception {
        group = new Group();
        group.setId(1L);
        Project project = createProject();
        group.setProjects(new HashSet<>(Collections.singletonList(project)));
        when(groupService.save(isA(Group.class))).thenReturn(group);
        when(projectService.findOne(anyLong())).thenReturn(project);
        startPanel();
    }

    @Test
    public void must_enter_title() throws Exception {
        getForm().submit();
        tester.assertErrorMessages("'title' is required.");
    }

    @Test
    public void saves() throws Exception {
        FormTester formTester = getForm();
        formTester.setValue(path("title"), "nice title");
        formTester.submit();
        tester.assertInfoMessages(panel.getString("saved"));
        verify(groupService).save(isA(Group.class));
    }

    private Project createProject() {
        Project project = Project.builder().title("title").projectType(ObjectMother.BACHELOR).build();
        project.setId(565L);
        return project;
    }

    private FormTester getForm() {
        return tester.newFormTester(path(panel.getId(), "form"));
    }

    private void startPanel() throws Exception {
        panel = tester.startComponentInPage(new EditGroupPanel("id", Model.of(group)));
    }
}