package se.su.dsv.scipro.group;

import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.notifications.settings.service.DeliveryConfigurationService;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.supervisor.pages.SupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.HashSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class ViewGroupProjectsPanelTest extends SciProTest {

    ViewGroupProjectsPanel panel;
    Group group;

    @Mock
    protected DeliveryConfigurationService deliveryConfigurationService;
    @Mock
    protected ReceiverConfigurationService receiverConfigurationService;
    private User supervisor;
    private User reviewer;
    private User coSupervisor;

    @Before
    public void setUp() throws Exception {
        group = new Group();
        group.setId(1L);
        supervisor = createUser(5L, "John", "Smith", "john@example.com");
        reviewer = createUser(7L, "Reviewer", "Hej", "reviewer@example.com");
        coSupervisor = createUser(6L, "Co", "Supervisor", "co@example.com");
        group.setUser(supervisor);
        group.setProjects(new HashSet<>(Arrays.asList(project())));
        when(groupService.save(isA(Group.class))).thenReturn(group);
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void link_to_supervisor_details_when_clicking_title() throws Exception {
        startPanel();
        tester.assertBookmarkablePageLink(path(panel, "projects", 0, "titleLink"), SupervisorProjectDetailsPage.class, getPageParameters());
    }

    @Test
    public void link_enabled_when_logged_in_as_supervisor() throws Exception {
        setLoggedInAs(supervisor);
        startPanel();
        tester.assertEnabled(path(panel, "projects", 0, "titleLink"));
    }

    @Test
    public void link_enabled_when_logged_in_as_co_supervisor() throws Exception {
        setLoggedInAs(coSupervisor);
        startPanel();
        tester.assertEnabled(path(panel, "projects", 0, "titleLink"));
    }

    @Test
    public void link_enabled_when_logged_in_as_reviewer() throws Exception {
        setLoggedInAs(reviewer);
        startPanel();
        tester.assertEnabled(path(panel, "projects", 0, "titleLink"));
    }

    @Test
    public void link_disabled_when_logged_in_as_other() throws Exception {
        startPanel();
        tester.assertDisabled(path(panel, "projects", 0, "titleLink"));
    }

    private User createUser(long id, String firstName, String lastName, String emailAddress) {
        User user = User.builder().firstName(firstName).lastName(lastName).emailAddress(emailAddress).build();
        user.setId(id);
        return user;
    }

    private PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 55L);
        return pageParameters;
    }

    private Project project() {
        Project project = Project.builder().title("title").projectType(ObjectMother.BACHELOR).build();
        project.setId(55L);
        project.setHeadSupervisor(supervisor);
        project.addReviewer(reviewer);
        project.addCoSupervisor(coSupervisor);
        return project;
    }

    private void startPanel() throws Exception {
        panel = tester.startComponentInPage(new ViewGroupProjectsPanel("id", Model.of(group)));
    }
}