package se.su.dsv.scipro.latestevents;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.ProjectFile;
import se.su.dsv.scipro.forum.dataobjects.*;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class LatestEventsPanelTest extends SciProTest {

    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description");
    Project project;
    LatestEventsPanel panel;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().create();
        project = Project.builder().title("Some title").projectType(SOME_PROJECT_TYPE).headSupervisor(user).build();

        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testNoFileComponentVisibleWhenNoEvent() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, LatestEventsPanel.NO_EVENT), WebMarkupContainer.class);
        tester.assertVisible(path(panel, LatestEventsPanel.NO_EVENT));
    }

    @Test
    public void testNoFileComponentInvisibleWhenFile() throws Exception {
        startPanelWithLatestEvents();
        tester.assertInvisible(path(panel.getId(), LatestEventsPanel.NO_EVENT));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new LatestEventsPanel("panel", Model.of(project)));
    }

    private void startPanelWithLatestEvents() {
        addFilesEvent();
        addForumEvent();
        startPanel();
    }

    private void addFilesEvent() {
        ProjectFile pf = new ProjectFile();
        pf.setUploader(project.getHeadSupervisor());
        pf.setIdentifier("1");
        pf.setId(1L);
        List<ProjectFile> projectFiles = new ArrayList<>();
        projectFiles.add(pf);
        when(fileDescriptionService.latestUpload(isA(Project.class), eq(LatestEventsPanel.AMOUNT))).thenReturn(projectFiles);
        when(userService.findOne(isA(Long.class))).thenReturn(project.getHeadSupervisor());

    }

    private void addForumEvent() {
        ForumThread forumThread = new ForumThread();
        ProjectThread ft = new ProjectThread();
        ft.setProject(project);
        ForumPost fp = new ForumPost();
        fp.setForumThread(forumThread);
        fp.setPostedBy(project.getMembers().get(0).getUser());
        fp.setDateCreated(new Date());
        fp.setId(1L);
        List<ForumPost> forumPosts = new ArrayList<>();
        forumPosts.add(fp);
        when(projectForumService.latestPost(isA(Project.class), eq(LatestEventsPanel.AMOUNT))).thenReturn(forumPosts);
    }

}
