package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.util.tester.FormTester;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.User;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class ForgotPasswordPageTest extends SciProTest {

    @Test
    public void renders() throws Exception {
        getPage();
        tester.assertRenderedPage(ForgotPasswordPage.class);
    }

    @Test
    public void requires_input() throws Exception {
        getPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.submit();
        tester.assertErrorMessages("Field 'email' is required");
    }

    @Test
    public void validates_input() throws Exception {
        getPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("email", "value");
        formTester.submit();
        tester.assertErrorMessages("The value of 'email' is not a valid email address.");
    }

    @Test
    public void finds_user_and_creates_hash() throws Exception {
        User user = User.builder().firstName("John").lastName("Rambo").emailAddress("john.rambo@gov.us").build();
        when(userService.findByEmailAddress(isA(String.class))).thenReturn(user);
        when(hashService.createForUser(isA(User.class))).thenReturn(Hash.builder().hash("anicehash").user(user).build());
        getPage();
        FormTester formTester = tester.newFormTester("form");
        formTester.setValue("email", "john.rambo@gov.us");
        formTester.submit();
        verify(hashService).createForUser(isA(User.class));
    }

    private ForgotPasswordPage getPage(){
        return tester.startPage(ForgotPasswordPage.class);
    }

}