package se.su.dsv.scipro.loginlogout.pages;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class LoginPageTest extends SciProTest {

    @Before
    public void setUp() throws Exception {
        when(userService.count()).thenReturn(1L);
    }

    @Test
    public void renders() throws Exception {
        tester.startPage(LoginPage.class);
    }

    @Test
    public void stateless() throws Exception {
        when(userService.count()).thenReturn(1L);
        LoginPage page = tester.startPage(LoginPage.class);
        assertTrue(page.isPageStateless());
    }

    @Test
    public void redirects_to_local_login_if_sso_is_turned_off() throws Exception {
        mockSSOSettings(false);
        tester.startPage(LoginPage.class);
        tester.assertRenderedPage(LocalLogInPage.class);
    }

    @Test
    public void stays_on_identity_selection_if_sso_is_turned_on() throws Exception {
        mockSSOSettings(true);
        tester.startPage(LoginPage.class);
        tester.assertRenderedPage(LoginPage.class);
    }


    private void mockSSOSettings(boolean showSingleSignOn) {
        GeneralSystemSettings settings = new GeneralSystemSettings();
        settings.setShowSingleSignOn(showSingleSignOn);
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(settings);
    }
}
