package se.su.dsv.scipro.match;

import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.markup.html.form.FormComponent;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class AbstractAdminIdeaPanelTest extends SciProTest {

    private AbstractAdminIdeaPanel panel;

    @Before
    public void setUp() throws Exception {
        GeneralSystemSettings settings = Mockito.mock(GeneralSystemSettings.class);
        Mockito.when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(settings);
        Mockito.when(applicationPeriodService.findAll()).thenReturn(new ArrayList<ApplicationPeriod>());
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void application_period_filter() throws Exception {
        ApplicationPeriod wanted = new ApplicationPeriod();
        DomainObjects.injectId(wanted, 42L);

        Mockito.when(applicationPeriodService.findAll()).thenReturn(Collections.singletonList(wanted));

        startPanel();
        filterApplicationPeriod(wanted);

        ArgumentCaptor <IdeaService.Filter> captor = ArgumentCaptor.forClass(IdeaService.Filter.class);
        Mockito.verify(ideaService, Mockito.atLeastOnce()).count(captor.capture());
        Assert.assertEquals(wanted, captor.getValue().getApplicationPeriod());
    }

    @Test
    public void no_first_meeting_filter() throws Exception {
        startPanel();
        tester.executeAjaxEvent(path(panel, "noFirstMeeting"), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel, "dataTable"));
    }

    private void filterApplicationPeriod(ApplicationPeriod wanted) {
        FormComponent<?> applicationPeriodFilter = (FormComponent<?>) tester.getComponentFromLastRenderedPage(path(panel, "applicationPeriodFilter"));
        applicationPeriodFilter.setDefaultModelObject(wanted);
        tester.getRequest().setParameter(applicationPeriodFilter.getInputName(), String.valueOf(wanted.getId()));
        tester.executeAjaxEvent(path(panel, "applicationPeriodFilter"), "change");
    }

    private void startPanel() {
        panel = new AbstractAdminIdeaPanel("panel") {
            @Override
            protected List<IColumn<Idea, String>> addPanelSpecificColumns() {
                return new ArrayList<>();
            }
        };
        tester.startComponentInPage(panel);
    }
}
