package se.su.dsv.scipro.match;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.util.Pair;

import java.util.Collections;
import java.util.Date;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AcceptIdeaDialogPanelTest extends SciProTest{

    private Idea idea;
    private ApplicationPeriod applicationPeriod;

    @Before
    public void setUp() throws Exception {
        when(userService.findOne(anyLong())).thenReturn(ObjectMother.SOME_USER);
        idea = getIdea();
        when(ideaService.findOne(isA(Long.class))).thenReturn(idea);
        applicationPeriod = new ApplicationPeriod();
    }

    @Test
    public void renders_as_confirmed() throws Exception {
        assert startPanel() != null;
    }

    @Test
    public void select_idea() throws Exception {
        validateStudentAcceptance(true, "test");
        startPanel();
        submitForm();
        verify(ideaService).acceptIdea(eq(idea), eq(SciProSession.get().getUser()), any(Program.class), (User) isNull(), eq(applicationPeriod));
    }

    @Test
    public void select_idea_fails() throws Exception {
        String errorMessage = "this didn't work!";
        validateStudentAcceptance(false, errorMessage);
        startPanel();
        submitForm();
        tester.assertErrorMessages(errorMessage);
    }

    private void submitForm() {
        FormTester formTester = tester.newFormTester(path("panel", "form"));
        formTester.setValue("contract", true);
        tester.executeAjaxEvent(path("panel", "form", "selectLink"), "onclick");
    }

    private void validateStudentAcceptance(boolean b, String reason) {
        when(ideaService.validateStudentAcceptance(idea,
                SciProSession.get().getUser(), null, applicationPeriod)).thenReturn(
                new Pair<>(b, reason));
    }

    private AcceptIdeaDialogPanel startPanel() {
        return tester.startComponentInPage(new AcceptIdeaDialogPanel("panel",
                Model.of(idea), applicationPeriod));
    }

    private Idea getIdea() {
        Language english = new Language();
        english.setName("English");
        idea = new Idea();
        idea.setId(1L);
        Match match = new Match();
        match.setStatus(Idea.Status.MATCHED);
        idea.setMatch(match);
        idea.setType(Idea.Type.SUPERVISOR);
        idea.setProjectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        idea.setLanguages(Collections.singleton(english));
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setCourseStartDate(new Date());
        idea.setApplicationPeriod(applicationPeriod);
        return idea;
    }
}
