package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;

public class IdeaExportPanelTest extends SciProTest {

    IdeaExportPanel panel;

    private void start_panel() {
        start_panel(create_default_idea());
    }

    private Idea create_default_idea() {
        Idea idea = new Idea();
        IdeaExport export = new IdeaExport();
        idea.setExport(export);
        export.setResult(IdeaExport.Result.FAIL);
        return idea;
    }

    private void start_panel(final Idea idea) {
        panel = tester.startComponentInPage(new IdeaExportPanel("panel", Model.of(idea)));
    }

    @Test
    public void test_renders() throws Exception {
        start_panel();
        assert panel != null : "Could not render panel";
    }

    @Test
    public void test_hides_if_not_exported() {
        // given
        Idea idea = new Idea();
        idea.setExport(null);

        // when
        start_panel(idea);

        // then
        Assert.assertFalse("Panel is visible when idea is not exported", panel.isVisibleInHierarchy());
    }

    @Test
    public void test_hides_if_export_succeeded() throws Exception {
        // given
        Idea idea = new Idea();
        IdeaExport export = new IdeaExport();

        export.setResult(IdeaExport.Result.SUCCESS);
        idea.setExport(export);

        // when
        start_panel(idea);

        // then
        Assert.assertFalse("Panel is visible when idea was successfully exported", panel.isVisibleInHierarchy());
    }

    @Test
    public void test_shown_if_export_failed() throws Exception {
        // given
        Idea idea = new Idea();
        IdeaExport export = new IdeaExport();

        export.setResult(IdeaExport.Result.FAIL);
        idea.setExport(export);

        // when
        start_panel(idea);

        // then
        Assert.assertTrue("Panel is hidden when idea exporting failed", panel.isVisibleInHierarchy());
    }

    @Test
    public void test_includes_reason() throws Exception {
        // given
        String reason = "Student not registered on course";

        Idea idea = new Idea();
        IdeaExport export = new IdeaExport();

        export.setResult(IdeaExport.Result.FAIL);
        export.setReason(reason);

        idea.setExport(export);

        // when
        start_panel(idea);

        // then
        tester.assertComponent(path(panel, "reason"), Label.class);
        String renderedReason = tester.getComponentFromLastRenderedPage(path(panel, "reason")).getDefaultModelObjectAsString();
        Assert.assertTrue("Rendered reason does not contain export failure reason", renderedReason.contains(reason));
    }

    @Test
    public void test_reason_renders_if_export_reason_null() throws Exception {
        // given
        Idea idea = new Idea();
        IdeaExport export = new IdeaExport();

        export.setResult(IdeaExport.Result.FAIL);
        export.setReason(null);

        idea.setExport(export);

        // when
        start_panel(idea);

        // then
        tester.assertComponent(path(panel, "reason"), Label.class);
    }

    @Test
    public void test_shows_explanation() throws Exception {
        // given

        // when
        start_panel();

        // then
        tester.assertComponent(path(panel, "explanation"), Label.class);
    }
}
