package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class MilestoneStatusPanelTest extends SciProTest {

    MileStoneStatusPanel panel;
    Project project;

    @Before
    public void setUp() throws Exception {
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .build();
        DomainObjects.injectId(project, 1L);
    }

    @Test
    public void testRenders() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(true));
        startPanel(Model.of(new MilestoneActivityTemplate()));
        assert panel != null;
    }

    @Test
    public void testCheckIconIsVisibleWithConfirmedMileStone() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(true));
        startPanel(Model.of(new MilestoneActivityTemplate()));

        boolean isVisible = panel.get(MileStoneStatusPanel.CHECK_ICON).isVisibleInHierarchy();
        assertTrue(isVisible);
    }

    @Test
    public void testCheckIconIsInvisibleWithUnconfirmedMileStone() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(false));
        startPanel(Model.of(new MilestoneActivityTemplate()));

        boolean isVisible = panel.get(MileStoneStatusPanel.CHECK_ICON).isVisibleInHierarchy();
        assertFalse(isVisible);
    }

    @Test
    public void testCheckIconIsVisibleForStudentMilestones() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(User.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(true));
        startPanel(Model.of(new MilestoneActivityTemplate()), Model.of(new User()));

        boolean isVisible = panel.get(MileStoneStatusPanel.CHECK_ICON).isVisibleInHierarchy();
        assertTrue(isVisible);
    }

    @Test
    public void testCheckIconIsInvisibleForStudentMilestonesNotConfirmed() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(User.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(false));
        startPanel(Model.of(new MilestoneActivityTemplate()), Model.of(new User()));

        boolean isVisible = panel.get(MileStoneStatusPanel.CHECK_ICON).isVisibleInHierarchy();
        assertFalse(isVisible);
    }

    @Test
    public void testNotCheckedContainerIsVisibleIfMileStoneIsCorrectTypeAndUnconfirmed() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(false));
        startPanel(Model.of(new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT)));

        boolean isVisible = panel.get(MileStoneStatusPanel.NOT_CHECKED).isVisibleInHierarchy();
        assertTrue(isVisible);
    }

    @Test
    public void testNotCheckedContainerIsNotVisibleIfMilestoneIsConfirmed() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(true));
        startPanel(Model.of(new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT)));

        boolean isVisible = panel.get(MileStoneStatusPanel.NOT_CHECKED).isVisibleInHierarchy();
        assertFalse(isVisible);
    }

    @Test
    public void testNotCheckedContainerIsNotVisibleIfMilestoneIsWrongType() throws Exception {
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(getMileStone(false));
        startPanel(Model.of(new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.STUDENT)));

        boolean isVisible = panel.get(MileStoneStatusPanel.NOT_CHECKED).isVisibleInHierarchy();
        assertFalse(isVisible);
    }

    private Milestone getMileStone(boolean isConfirmed) {
        Milestone milestone = new Milestone(project, new MilestoneActivityTemplate());
        milestone.setConfirmed(isConfirmed);
        return milestone;
    }

    private void startPanel(final IModel<MilestoneActivityTemplate> activityModel) {
        panel = tester.startComponentInPage(new MileStoneStatusPanel("panel", Model.of(project), activityModel, MilestoneActivityTemplate.Type.PROJECT));
    }

    private void startPanel(final IModel<MilestoneActivityTemplate> activityModel, final IModel<User> studentModel) {
        panel = tester.startComponentInPage(new MileStoneStatusPanel("panel", Model.of(project), activityModel, studentModel, MilestoneActivityTemplate.Type.STUDENT));
    }
}
