package se.su.dsv.scipro.peer;

import org.apache.wicket.model.Model;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.ChecklistAnswerEnum;

public class DisplayQuestionPanelTest extends SciProTest {

    private int index = 1;

    @Test
    public void renders_with_green_answer() throws Exception {
        startPanel(ChecklistAnswerEnum.GREEN, "motivation");
    }

    @Test
    public void renders_with_red_answer() throws Exception {
        startPanel(ChecklistAnswerEnum.RED, "motivation");
    }

    @Test
    public void renders_with_yellow_answer() throws Exception {
        startPanel(ChecklistAnswerEnum.YELLOW, "motivation");
    }

    @Test
    public void renders_with_not_applicable_answer() throws Exception {
        startPanel(ChecklistAnswerEnum.NOT_APLICABLE, "motivation");
    }

    @Test(expected = IllegalStateException.class)
    public void throws_exception_with_unknown_answer() throws Exception {
        startPanel(ChecklistAnswerEnum.NO_ANSWER, "motivation");
    }

    @Test
    public void question_number_is_index_plus_one() throws Exception {
        startPanel(ChecklistAnswerEnum.GREEN, "motivation");
        tester.assertLabel(path("id", "questionNumber"), String.valueOf(index + 1));
    }

    @Test
    public void no_motivation_label_invisible_if_motivation_exists() throws Exception {
        startPanel(ChecklistAnswerEnum.GREEN, "motivation");
        tester.assertInvisible(path("id", "noMotivation"));
    }

    @Test
    public void no_motivation_label_visible_if_motivation_is_missing() throws Exception {
        startPanel(ChecklistAnswerEnum.GREEN, null);
        tester.assertVisible(path("id", "noMotivation"));
    }

    private DisplayQuestionPanel startPanel(ChecklistAnswerEnum answer, String motivation) {
        return tester.startComponentInPage(new DisplayQuestionPanel("id", Model.of(getAnswer(answer, motivation)), index));
    }

    private Answer getAnswer(ChecklistAnswerEnum cqa, String motivation) {
        Answer answer = new Answer();
        answer.setAnswer(cqa);
        answer.setMotivation(motivation);
        return answer;
    }

}
