package se.su.dsv.scipro.peer;

import com.google.common.eventbus.EventBus;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Arrays;
import java.util.Date;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class ExpiredRequestWorkerTest {

    @Mock
    PeerRequestService peerRequestService;
    @Mock
    EventBus eventBus;
    @Mock
    GeneralSystemSettingsService settingsService;

    ExpiredRequestWorker worker = new ExpiredRequestWorker() {
        @Override
        protected void beginTransaction() {
            // do nothing
        }

        @Override
        protected void commitTransaction() {
            // do nothing
        }

        @Override
        protected void rollbackTransaction() {
            // do nothing
        }
    };

    @Before
    public void setUp() throws Exception {
        worker.eventBus = eventBus;
        worker.peerRequestService = peerRequestService;
    }

    @Test
    public void testDoWorkSendsExpiredNotifications() throws Exception {
        // given
        int expirationDays = 7;
        Long id = 17L;

        PeerRequest expiredRequest = DomainObjects.injectId(new PeerRequest(), id);
        PeerRequest currentRequest = new PeerRequest();

        ProjectTypeSettings settings = new ProjectTypeSettings();
        settings.setNumDaysBeforePeerGetsCancelled(expirationDays);

        ProjectType pClass = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        pClass.setProjectTypeSettings(settings);

        Project project = new Project();
        project.setProjectType(pClass);

        expiredRequest.setProject(project);
        expiredRequest.setDateCreated(new DateTime().minusDays(expirationDays + 1).toDate());
        currentRequest.setProject(project);
        currentRequest.setDateCreated(new Date());

        doReturn(Arrays.asList(currentRequest, expiredRequest)).when(peerRequestService).findByStatus(RequestStatus.WAITING);

        // when
        worker.doWork();

        // then
        ArgumentCaptor<PeerRequestExpiredEvent> captor = ArgumentCaptor.forClass(PeerRequestExpiredEvent.class);
        verify(eventBus, times(1)).post(captor.capture());

        PeerRequestExpiredEvent capture = captor.getValue();
        assertEquals(capture.getPeerRequest(), expiredRequest);
    }
}
