package se.su.dsv.scipro.peer;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import scala.Option;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class ProjectPeerStatsPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);

        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
        projectParticipants.addAll(Collections.singletonList(user));
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.findOne(anyLong())).thenReturn(project);
        when(projectService.isPartOfProject(project, user)).thenReturn(true);
        when(performReviewService.getInProgressReview(any(User.class), eq(project))).thenReturn(Option.<PeerReview>empty());
    }

    @Override
    protected Class<? extends Page> getPage() {
        return PeerPortalPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), "1");
        return pageParameters;
    }
}
