package se.su.dsv.scipro.profile;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.system.User;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.profile.UserLinkPanel.*;

@RunWith(MockitoJUnitRunner.class)
public class UserLinkPanelTest extends SciProTest {

    UserLinkPanel panel;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        when(userService.findOne(user.getId())).thenReturn(user);
        when(userProfileService.findByUser(any(User.class))).thenReturn(new UserProfile());
        panel = tester.startComponentInPage(new UserLinkPanel("panel", Model.of(user)));
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testLink() throws Exception {
        tester.assertComponent(path(DIALOGLINK), AjaxLink.class);
    }

    @Test
    public void testName() throws Exception {
        tester.assertComponent(path(DIALOGLINK, NAME), UserLabel.class);
        tester.assertModelValue(path(DIALOGLINK, NAME), user);
    }

    @Test
    public void testDialog() throws Exception {
        tester.assertComponent(path(DIALOG), ModalWindowPlus.class);
    }

    @Test
    public void testContent() throws Exception {
        tester.clickLink(path(DIALOGLINK));
        tester.assertComponent(path(DIALOG, CONTENT), DisplayUserProfilePanel.class);
    }

    @Test
    public void testMail() throws Exception {
        tester.assertContains("[" + path(UserLinkPanel.MAILLINK) + "]");
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel.getId(), super.path(parts));
    }
}
