package se.su.dsv.scipro.project.pages;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.grading.Grade;
import se.su.dsv.scipro.group.AuthorGroupPage;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.project.ExternalOrganization;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class ProjectDetailsPageTest extends PageTest {

	private static final Long PROJECT_PP = 1L;

	private static final User SOME_USER = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
	private static final User SOME_SUPERVISOR = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
	private Project someBachelorProject;
	private static final Project SOME_MASTER_PROJECT = Project.builder()
			.title("master")
			.projectType(new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master"))
			.headSupervisor(SOME_SUPERVISOR)
			.build();
	private Project ICT4D_PROJECT = Project.builder().title("ICT4D")
			.projectType(new ProjectType(DegreeLevel.NONE, "ICT4D", "ICT4D"))
			.headSupervisor(SOME_SUPERVISOR)
			.build();

	@Before
	public void setUp() throws Exception {
		setLoggedInAs(SOME_USER);
		someBachelorProject = Project.builder()
				.title("some title")
				.projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
				.headSupervisor(SOME_SUPERVISOR)
				.build();
		DomainObjects.injectId(someBachelorProject, 1L);
		DomainObjects.injectId(ICT4D_PROJECT, 2L);
		DomainObjects.injectId(SOME_MASTER_PROJECT, 3L);
		mockProject(someBachelorProject);
        when(gradeService.getGrade(any(Project.class))).thenReturn(Option.<Grade>empty());
	}

	@Test
	public void shows_final_seminar_panel_for_bachelor_project() throws Exception {
		startPage();
		tester.assertVisible(ProjectDetailsPage.FINAL_SEMINAR_NEW);
	}

	@Test
	public void shows_final_seminar_panel_for_master_project() throws Exception {
		mockProject(SOME_MASTER_PROJECT);
		startPage();
		tester.assertVisible(ProjectDetailsPage.FINAL_SEMINAR_NEW);
	}

    @Test
    public void hides_final_seminar_panel_for_ICT4D_project() throws Exception {
        setModules(ProjectModule.CHECKLIST);
        mockProject(ICT4D_PROJECT);
        startPage();
        tester.assertInvisible(ProjectDetailsPage.FINAL_SEMINAR_NEW);
    }

	@Test
	public void shows_external_organization() throws Exception {
		ExternalOrganization externalOrganization = new ExternalOrganization();
		externalOrganization.setName("Some external organization");
		someBachelorProject.setExternalOrganization(externalOrganization);
		mockProject(someBachelorProject);
		startPage();
		tester.assertModelValue(ProjectDetailsPage.EXTERNAL_ORGANIZATION, someBachelorProject.getExternalOrganization().getName());
	}

	@Test
	public void hides_external_organization_if_project_has_none() throws Exception {
		mockProject(someBachelorProject);
		startPage();
		tester.assertInvisible(ProjectDetailsPage.EXTERNAL_ORGANIZATION);
	}

    @Test
    public void group_overview_link_to_author_group_page() throws Exception {
        long id = 55L;
        mockGroup(id);
        mockProject(someBachelorProject);
        startPage();
        PageParameters parameters = new PageParameters();
        parameters.add(PageParameterKeys.MAP.get(Group.class), id);
        tester.assertBookmarkablePageLink(path("groups", "groupList", 0, "groupLink"),
                AuthorGroupPage.class, parameters);
    }

    private void mockGroup(long id) {
        Group group = new Group();
        group.setId(id);
        when(groupService.findAll(any(GroupService.Filter.class))).thenReturn(
				Collections.singletonList(group)
        );
    }

    @Override
	protected Class<? extends Page> getPage() {
		return ProjectDetailsPage.class;
	}

	@Override
	protected PageParameters getPageParameters() {
		PageParameters pageParameters = new PageParameters();
		pageParameters.set(PageParameterKeys.MAP.get(Project.class), String.valueOf(PROJECT_PP));
		return pageParameters;
	}

	private void mockProject(Project project) {
		when(projectService.findOne(PROJECT_PP)).thenReturn(project);
		when(projectService.isPartOfProject(project, SOME_USER)).thenReturn(true);
	}
}
