package se.su.dsv.scipro.project.pages;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

public class ProjectStartPageTest extends SciProTest {
    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);

        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.getProjectsByUser(user)).thenReturn(Collections.singletonList(project));
        when(projectService.findOne(anyLong())).thenReturn(project);
        when(projectService.isPartOfProject(project, user)).thenReturn(true);
        when(projectService.findAll(any(ProjectService.Filter.class))).thenReturn(Arrays.asList(project, project));
    }

    @Test
    public void renders() throws Exception {
        tester.startPage(ProjectStartPage.class);
        tester.assertRenderedPage(ProjectStartPage.class);
    }
}
