package se.su.dsv.scipro.project.panels;

import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ExternalResource;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class ExternalResourcesPanelTest extends SciProTest {
    private ProjectType bachelor;

    @Before
    public void setUp() throws Exception {
        bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        when(externalResourceService.findBy(bachelor)).thenReturn(Collections.singletonList(createResource()));
        tester.startComponentInPage(new ExternalResourcesPanel("id", Model.of(bachelor)));
    }

    @Test
    public void shows_all_external_resources_for_the_correct_project_type() throws Exception {
        tester.assertListView(path("id", "resources"), externalResourceService.findBy(bachelor));
    }

    @Test
    public void has_external_link_to_the_resource() throws Exception {
        tester.assertComponent(path("id", "resources", 0, "link"), ExternalLink.class);
    }

    @Test
    public void is_hidden_if_no_resources_exists() throws Exception {
        when(externalResourceService.findBy(any(ProjectType.class))).thenReturn(Collections.<ExternalResource>emptyList());
        tester.startComponentInPage(new ExternalResourcesPanel("id", Model.of(bachelor)));
        tester.assertInvisible("id");
    }

    private ExternalResource createResource() {
        ExternalResource externalResource = new ExternalResource();
        externalResource.setRelevantFor(bachelor);
        externalResource.setLabel("SciPro resources");
        externalResource.setUrl("https://scipro.dsv.su.se");
        return externalResource;
    }
}