package se.su.dsv.scipro.project.panels;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.test.DomainObjects;

import static org.hamcrest.core.IsNot.not;
import static org.hamcrest.core.StringContains.containsString;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.when;

public class ProjectSubTabMenuPanelTest extends SciProTest {

    private static final Project SOME_BACHELOR_PROJECT = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
    private static final Project SOME_MASTER_PROJECT = Project.builder().title("master").projectType(new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master")).build();
    private static final Project ICT4D_PROJECT = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.NONE, "ICT4D", "ICT4D")).build();
    private static final Project SOME_PHD_PROJECT = Project.builder().title("phd").projectType(new ProjectType(DegreeLevel.THIRD_CYCLE, "PHD", "PHD")).build();

    @Before
    public void setUp() throws Exception {
        DomainObjects.injectId(SOME_BACHELOR_PROJECT, 1L);
        DomainObjects.injectId(ICT4D_PROJECT, 2L);
        DomainObjects.injectId(SOME_MASTER_PROJECT, 3L);
        DomainObjects.injectId(SOME_PHD_PROJECT, 4L);
    }

    @Test
    public void renders() throws Exception {
        assertNotNull(startPanel(SOME_BACHELOR_PROJECT));
    }

    @Test
    public void final_seminar_tab_is_visible_for_bachelor_project_that_has_final_seminar() throws Exception {
        when(finalSeminarService.findByProject(SOME_BACHELOR_PROJECT)).thenReturn(new FinalSeminar());
        assertVisible(ProjectSubTabMenuPanel.FINAL_SEMINAR, SOME_BACHELOR_PROJECT);
    }

    @Test
    public void final_seminar_tab_is_visible_for_master_project_that_has_final_seminar() throws Exception {
        when(finalSeminarService.findByProject(SOME_MASTER_PROJECT)).thenReturn(new FinalSeminar());
        assertVisible(ProjectSubTabMenuPanel.FINAL_SEMINAR, SOME_MASTER_PROJECT);
    }

    @Test
    public void final_seminar_tab_is_not_visible_for_bachelor_project_that_has_no_final_seminar() throws Exception {
        when(finalSeminarService.findByProject(SOME_BACHELOR_PROJECT)).thenReturn(null);
        assertNotVisible(ProjectSubTabMenuPanel.FINAL_SEMINAR, SOME_BACHELOR_PROJECT);
    }

    @Test
    public void final_seminar_tab_is_not_visible_without_seminar_module() throws Exception {
        setModules(ProjectModule.CHECKLIST);
        assertNotVisible(ProjectSubTabMenuPanel.FINAL_SEMINAR, ICT4D_PROJECT);
    }

    private String renderedPanel(Project project) {
        startPanel(project);
        return tester.getLastResponseAsString();
    }

    private void assertVisible(String menuChoice, Project project) {
        assertThat(renderedPanel(project), containsString(menuChoice));
    }

    private void assertNotVisible(String menuChoice, Project project) {
        assertThat(renderedPanel(project), not(containsString(menuChoice)));
    }

    private ProjectSubTabMenuPanel startPanel(Project project) {
        return tester.startComponentInPage(new ProjectSubTabMenuPanel("id", AbstractProjectDetailsPage.class, new PageParameters(), project));
    }

}
