package se.su.dsv.scipro.repository.panels;

import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.repository.components.FileDownloadLink;

import static org.junit.Assert.assertNotNull;

public class ViewAttachmentPanelTest extends SciProTest {

    ViewAttachmentPanel panel;
    private FileDescription fileDescription;

    @Before
    public void setUp() throws Exception {
        fileDescription = new FileDescription();
        fileDescription.setIdentifier("1");
        fileDescription.setId(1L);
    }

    @Test
    public void testRenders() throws Exception {
        startPanel(Model.of(fileDescription));
        assertNotNull("panel does not render", panel);
    }

    @Test
    public void testPanelHiddenIfNoFileAvailable() throws Exception {
        startPanel(new Model<FileDescription>(null));
        tester.assertInvisible(path(panel));
    }

    @Test
    public void testPanelIsVisibleIfFileIsAvailable() throws Exception {
        startPanel(Model.of(fileDescription));
        tester.assertVisible(path(panel));
    }

    @Test
    public void testPanelContainsAttachmentName() throws Exception {
        startPanel(Model.of(newFile()));
        tester.assertComponent(path(panel, ViewAttachmentPanel.NAME), FileDownloadLink.class);
    }

    @Test
    public void date_label_visible_if_date_style_provided() throws Exception {
        tester.startComponentInPage(new ViewAttachmentPanel("panel", Model.of(newFile()), DateStyle.DATE));
        tester.assertVisible(path("panel", ViewAttachmentPanel.UPLOAD_TIME));
    }

    @Test
    public void date_label_not_visible_if_no_date_style_provided() throws Exception {
        startPanel(Model.of(newFile()));
        tester.assertInvisible(path("panel", ViewAttachmentPanel.UPLOAD_TIME));
    }

    private void startPanel(final IModel<FileDescription> model) {
        panel = tester.startComponentInPage(new ViewAttachmentPanel("panel", model));
    }

    private FileDescription newFile() {
        FileDescription file = new FileDescription();
        file.setName("name");
        file.setPath("/foo");
        file.setIdentifier("1");
        file.setId(1L);
        return file;
    }
}
