package se.su.dsv.scipro.security.auth;

import org.apache.wicket.Component;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.ProjectModule;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class ProjectProjectModuleBasedAuthorizationStrategyTest extends SciProTest {

    @Mock
    private ProjectModuleComponent authorization;
    @Mock
    private Component component;

    private ProjectModuleBasedAuthorizationStrategy authorizationStrategy;

    @Before
    public void setUp() throws Exception {
        authorizationStrategy = new ProjectModuleBasedAuthorizationStrategy() {
            @Override
            protected ProjectModuleComponent internalGetAnnotation(Class<? extends Component> authRequired) {
                return authorization;
            }
        };
    }

    @Test
    public void instantiation_always_authorized() throws Exception {
        assertTrue(authorizationStrategy.isInstantiationAuthorized(component.getClass()));
    }

    @Test
    public void authorized_if_module_is_undefined() throws Exception {
        authorization = null;
        assertTrue(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

    @Test
    public void authorized_if_module_attached_to_session() throws Exception {
        setModules(ProjectModule.PEER);
        when(authorization.value()).thenReturn(ProjectModule.PEER);
        assertTrue(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

    @Test
    public void not_authorized_if_module_not_present() throws Exception {
        setModules(ProjectModule.CHECKLIST);
        when(authorization.value()).thenReturn(ProjectModule.PEER);
        assertFalse(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }
}
