package se.su.dsv.scipro.statistics;

import com.google.common.base.Joiner;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static se.su.dsv.scipro.statistics.AdminMultipleAuthorsStatisticsPage.AUTHOR_LIST;
import static se.su.dsv.scipro.statistics.AdminMultipleAuthorsStatisticsPage.PROJECTS;
import static se.su.dsv.scipro.test.ObjectMother.SOME_USER;

public class AdminMultipleAuthorsStatisticsPageTest extends SciProTest {

    private ProjectService.Filter filter;

    @Before
    public void setUp() throws Exception {
        filter = new ProjectService.Filter();
        Mockito.when(getMultipleAuthors()).thenReturn(Collections.singletonList(SOME_USER));
        Mockito.when(projectService.findAll(Matchers.any(ProjectService.Filter.class))).thenReturn(Collections.singletonList(ObjectMother.SOME_PROJECT));
        tester.startPage(new AdminMultipleAuthorsStatisticsPage(filter));
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminMultipleAuthorsStatisticsPage.class);
    }

    @Test
    public void lists_all_authors_with_more_than_one_projects() throws Exception {
        tester.assertListView(AUTHOR_LIST, getMultipleAuthors());
    }

    @Test
    public void authors_name() throws Exception {
        tester.assertLabel(path(AUTHOR_LIST, 0, AdminMultipleAuthorsStatisticsPage.AUTHOR_NAME), SOME_USER.getFullName());
    }

    @Test
    public void lists_all_projects_for_specific_authors() throws Exception {
        tester.assertLabel(path(AUTHOR_LIST, 0, PROJECTS), getAuthorProjects());
    }

    private String getAuthorProjects() {
        List<String> result = new ArrayList<>();
        for (Project project : projectService.findAll(Matchers.any(ProjectService.Filter.class))) {
            result.add(project.getTitle());
        }
        return Joiner.on(", ").join(result);
    }

    private List<User> getMultipleAuthors() {
        return projectService.findMultipleAuthors(filter);
    }
}
