package se.su.dsv.scipro.statistics;

import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Collections;
import java.util.Date;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class AdminUnfinishedFinalSeminarsPageTest extends SciProTest {

    public static final String EXPORT_LINK_PATH = "table:bottomToolbars:toolbars:4:td:linkContainer:1:exportLink";

    @Before
    public void setUp() throws Exception {
        when(finalSeminarService.findUnfinishedSeminars(any(Date.class),any(Date.class), any(Pageable.class))).thenReturn(Collections.singletonList(getFinalSeminar()));
        when(finalSeminarService.countUnfinishedSeminars(any(Date.class), any(Date.class))).thenReturn(1L);
    }

    @Test
    public void renders() throws Exception {
        tester.startPage(new AdminUnfinishedFinalSeminarsPage());
        tester.assertRenderedPage(AdminUnfinishedFinalSeminarsPage.class);
    }

    @Test
    public void excel_exports_render() throws Exception {
        tester.startPage(new AdminUnfinishedFinalSeminarsPage());
        tester.clickLink(EXPORT_LINK_PATH);
        assertEquals(tester.getContentTypeFromResponseHeader(), "application/vnd.ms-excel");
    }

    private FinalSeminar getFinalSeminar() {
        FinalSeminar seminar = new FinalSeminar();
        seminar.setProject(ObjectMother.SOME_PROJECT);
        FinalSeminarOpposition opposition = new FinalSeminarOpposition();
        opposition.setProject(Project.builder().title("hej").projectType(ObjectMother.BACHELOR).build());
        opposition.setUser(ObjectMother.SOME_USER);
        seminar.addOpposition(opposition);
        return seminar;
    }
}
