package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Date;

import static org.mockito.Mockito.*;

public class FinalSeminarApprovalPanelTest extends SciProTest {

    private FinalSeminarApprovalPanel panel;
    private Project project;

    @Before
    public void setUp() throws Exception {
        ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        project = Project.builder().title("My project").projectType(bachelor).build();
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void has_form_to_upload_initial_thesis_for_approval() throws Exception {
        startPanel();
        tester.assertVisible(path(panel, "requestApproval"));
    }

    @Test
    public void upload_initial_thesis_for_approval() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, "requestApproval"));
        formTester.setFile("thesis", thesisFile(), "application/pdf");
        formTester.submit();

        tester.assertNoErrorMessage();
        verify(finalSeminarApprovalService).requestApproval(eq(project), any(FileUpload.class), anyString());
    }

    @Test
    public void if_the_approval_process_has_started_do_not_show_initial_upload() throws Exception {
        startPanelWithOngoingProcess();
        tester.assertInvisible(path(panel, "requestApproval"));
    }

    @Test
    public void if_the_process_has_started_show_process_details() throws Exception {
        FinalSeminarApproval finalSeminarApproval = startPanelWithOngoingProcess();
        tester.assertComponent(path(panel, "process"), FinalSeminarApprovalProcessPanel.class);
        tester.assertModelValue(path(panel, "process"), finalSeminarApproval);
    }

    @Test
    public void hides_the_process_details_if_the_process_has_not_started() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel, "process"));
    }

    private void startPanel() {
        when(finalSeminarApprovalService.requiresUpload(project)).thenReturn(true);
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.<FinalSeminarApproval>empty());
        panel = tester.startComponentInPage(new FinalSeminarApprovalPanel("id", Model.of(project)));
    }

    private FinalSeminarApproval startPanelWithOngoingProcess() {
        FinalSeminarApproval finalSeminarApproval = new FinalSeminarApproval(project, mock(FileDescription.class),"test", new Date());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.apply(finalSeminarApproval));
        panel = tester.startComponentInPage(new FinalSeminarApprovalPanel("id", Model.of(project)));
        return finalSeminarApproval;
    }

    private static File thesisFile() throws Exception {
        return new File(Thread.currentThread().getContextClassLoader().getResource("thesis.pdf").toURI());
    }
}
