package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.ReviewerApproval;

import java.util.Date;

import static org.mockito.Mockito.mock;

public class FinalSeminarApprovalProcessPanelTest extends SciProTest {
    private ReviewerApproval finalSeminarApproval;
    private FinalSeminarApprovalProcessPanel panel;

    @Before
    public void setUp() throws Exception {
        finalSeminarApproval = new FinalSeminarApproval(mock(Project.class), mock(FileDescription.class),"test", new Date());
        startPanel();
    }

    @Test
    public void shows_current_file() throws Exception {
        tester.assertComponent(path(panel, "currentThesis"), ViewAttachmentPanel.class);
        tester.assertModelValue(path(panel, "currentThesis"), finalSeminarApproval.getCurrentThesis());
    }

    @Test
    public void shows_current_status() throws Exception {
        tester.assertComponent(path(panel, "currentStatus"), EnumLabel.class);
        tester.assertModelValue(path(panel, "currentStatus"), finalSeminarApproval.getCurrentStatus());
    }

    @Test
    public void shows_current_reason_if_a_decision_has_been_made() throws Exception {
        startPanelWithApprovedFinalSeminar();
        tester.assertComponent(path(panel, "currentReason"), MultiLineLabel.class);
    }

    @Test
    public void does_not_show_current_reason_if_no_decision_has_been_made() throws Exception {
        tester.assertInvisible(path(panel, "currentReason"));
    }

    @Test
    public void shows_current_attachment() throws Exception {
        startPanelWithApprovedFinalSeminar();
        tester.assertComponent(path(panel, "currentAttachment"), ViewAttachmentPanel.class);
        tester.assertModelValue(path(panel, "currentAttachment"), finalSeminarApproval.getCurrentAttachment().get());
    }

    private void startPanelWithApprovedFinalSeminar() {
        finalSeminarApproval.approve("Very good", Option.apply(mock(FileDescription.class)), Option.<FileDescription>empty());
        startPanel();
    }

    @Test
    public void shows_history() throws Exception {
        tester.assertListView(path(panel, "history"), finalSeminarApproval.getHistory());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new FinalSeminarApprovalProcessPanel("id", modelOf(finalSeminarApproval)));
    }

    private static IModel<ReviewerApproval> modelOf(final ReviewerApproval finalSeminarApproval) {
        return new LoadableDetachableModel<ReviewerApproval>() {
            @Override
            protected ReviewerApproval load() {
                return finalSeminarApproval;
            }
        };
    }
}
