package se.su.dsv.scipro.thesislink;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.thesislink.ExternalLinkPanel.*;

public class ExternalLinkPanelTest extends SciProTest {

    private ExternalLinkPanel panel;
    private Project project;

    @Before
    public void setUp() throws Exception {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);
        project = Project.builder().title("My project").projectType(projectType).build();
        User user = new UserBuilder().setFirstName("Sten").setLastName("Sture").setEmail("sten@example.com").create();
        setLoggedInAs(user);
        when(externalLinkService.findByProject(isA(Project.class))).thenReturn(Arrays.asList(ExternalLink.builder().project(project).url("http://example.com").user(user).description("bla bla").build()));
        startPanel();
    }

    @Test
    public void renders() throws Exception {
        assertNotNull("Does not render", panel);
    }

    @Test
    public void saves() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, CONTAINER, FORM));
        formTester.setValue(URL, "http://www.example.com");
        formTester.setValue(DESCRIPTION, "blabla");
        formTester.submit();
        tester.assertNoErrorMessage();
        verify(externalLinkService).save(isA(ExternalLink.class));
    }

    @Test
    public void deletes() throws Exception {
        tester.clickLink(path(panel, CONTAINER, LINKS, 0, DELETE));
        verify(externalLinkService).delete(isA(ExternalLink.class));
    }

    private void startPanel(){
        panel = tester.startComponentInPage(new ExternalLinkPanel("panel", Model.of(project)));
    }
}