package se.su.dsv.scipro.user;

import org.apache.wicket.Page;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.Pair;

import java.net.URL;
import java.util.Arrays;

import static org.mockito.Mockito.*;

public class AdminImportUserPageTest extends PageTest {

    @Override
    protected Class<? extends Page> getPage() {
        return AdminImportUserPage.class;
    }

    @Test
    public void validFile() throws Exception {
        when(userFacade.attemptCreateUser(isA(String.class), isA(String.class), isA(String.class))).thenReturn(new Pair<>(new User(), "anicehash"));
        tester.startPage(getPage());
        URL resource = AdminImportUserPageTest.class.getResource("valid_file.csv");
        File file = new File(resource.toURI());
        FormTester formTester = tester.newFormTester("form");
        formTester.setFile("upload", file, "text/csv");
        formTester.submit();
        verify(userFacade, times(3)).attemptCreateUser(isA(String.class), isA(String.class), isA(String.class));
        tester.assertLabel("successNo", "3");
    }

    @Test
    public void noFile() throws Exception {
        tester.startPage(getPage());
        FormTester formTester = tester.newFormTester("form");
        formTester.submit();
        tester.assertErrorMessages("No file selected");
    }

    @Test
    public void invalidSingleRowFile() throws Exception {
        tester.startPage(getPage());
        URL resource = AdminImportUserPageTest.class.getResource("invalid_single_row_file.csv");
        File file = new File(resource.toURI());
        FormTester formTester = tester.newFormTester("form");
        formTester.setFile("upload", file, "text/csv");
        formTester.submit();
        verifyZeroInteractions(userFacade);
        tester.assertLabel("successNo", "0");
        tester.assertNoErrorMessage();
    }

    @Test
    public void invalidHeadersFile() throws Exception {
        tester.startPage(getPage());
        URL resource = AdminImportUserPageTest.class.getResource("invalid_header_file.csv");
        File file = new File(resource.toURI());
        FormTester formTester = tester.newFormTester("form");
        formTester.setFile("upload", file, "text/csv");
        formTester.submit();
        verifyZeroInteractions(userFacade);
        tester.assertLabel("successNo", "0");
        tester.assertErrorMessages("Invalid file");
    }

    @Test
    public void validHeadersInvalidRowsFile() throws Exception {
        when(userFacade.attemptCreateUser(isA(String.class), isA(String.class), isA(String.class))).thenReturn(new Pair<User, String>(null, "u so bad"));
        tester.startPage(getPage());
        URL resource = AdminImportUserPageTest.class.getResource("valid_header_invalid_row_file.csv");
        File file = new File(resource.toURI());
        FormTester formTester = tester.newFormTester("form");
        formTester.setFile("upload", file, "text/csv");
        formTester.submit();
        //verify 4 because one row will never call userFacade
        verify(userFacade, times(4)).attemptCreateUser(isA(String.class), isA(String.class), isA(String.class));
        tester.assertLabel("successNo", "0");
        tester.assertListView("failList", Arrays.asList("Row 1: invalid row [aaa, aaa]",
                "Row 2: u so bad [aaa, aaa, ]",
                "Row 3: u so bad [aaa, aaa, aaa]",
                "Row 4: u so bad [aaa, aaa, aaa, ]",
                "Row 5: u so bad [aaa, aaa, aaa, aaa]"));
        tester.assertNoErrorMessage();
    }
}